/*******************************************************************************
 * CELLmicrocosmos 2.2 MembraneEditor (version 2.2.2_2)
 *
 * Copyright (c) 2005, 2017 The CELLmicrocosmos 2 Development Team
 * (Bjorn Sommer, Tim Dingersen et al.
 *  University of Konstanz, Monash University, Bielefeld University)
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *  
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *  
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *******************************************************************************/
package org.cellmicrocosmos.cm2.coarsegrainsupport;

import java.awt.Dimension;
import java.awt.Font;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JToolBar;

/**
 * This class describes the toolbar of the MoleculeEditor
 * @author Tim Dingersen
 * dackelnose@gmx.net
 *
 */
public class MoleculeEditorToolbar extends JToolBar implements
		ActionListener {

	private static final long serialVersionUID = 1L;

	JButton saveAsBtn = new JButton("Save Only");
	JButton applyBtn = new JButton("Apply Current Profile");
	// JButton deactivateBtn = new JButton("Apply Default Model");
	// JButton saveAsBtn = new JButton("<html><b>Save</b> Profile</html>");
	// JButton saveAsAndApplyBtn = new
	// JButton("<html><b>Save</b> and <b>Apply</b> Profile</html>");
	JButton exitBtn = new JButton("Exit");
	JComboBox combo = new JComboBox();

	// JCheckBox cbConf = new JCheckBox("Apply Conformation");

	private MoleculeEditor editor;

	public MoleculeEditorToolbar(MoleculeEditor moleculeEditor) {
		this.editor = moleculeEditor;

		this.setFloatable(false);
		combo.setEditable(false);

		applyBtn.addActionListener(this);
		// deactivateBtn.addActionListener(this);
		saveAsBtn.addActionListener(this);
		this.exitBtn.addActionListener(this);

		Font font = this.getFont().deriveFont(15);
		Font font2 = this.getFont().deriveFont(12);

		saveAsBtn.setFont(font2);
		applyBtn.setFont(font.deriveFont(Font.BOLD));
		// deactivateBtn.setFont(font);
		combo.getEditor().getEditorComponent().setFont(
				font.deriveFont(Font.BOLD));
		exitBtn.setFont(font);

		addSeparator(new Dimension(5, 0));
		addSeparator();
		addSeparator(new Dimension(10, 0));
		add(applyBtn);
		addSeparator(new Dimension(10, 0));
		addSeparator();
		addSeparator(new Dimension(20, 0));

		JLabel label = new JLabel("Current Profile: ");
		label.setFont(font2);
		addSeparator(new Dimension(5, 0));
		add(label);
		add(combo);

		addSeparator(new Dimension(5, 0));
		addSeparator();
		addSeparator(new Dimension(10, 0));
		add(saveAsBtn);
		addSeparator(new Dimension(10, 0));
		addSeparator();
		// addSeparator(new Dimension(10, 0));
		// add(this.deactivateBtn);
		// addSeparator(new Dimension(10, 0));
		// addSeparator();

		// add(cbConf);
		this.add(new JPanel());
		addSeparator();
		addSeparator(new Dimension(5, 0));
		add(exitBtn);
		addSeparator(new Dimension(5, 0));
		addSeparator();
		addSeparator(new Dimension(5, 0));

		combo.setFont(combo.getFont().deriveFont(
				combo.getFont().getSize2D() + 2));
		combo.setPreferredSize(new Dimension(200, 25));
		combo.setMaximumSize(new Dimension(200, 25));
		combo.setMinimumSize(new Dimension(200, 25));
		this.setBorder(BorderFactory.createEtchedBorder());
	}

	public void setComboBoxData(String[] data, String selection) {

		combo.removeActionListener(this);
		combo.removeAllItems();

		combo.addItem("Default Profile");

		int selInd = 0;
		if (data != null) {
			for (int i = 0; i < data.length; i++) {
				combo.addItem(data[i]);
				if (data[i].equals(selection))
					selInd = i + 1;
			}
		}
		combo.setSelectedIndex(selInd);

		combo.addActionListener(this);

	}

	public void actionPerformed(ActionEvent arg0) {

		if (arg0.getSource() == this.saveAsBtn) {
			this.editor.treatCurrent(true, false);
		}
		if (arg0.getSource() == this.applyBtn) {
			this.editor.treatCurrent(true, true);
		}
		if (arg0.getSource() == this.combo) {
			this.editor.setProfile((String) combo.getSelectedItem());

		}
		if (arg0.getSource() == this.exitBtn) {
			this.editor.cancelCurrent();
		}
		

	}
}
