/*******************************************************************************
 * CELLmicrocosmos 2.2 MembraneEditor (version 2.2.2_2)
 *
 * Copyright (c) 2005, 2017 The CELLmicrocosmos 2 Development Team
 * (Bjorn Sommer, Tim Dingersen et al.
 *  University of Konstanz, Monash University, Bielefeld University)
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *  
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *  
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *******************************************************************************/
package org.cellmicrocosmos.cm2.functionality3d;

import java.awt.Color;

import javax.media.j3d.Appearance;
import javax.media.j3d.BranchGroup;
import javax.media.j3d.Group;
import javax.media.j3d.Link;
import javax.media.j3d.Material;
import javax.media.j3d.Node;
import javax.media.j3d.PolygonAttributes;
import javax.media.j3d.SharedGroup;
import javax.media.j3d.Transform3D;
import javax.media.j3d.TransformGroup;
import javax.vecmath.Color3f;
import javax.vecmath.Vector3f;

import org.cellmicrocosmos.cm2.constants.CM2_CONSTANTS;
import org.cellmicrocosmos.cm2.dataobjects.Atom_3D;
import org.cellmicrocosmos.cm2.dataobjects.Molecule_3D;
import org.cellmicrocosmos.cm2.dataobjects.Particle;
import org.cellmicrocosmos.cm2.engine.DataPool;
import org.cellmicrocosmos.cm2.misc.Cm2_Toolkit;

import com.sun.j3d.utils.geometry.Box;
import com.sun.j3d.utils.geometry.Primitive;
import com.sun.j3d.utils.geometry.Sphere;

/**
 * This class creates and attaches the visuals for Atomic representation of a
 * molecule
 * 
 * @author Tim Dingersen dackelnose@gmx.net, Philipp Unruh, Bjoern Sommer bjoern@CELLmicrocosmos.org
 * 
 */
public class Atom_Creator_3D {

	public static void setUpMolecule(final Molecule_3D mol, final int mode) {

		Atom_3D[] atoms = mol.getAtoms(false, false);

		//		if (mode > 0 && mol instanceof Protein_3D)
		//			atoms = Atom_Creator_3D.removeInnerAtoms(mol, atoms,
		//					DataPool.J3DSettings.removeInnerAtomsFactor);

		if (mode == CM2_CONSTANTS.RENDERMODE.RENDER_MODE_BALLS_N_STICKS
				|| mode == CM2_CONSTANTS.RENDERMODE.RENDER_MODE_COLORED_STICKS) {
			for (int i = 0; i < mol.getActiveConstraints().size(); i++) {
				mol.getTransformGroup()
						.addChild(mol.getActiveConstraints().get(i).getBranchGroup());
			}
		}
		if (mode == CM2_CONSTANTS.RENDERMODE.RENDER_MODE_BALLS_N_STICKS) {
			Atom_Creator_3D.addBalls(mol, atoms, false, null, 0, false);

		} else if (mode == CM2_CONSTANTS.RENDERMODE.RENDER_MODE_COLORED_STICKS) {

		} else {
			Atom_Creator_3D
					.addBalls(
							mol,
							atoms,
							true,
							null,
							mol.getRenderMode() != CM2_CONSTANTS.RENDERMODE.RENDER_MODE_VAN_DER_WAALS ? 0
									: 1, 
							false);

		}

	}

	public static void addBalls(final Molecule_3D mol, final Atom_3D[] atoms,
			final boolean fullSize, Color selectionColor, int radiusMode, boolean opmAtoms) {

		Thread.currentThread().setPriority(Thread.MIN_PRIORITY);
		
		if (DataPool.J3DSettings.enableAtomPerformanceMode) {
		
			SharedGroup sharedGroup = null;
	
			if (opmAtoms) {
				sharedGroup = DataPool.sharedOPMGroupsOfMolecules.get(mol.getPDBcode());
			} else {
				sharedGroup = DataPool.sharedGroupsOfMolecules.get(mol.getPDBcode());
			}
	
			Integer sharedGroupRenderMode = DataPool.sharedGroupsRenderModeOfMolecules.get(mol
					.getPDBcode());
	
			if (sharedGroup == null
					|| (sharedGroupRenderMode != null && sharedGroupRenderMode != mol.getRenderMode())) {
				sharedGroup = new SharedGroup();
				sharedGroup.setCapability(SharedGroup.ALLOW_LINK_READ);
				sharedGroup.setCapability(SharedGroup.ALLOW_CHILDREN_READ);
				sharedGroup.setCapability(SharedGroup.ALLOW_CHILDREN_WRITE);
				sharedGroup.setCapability(SharedGroup.ALLOW_CHILDREN_EXTEND);
				sharedGroup.setCapability(SharedGroup.ALLOW_LOCAL_TO_VWORLD_READ);
				sharedGroup.setCapability(SharedGroup.ALLOW_PICKABLE_READ);
				sharedGroup.setCapability(SharedGroup.ALLOW_PICKABLE_WRITE);
				sharedGroup.setCapability(SharedGroup.ALLOW_PARENT_READ);
	
				for (int i = 0; i < atoms.length; i++) {
	
					final Appearance app = new Appearance();
					final Material mat = new Material();
	
					if (mol.isUsingAtomColors()) {
						mat.setAmbientColor(new Color3f(atoms[i].color));
						mat.setDiffuseColor(new Color3f(atoms[i].color));
						mat.setSpecularColor(new Color3f(atoms[i].color));
					} else {
						mat.setAmbientColor(new Color3f(mol.getColor()));
						mat.setDiffuseColor(new Color3f(mol.getColor()));
						mat.setSpecularColor(new Color3f(mol.getColor()));
					}
	
					mat.setShininess(25);
					mat.setCapability(Material.ALLOW_COMPONENT_WRITE);
					app.setMaterial(mat);
	
					Cm2_Toolkit.setRenderingAttributes(app);
	
					float desiredUsualRadius = (radiusMode == 0 ? atoms[i].radiusCov
							: atoms[i].radiusVDW);
	
					float rad = fullSize ? desiredUsualRadius : desiredUsualRadius / 1.4f;
	
					if (selectionColor != null) {
						//				 if(atoms[i]instanceof Particle)
						//				 rad+=0.2f;
						//				 else
						rad += 0.05f;
					}
	
					final Primitive object_3D;
	
					//			if(!(atoms[i]instanceof Particle))
	
					//			else 
					//				object_3D = new Box(rad
					//					* CM2_CONSTANTS.J3D.J3D_ANGSTROM_SCALE,rad
					//					* CM2_CONSTANTS.J3D.J3D_ANGSTROM_SCALE,rad
					//					* CM2_CONSTANTS.J3D.J3D_ANGSTROM_SCALE,app);
	
					if (selectionColor != null) {
						PolygonAttributes attrs = new PolygonAttributes();
	
						if (selectionColor.equals(Color.orange)) {
							attrs.setPolygonMode(PolygonAttributes.POLYGON_FILL);
							selectionColor = Color.GREEN;
						} else {
							attrs.setPolygonMode(PolygonAttributes.POLYGON_LINE);
						}
	
						app.setPolygonAttributes(attrs);
	
						mat.setAmbientColor(new Color3f(selectionColor));
						mat.setDiffuseColor(new Color3f(selectionColor));
						mat.setSpecularColor(new Color3f(selectionColor));
	
						float newRad;
						//				if(atoms[i] instanceof Particle) {
						//				newRad=rad*1.2f;
						//				int mC=((Particle)atoms[i]).includedAtoms.size();
						//				
						//				newRad+=((float)mC)/20f;
						//				}
						//				else
						newRad = rad;
	
						newRad *= CM2_CONSTANTS.J3D.J3D_ANGSTROM_SCALE;
	
						object_3D = new Box(newRad, newRad, newRad, null);
	
						object_3D.setPickable(false);
					} else {
						object_3D = new Sphere(rad * CM2_CONSTANTS.J3D.J3D_ANGSTROM_SCALE);
						atoms[i].object_3D = object_3D;
						object_3D.setUserData(atoms[i]);
					}
	
					Atom_Creator_3D.setCapabilities(object_3D);
	
					object_3D.setAppearance(app);
	
					final TransformGroup tg = new TransformGroup();
					tg.addChild(object_3D);
	
					final Transform3D t3d = new Transform3D();
					final Vector3f v = new Vector3f(atoms[i].position);
					v.scale(CM2_CONSTANTS.J3D.J3D_ANGSTROM_SCALE);
					t3d.setTranslation(v);
					tg.setTransform(t3d);
					tg.setCapability(TransformGroup.ALLOW_TRANSFORM_WRITE);
	
					if (selectionColor == null && atoms[i] instanceof Particle
					//					&&mol.isDisplayedInMoleculeEditor
					) {
	
						float newRad = rad * 1.2f;
						//				int mC=((Particle)atoms[i]).includedAtoms.size();
	
						//				newRad+=((float)mC)/20f;
						newRad *= CM2_CONSTANTS.J3D.J3D_ANGSTROM_SCALE;
						Sphere additionalSphere = new Sphere(newRad);
	
						Atom_Creator_3D.setCapabilities(additionalSphere);
						additionalSphere.setPickable(false);
						Appearance app2 = new Appearance();
						app2.setMaterial(mat);
						Cm2_Toolkit.setRenderingAttributes(app2);
						PolygonAttributes attrs = new PolygonAttributes();
						attrs.setPolygonMode(PolygonAttributes.POLYGON_LINE);
						app2.setPolygonAttributes(attrs);
						additionalSphere.setAppearance(app2);
						tg.addChild(additionalSphere);
	
					}
	
					final BranchGroup bg = new BranchGroup();
					bg.setUserData(atoms[i]);
					bg.addChild(tg);
					bg.setCapability(BranchGroup.ALLOW_DETACH);
					bg.setCapability(BranchGroup.ALLOW_CHILDREN_EXTEND);
					bg.setCapability(BranchGroup.ALLOW_CHILDREN_WRITE);
					bg.setCapability(BranchGroup.ALLOW_CHILDREN_READ);
	
					if (atoms[i].branchGroup != null) {
						Node node = atoms[i].branchGroup.getParent();
						SharedGroup parent = (SharedGroup) node;
						parent.removeChild(atoms[i].branchGroup);
					}
					atoms[i].branchGroup = bg;
					sharedGroup.addChild(atoms[i].branchGroup);
					sharedGroup.setUserData(mol);
				}
	
				if (opmAtoms) {
					DataPool.sharedOPMGroupsOfMolecules.put(mol.getPDBcode(), sharedGroup);
				} else {
					DataPool.sharedGroupsOfMolecules.put(mol.getPDBcode(), sharedGroup);
				}
				DataPool.sharedGroupsRenderModeOfMolecules.put(mol.getPDBcode(), mol.getRenderMode());
			}
	
			Link shapeLink = new Link(sharedGroup);
			shapeLink.setUserData(mol);
	
			//mol.getTransformGroup().addChild(bGroup);
			mol.getTransformGroup().addChild(shapeLink);
			
	//		BranchGroup bg = new BranchGroup();
	//		bg.addChild(shapeLink);
	//		mol.getTransformGroup().addChild(bg);
			
		} else {
			
			for (int i = 0; i < atoms.length; i++) {

				final Appearance app = new Appearance();
				final Material mat = new Material();

				if (mol.isUsingAtomColors()) {
					mat.setAmbientColor(new Color3f(atoms[i].color));
					mat.setDiffuseColor(new Color3f(atoms[i].color));
					mat.setSpecularColor(new Color3f(atoms[i].color));
				} else {
					mat.setAmbientColor(new Color3f(mol.getColor()));
					mat.setDiffuseColor(new Color3f(mol.getColor()));
					mat.setSpecularColor(new Color3f(mol.getColor()));
				}

				mat.setShininess(25);
				mat.setCapability(Material.ALLOW_COMPONENT_WRITE);
				app.setMaterial(mat);
				
				Cm2_Toolkit.setRenderingAttributes(app);
				
				float desiredUsualRadius=(radiusMode==0?atoms[i].radiusCov:atoms[i].radiusVDW);
				
				 float rad = fullSize ? desiredUsualRadius
						:desiredUsualRadius / 1.4f;
				
				 if(selectionColor!=null) {
//					 if(atoms[i]instanceof Particle)
//					 rad+=0.2f;
//					 else
						 rad+=0.05f;
				 }
					
				final Primitive object_3D;
				
//				if(!(atoms[i]instanceof Particle))
				
//				else 
//					object_3D = new Box(rad
//						* CM2_CONSTANTS.J3D.J3D_ANGSTROM_SCALE,rad
//						* CM2_CONSTANTS.J3D.J3D_ANGSTROM_SCALE,rad
//						* CM2_CONSTANTS.J3D.J3D_ANGSTROM_SCALE,app);
				
				if(selectionColor!=null) {
					PolygonAttributes attrs= new PolygonAttributes();
					
					if(selectionColor.equals(Color.orange)) {
						attrs.setPolygonMode(PolygonAttributes.POLYGON_FILL);
						selectionColor=Color.GREEN;
					}else {
						attrs.setPolygonMode(PolygonAttributes.POLYGON_LINE);
					}
					
					app.setPolygonAttributes(attrs);
					
					mat.setAmbientColor(new Color3f(selectionColor));
					mat.setDiffuseColor(new Color3f(selectionColor));
					mat.setSpecularColor(new Color3f(selectionColor));
					
					float newRad;
//					if(atoms[i] instanceof Particle) {
//					newRad=rad*1.2f;
//					int mC=((Particle)atoms[i]).includedAtoms.size();
//					
//					newRad+=((float)mC)/20f;
//					}
//					else
					newRad=rad;
					
					newRad*= CM2_CONSTANTS.J3D.J3D_ANGSTROM_SCALE;
					
					object_3D	= new Box(newRad
							,newRad
							,newRad
							,null);
					
					object_3D.setPickable(false);
				}else {
					object_3D	= new Sphere(rad
							* CM2_CONSTANTS.J3D.J3D_ANGSTROM_SCALE);
					atoms[i].object_3D=object_3D;
					object_3D.setUserData(atoms[i]);
				}
				
				Atom_Creator_3D.setCapabilities(object_3D);
			
				object_3D.setAppearance(app);
				
				final TransformGroup tg = new TransformGroup();
				tg.addChild(object_3D);
				
				final Transform3D t3d = new Transform3D();
				final Vector3f v = new Vector3f(atoms[i].position);
				v.scale(CM2_CONSTANTS.J3D.J3D_ANGSTROM_SCALE);
				t3d.setTranslation(v);
				tg.setTransform(t3d);
				tg.setCapability(TransformGroup.ALLOW_TRANSFORM_WRITE);
				
				if(selectionColor==null&&atoms[i]instanceof Particle
//						&&mol.isDisplayedInMoleculeEditor
						) {
					
					float newRad=rad*1.2f;
//					int mC=((Particle)atoms[i]).includedAtoms.size();
					
//					newRad+=((float)mC)/20f;
					newRad*= CM2_CONSTANTS.J3D.J3D_ANGSTROM_SCALE;
					Sphere additionalSphere	= new Sphere(newRad);
					
					Atom_Creator_3D.setCapabilities(additionalSphere);
					additionalSphere.setPickable(false);
					Appearance app2=new Appearance();
					app2.setMaterial(mat);
					Cm2_Toolkit.setRenderingAttributes(app2);
					PolygonAttributes attrs= new PolygonAttributes();
					attrs.setPolygonMode(PolygonAttributes.POLYGON_LINE);
					app2.setPolygonAttributes(attrs);
					additionalSphere.setAppearance(app2);
					tg.addChild(additionalSphere);
					
				}
				
				if(atoms[i].branchGroup==null) {
				final BranchGroup bg = new BranchGroup();
				
				bg.setUserData(atoms[i]);
				// mol.getTransformGroup().addChild(tg);
				bg.addChild(tg);
				bg.setCapability(BranchGroup.ALLOW_DETACH);
				bg.setCapability(BranchGroup.ALLOW_CHILDREN_EXTEND);
				bg.setCapability(BranchGroup.ALLOW_CHILDREN_WRITE);
				bg.setCapability(BranchGroup.ALLOW_CHILDREN_READ);
				// bg.compile();
				atoms[i].branchGroup = bg;
				}else {
					Node node = atoms[i].branchGroup.getParent();
					TransformGroup parent = (TransformGroup)node;
					parent.removeChild(atoms[i].branchGroup);
					atoms[i].branchGroup.addChild(tg);
					parent.addChild(atoms[i].branchGroup);
				}
				
			}
			
		}
		
		Thread.currentThread().setPriority(Thread.NORM_PRIORITY);
		
	}

	private static void setCapabilities(Primitive object_3D) {

		object_3D.setCapability(Group.ALLOW_COLLISION_BOUNDS_READ);
		object_3D.setCapability(Group.ALLOW_COLLISION_BOUNDS_WRITE);
		object_3D.setCapability(Node.ALLOW_AUTO_COMPUTE_BOUNDS_READ);
		object_3D.setCapability(Node.ALLOW_AUTO_COMPUTE_BOUNDS_WRITE);
		object_3D.setCapability(Node.ALLOW_BOUNDS_READ);
		object_3D.setCapability(Node.ALLOW_BOUNDS_WRITE);
		object_3D.setCapability(Node.ALLOW_COLLIDABLE_READ);
		object_3D.setCapability(Node.ALLOW_COLLIDABLE_WRITE);
		object_3D.setCapability(Node.ALLOW_PICKABLE_WRITE);
		object_3D.setCapability(Node.ALLOW_PICKABLE_READ);
		object_3D.setCapability(Node.ALLOW_COLLIDABLE_WRITE);
		object_3D.setCapability(Primitive.ENABLE_GEOMETRY_PICKING);
		//		object_3D.setPickable(false);

	}

	//	private static Atom_3D[] removeInnerAtoms(final Molecule_3D mol,
	//			final Atom_3D[] atoms, final int factor) {
	//
	//		if (factor == 30)
	//			return atoms;
	//
	//		final float scale = ((100f - factor)) / 100f;
	//		final Point3f[] points = mol.getOriginalTrianglePoints();
	//
	//		final Vector<Point3f> newShape = new Vector<Point3f>();
	//		for (int i = 0; i < points.length; i++) {
	//			final Point3f p = new Point3f(points[i]);
	//			p.scale(scale);
	//
	//			newShape.addElement(p);
	//		}
	//		final Box b = mol.getUnrotatedBounds();
	//		final BoundingBox bb = new BoundingBox();
	//
	//		final Point3d lower = new Point3d(-(b.getXdimension()), -(b
	//				.getYdimension()), -(b.getZdimension()));
	//		lower.scale(scale);
	//
	//		final Point3d upper = new Point3d(lower);
	//		upper.scale(-1);
	//
	//		bb.setLower(lower);
	//		bb.setUpper(upper);
	//
	//		final Vector<Atom_3D> ret = new Vector<Atom_3D>();
	//
	//		for (int i = 0; i < atoms.length; i++) {
	//			final Point3d p = new Point3d(atoms[i].position);
	//			p.scale(CM2_CONSTANTS.J3D.J3D_ANGSTROM_SCALE);
	//			if (!bb.intersect(p)) {
	//
	//				ret.addElement(atoms[i]);
	//				continue;
	//			}
	//
	//			final int size = newShape.size();
	//
	//			final float[] mesh = new float[size * 3];
	//			int c = -1;
	//			for (int j = 0; j < size; j += 3) {
	//
	//				final Point3f A = newShape.get(j);
	//				final Point3f B = newShape.get(j + 1);
	//				final Point3f C = newShape.get(j + 2);
	//
	//				mesh[++c] = A.x;
	//				mesh[++c] = A.y;
	//				mesh[++c] = A.z;
	//
	//				mesh[++c] = B.x;
	//				mesh[++c] = B.y;
	//				mesh[++c] = B.z;
	//
	//				mesh[++c] = C.x;
	//				mesh[++c] = C.y;
	//				mesh[++c] = C.z;
	//
	//			}
	//
	//			if (!new IntersectManager().isCompletelyInside(p, mesh))
	//				ret.addElement(atoms[i]);
	//		}
	//
	//		return ret.toArray(new Atom_3D[0]);
	//
	//	}

}
