/*******************************************************************************
 * CELLmicrocosmos 2.2 MembraneEditor (version 2.2.2_2)
 *
 * Copyright (c) 2005, 2017 The CELLmicrocosmos 2 Development Team
 * (Bjorn Sommer, Tim Dingersen et al.
 *  University of Konstanz, Monash University, Bielefeld University)
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *  
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *  
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *******************************************************************************/
package org.cellmicrocosmos.cm2.functionality3d;

import javax.media.j3d.Appearance;
import javax.media.j3d.BranchGroup;
import javax.media.j3d.ColoringAttributes;
import javax.media.j3d.Geometry;
import javax.media.j3d.GeometryArray;
import javax.media.j3d.LineArray;
import javax.media.j3d.Shape3D;
import javax.vecmath.Color3f;
import javax.vecmath.Point3f;

import org.cellmicrocosmos.cm2.constants.CM2_CONSTANTS;
import org.cellmicrocosmos.cm2.engine.DataPool;
import org.cellmicrocosmos.cm2.misc.Cm2_Toolkit;

/**
 * The Raster that is painted on the plane in the 3d view.
 * 
 * @author Tim Dingersen dackelnose@gmx.net
 * 
 */

public class Raster {

	BranchGroup branchGroup = new BranchGroup();

	Shape3D shape = new Shape3D();

	private Appearance appear;

	private Color3f color;

	public Raster() {
		Cm2_Toolkit.setAllCapabilitysForBranchGroup(this.branchGroup);
		Cm2_Toolkit.setAllCapabilitysForShape3D(this.shape);
		if (DataPool.J3DSettings.bright_appearance)
			this.color = new Color3f(
					CM2_CONSTANTS.J3D.COLOR_COORDINATEBARS_BRIGHT);
		else
			this.color = new Color3f(
					CM2_CONSTANTS.J3D.COLOR_COORDINATEBARS_DARK);
		this.shape.setPickable(false);
	}

	/**
	 * Creates a raster. For that, lines are drawn horizontally and vertically.
	 * These lines will be placed in 10 angstrom intervals
	 * 
	 */

	public void setRasterVisible(final boolean b) {

		final BranchGroup parent = (BranchGroup) this.branchGroup.getParent();
		parent.removeChild(this.branchGroup);
		this.shape.removeAllGeometries();
		if (b) {
			final float W = DataPool.getMembraneModel().getSize().width
					* CM2_CONSTANTS.J3D.J3D_ANGSTROM_SCALE;
			final float H = DataPool.getMembraneModel().getSize().height
					* CM2_CONSTANTS.J3D.J3D_ANGSTROM_SCALE;
			this.appear = new Appearance();
			
			this.appear.setCapability(Appearance.ALLOW_COLORING_ATTRIBUTES_WRITE);
			
			final ColoringAttributes ca = new ColoringAttributes(this.color,
					ColoringAttributes.SHADE_GOURAUD);
			this.appear.setColoringAttributes(ca);
			
			Cm2_Toolkit.setRenderingAttributes(appear);

			this.shape.setAppearance(this.appear);

			final float step = 0.1f;

			float c = 0;

			while (c <= W + 0.001f) {

				final Point3f[] plaPts = new Point3f[2];
				plaPts[0] = new Point3f(c, 0, 0);
				plaPts[1] = new Point3f(c, 0, H);
				final LineArray pla = new LineArray(2,
						GeometryArray.COORDINATES);
				pla.setCoordinates(0, plaPts);
				pla.setCapability(Geometry.ALLOW_INTERSECT);
				this.shape.addGeometry(pla);
				
				c += step;
			}
			c = 0;

			while (c <= H + 0.001f) {

				final Point3f[] plaPts = new Point3f[2];
				plaPts[0] = new Point3f(0, 0, c);
				plaPts[1] = new Point3f(W, 0, c);
				final LineArray pla = new LineArray(2,
						GeometryArray.COORDINATES);
				pla.setCoordinates(0, plaPts);
				pla.setCapability(Geometry.ALLOW_INTERSECT);
				this.shape.addGeometry(pla);
				c += step;
			}

			if (this.shape.getParent() == null)
				this.branchGroup.addChild(this.shape);
		} else {
			if (this.shape.getParent() == this.branchGroup)
				this.branchGroup.removeChild(this.shape);
		}
		parent.addChild(this.branchGroup);
	}

	public BranchGroup getBranchGroup() {
		return this.branchGroup;
	}

	public void setColor(final Color3f color) {
		this.color = color;
	}

}
