/*******************************************************************************
 * CELLmicrocosmos 2.2 MembraneEditor (version 2.2.2_2)
 *
 * Copyright (c) 2005, 2017 The CELLmicrocosmos 2 Development Team
 * (Bjorn Sommer, Tim Dingersen et al.
 *  University of Konstanz, Monash University, Bielefeld University)
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *  
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *  
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *******************************************************************************/
package org.cellmicrocosmos.cm2.gui;

import java.awt.BasicStroke;
import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.Event;
import java.awt.Font;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Insets;
import java.awt.Point;
import java.awt.Shape;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.awt.geom.RoundRectangle2D;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.util.Vector;

import javax.swing.AbstractAction;
import javax.swing.BorderFactory;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComponent;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JPopupMenu;
import javax.swing.JSlider;
import javax.swing.JToolBar;
import javax.swing.KeyStroke;
import javax.swing.UIManager;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import org.cellmicrocosmos.cm2.constants.CM2_CONSTANTS;
import org.cellmicrocosmos.cm2.engine.DataPool;
import org.cellmicrocosmos.cm2.engine.TestArea;
import org.cellmicrocosmos.cm2.engine.DataPool.APLVoroSettings;
import org.cellmicrocosmos.cm2.gui.dialogs.AboutPane;
import org.cellmicrocosmos.cm2.gui.dialogs.AlgorithmWizzard;
import org.cellmicrocosmos.cm2.gui.dialogs.HelpDialog;
import org.cellmicrocosmos.cm2.gui.dialogs.SetUpConnectionsDialog;
import org.cellmicrocosmos.cm2.interfaces.GuiListener;
import org.cellmicrocosmos.cm2.interfaces.MembraneViewListener;
import org.cellmicrocosmos.cm2.misc.Cm2_IconDepot;
import org.cellmicrocosmos.cm2.misc.Cm2_Toolkit;
import org.cellmicrocosmos.cm2.misc.ConfigParser;

import net.infonode.docking.DockingWindow;
import net.infonode.docking.RootWindow;
import net.infonode.docking.SplitWindow;
import net.infonode.docking.TabWindow;
import net.infonode.docking.View;
import net.infonode.docking.ViewSerializer;
import net.infonode.docking.properties.RootWindowProperties;
import net.infonode.docking.theme.ClassicDockingTheme;
import net.infonode.docking.theme.DockingWindowsTheme;
import net.infonode.docking.theme.GradientDockingTheme;
import net.infonode.docking.theme.ShapedGradientDockingTheme;
import net.infonode.docking.theme.SlimFlatDockingTheme;
import net.infonode.docking.theme.SoftBlueIceDockingTheme;
import net.infonode.docking.util.DockingUtil;
import net.infonode.docking.util.MixedViewHandler;
import net.infonode.docking.util.PropertiesUtil;
import net.infonode.docking.util.ViewMap;
import net.infonode.gui.shaped.border.ShapedBorder;
import net.infonode.tabbedpanel.titledtab.TitledTabBorderSizePolicy;
import net.infonode.util.Direction;

/**
 * Builds up a Gui Framework with multiple Dockable Frames. Each
 * Component that is added to this class will be shown in a seperate
 * internal Docking Frame. Handles all view options that can be accessed
 * set in a viewMenu by itself. Provides also major Options for the User
 * to perform loading and saving. All relevant use cases are sent to the
 * GuiListener, if one is added
 *         
 * @author Dingi dackelnose@gmx.net, Bjoern Sommer bjoern@CELLmicrocosmos.org
 * 
 */
public class MainGui implements ActionListener, ChangeListener {

	// this number must be increased if another item is added to the file menu
	private final int MENU_POSITION_TO_ADD_USED_FILES = 18; 
	
	JFrame frame = new JFrame();

	private WindowAdapter windowCloserStandard, windowCloserBusy; 
	
	private JMenuBar menuBar;

	private JToolBar toolbar;

	private JMenuItem loadDB, newDB, openGlobal, newMembrane, saveMembraneAs, saveImageAs,
			quicksaveMembrane, loadMembrane, generateMembrane, previewMembrane,previewMembrane2,
			editAttrs, changeSize, exit, contentsMenuItem, aboutMenuItem,
			showToolbar, showBordersAndTitles, lockgui, proxyItem, openExternal, j3dsettings,
			appearanceBrightItem, appearanceDarkItem, pluginManager,
			pdbSettingsItem, refreshItem, testItem, newDomainItem,
			addMultilayerItem, restoreFromPDBItem, showAlgorithmProtocol;

	private boolean showBordersAndTitlesIsSelected = false;
	private static JMenuItem analyzeMembrane;
	
	private JButton newMembraneBtn, saveMembraneAsBtn, quicksaveMembraneBtn, saveImageAsBtn, 
			loadMembraneBtn, generateMembraneBtn, previewMembraneBtn,previewMembraneBtn2, 
			editAttrsBtn, changeSizeBtn, pdbSettingsBtn, loadDBBtn, newDBBtn,
			helpBtn, refreshBtn, newDomainBtn, addMultilayerBtn;

	private static JButton analyzeMembraneBtn;
	
	private JMenu viewMenu, selectTheme, helpMenu, databaseMenu, fileMenu,
			layoutMenu, settingsMenu, extrasMenu, appearanceMenu;

	private JSlider stereoSlider;

	private JLabel stereoSliderPosition;

	private JCheckBox stereoEnable;

	private RootWindow rootWindow;

	final private RootWindowProperties rootwinprops = new RootWindowProperties();
	
	private RootWindowProperties hideTabTitleStyleProperties;
	
	final private ViewMap viewmap = new ViewMap();

	private final String[] THEMES = { "Shaped Gradient", "Gradient",
			"Slim Flat", "Ice", "Classic" };

	public GuiListener guiListener;
	
	private MembraneViewListener membraneViewListener;

	private boolean isGuilocked = false;

	private String currentTheme = "Shaped Gradient";

	private File layoutFile;

	private String defaultTitle;

	private boolean enabled = true;

	public JCheckBox getStereoEnable() {
		return this.stereoEnable;
	}

	public void setStereoEnable(final JCheckBox stereoEnable) {
		this.stereoEnable = stereoEnable;
	}

	public JLabel getStereoSliderPosition() {
		return this.stereoSliderPosition;
	}

	public void setStereoSliderPosition(final JLabel stereoSliderPosition) {
		this.stereoSliderPosition = stereoSliderPosition;
	}

	public JSlider getStereoSlider() {
		return this.stereoSlider;
	}

	/**
	 * Adds a JComponent to the Gui
	 * 
	 * @param name
	 *            - the title for the component's window
	 * @param compo
	 *            - the component to add
	 */
	public void addView(final String name, final JComponent compo) {

		this.viewmap.addView(this.viewmap.getViewCount(), new View(name, null,
				compo));

	}

	/**
	 * * Adds a JComponent to the Gui
	 * 
	 * @param name
	 *            - the title for the component's window
	 * @param compo
	 *            - the component to add
	 * @param icon
	 *            - a icon to add to the windowbar
	 */
	public void addView(final String name, final ImageIcon icon,
			final JComponent compo) {

		this.viewmap.addView(this.viewmap.getViewCount(), new View(name, icon,
				compo));

	}

	/**
	 * builds up the gui with all added views
	 * 
	 */
	public void generateGui() {

		// building up the toolbar
		this.createToolbar();

		// and the menubar
		this.createMenuBar();

		// putting the gui together
		this.buildMainFrame();
		this.insertToolbar();

		// ..and configure it
		this.arrangeGuiSettings();

		this.frame.pack();

	}

	private void insertToolbar() {

		String str;

		try {
			str = ConfigParser.getValue("toolbar_Orientation")[0];

		} catch (final Exception e) {

			this.frame.getContentPane().add(this.toolbar, BorderLayout.NORTH);

			return;
		}

		if (str.equals("NORTH")) {
			this.frame.getContentPane().add(this.toolbar, BorderLayout.NORTH);
		} else if (str.equals("SOUTH")) {

			this.frame.getContentPane().add(this.toolbar, BorderLayout.SOUTH);
		} else if (str.equals("EAST")) {
			this.toolbar.setOrientation(JToolBar.VERTICAL);
			this.frame.getContentPane().add(this.toolbar, BorderLayout.EAST);

		} else if (str.equals("WEST")) {
			this.toolbar.setOrientation(JToolBar.VERTICAL);
			this.frame.getContentPane().add(this.toolbar, BorderLayout.WEST);

		} else
			this.frame.getContentPane().add(this.toolbar, BorderLayout.NORTH);

	}

	/**
	 * Focus the view specified by its index and brings it on top if covered.
	 * 
	 * 
	 * @param the
	 *            index of the view
	 */
	public void focusView(final int index) {

		if (this.viewmap.getViewCount() < index - 1)
			return;
		this.viewmap.getView(index).restoreFocus();

	}

	/**
	 * Puts the frame together
	 * 
	 */
	private void buildMainFrame() {

		this.frame.setDefaultCloseOperation(JFrame.DO_NOTHING_ON_CLOSE);
		
		this.windowCloserStandard = new WindowAdapter() {
			/**
			 * initialize exit (standard closing listener)
			 */
			public void windowClosing(final WindowEvent e) {
				MainGui.this.storeCurrentGuiSettings();
				if (MainGui.this.guiListener != null)
					MainGui.this.guiListener.exitApplication();
				if (MainGui.this.membraneViewListener != null)
					MainGui.this.guiListener.exitApplication();
			}
		};
		
		this.windowCloserBusy = new WindowAdapter() {
			/**
			 * Force exit?
			 */
			public void windowClosing(final WindowEvent e) {
				int val = JOptionPane
					.showOptionDialog(
							frame,
							"CmME is busy!\nAn algorithm is running or a molecule is aligned!\nIf you exit now, ALL changes will be lost!\nAre you sure you want to exit anyway?"
							, "CmME is busy!",
							JOptionPane.YES_NO_OPTION,
							JOptionPane.WARNING_MESSAGE, null,
							new String[] { "Yes, exit CmME", "No, continue the work" }, null);
				if (val == JOptionPane.OK_OPTION)
					System.exit(0);
			}
		};
		
		this.frame.addWindowListener(this.windowCloserStandard);
		
		this.frame.setIconImage(Cm2_IconDepot.getSpecifiedIcon("Cm2_Icon")
				.getImage());

		/*
		 * generating a viewhandler for giving the rootwindow. This is necessary
		 * for proper serialization of the layout
		 */
		final MixedViewHandler handler = new MixedViewHandler(this.viewmap,
				new ViewSerializer() {

					// those two methods are required to implement here
					public View readView(final ObjectInputStream arg0)
							throws IOException {
						return null;
					}

					public void writeView(final View arg0,
							final ObjectOutputStream arg1) throws IOException {
					}
				});

		// initializes the rootwindow with all views
		this.rootWindow = DockingUtil.createRootWindow(this.viewmap, handler,
				true);

		// setting a default Theme
		this.rootwinprops.addSuperObject(new ShapedGradientDockingTheme()
				.getRootWindowProperties());

		// adding the properties to the rootwindow
		this.rootWindow.getRootWindowProperties().addSuperObject(
				this.rootwinprops);

		// avoids closing of the whole rootwindow
		this.rootwinprops.getDockingWindowProperties().setCloseEnabled(false);

		// enabling a window bar where views can be minimized to
		this.rootWindow.getWindowBar(Direction.DOWN).setEnabled(true);

		this.frame.setJMenuBar(this.menuBar);

		// frame.getContentPane().add(toolbar, BorderLayout.NORTH);
		this.frame.getContentPane().add(this.rootWindow, BorderLayout.CENTER);
		
		// initialize the borderless layout
		this.initilizeBorderlessLayout();

	}

	/**
	 * builds up the Toolbar
	 * 
	 */
	private void createToolbar() {

		this.toolbar = new JToolBar();

		this.newMembraneBtn = new JButton();
		this.loadMembraneBtn = new JButton();
		this.quicksaveMembraneBtn = new JButton();
		this.saveMembraneAsBtn = new JButton();
		this.saveImageAsBtn = new JButton();
		this.generateMembraneBtn = new JButton();
		this.newDBBtn = new JButton();
		this.loadDBBtn = new JButton();
		this.previewMembraneBtn = new JButton();
		this.previewMembraneBtn2 = new JButton();
		analyzeMembraneBtn = new JButton();
		this.editAttrsBtn = new JButton();
		this.changeSizeBtn = new JButton();
		this.pdbSettingsBtn = new JButton();
		this.refreshBtn = new JButton();
		this.helpBtn = new JButton();
		this.newDomainBtn = new JButton();
		this.addMultilayerBtn = new JButton();

		this.stereoEnable = new JCheckBox("Stereo");
		this.stereoEnable.setEnabled(false);
		this.stereoEnable.setSelected(CM2_CONSTANTS.J3D.STEREOSCOPIC_3D_QUAD_MODE_PREFERRED);
		this.stereoEnable.setVisible(false);
		
		this.stereoEnable.addMouseListener(new MouseListener() {

			public void mouseClicked(MouseEvent e) {
				
				MainGui.this.guiListener.userChangedStereoEnable(MainGui.this.stereoEnable.isSelected());
				if (e.getButton() == MouseEvent.BUTTON3 && e.getClickCount() == 1) {
					
					if (stereoEnable.isSelected()) {
						stereoEnable.setSelected(false);
						MainGui.this.membraneViewListener.validateStereoMode(MainGui.this.stereoEnable.isSelected());
					}
						
					JPopupMenu menu = new JPopupMenu();
					
					// select the 3D Mode
					
					JMenu select3DModeMenu = new JMenu("3D Stereo Modes");

					select3DModeMenu.add(new AbstractAction(
							
							(MainGui.this.membraneViewListener.getStereoViewMode() == CM2_CONSTANTS.J3D.STEREO_VIEW_MODE__OVER_UNDER__TOP_LEFT)?
									"[Over-Under (Top: Left)]":
									"Over-Under (Top: Left)") {

						private static final long serialVersionUID = 5695992038250333959L;

						public void actionPerformed(ActionEvent e) {
							MainGui.this.membraneViewListener.setStereoViewMode(CM2_CONSTANTS.J3D.STEREO_VIEW_MODE__OVER_UNDER__TOP_LEFT);
							stereoEnable.setSelected(true);
							MainGui.this.membraneViewListener.validateStereoMode(MainGui.this.stereoEnable.isSelected());
						}
					});
					
					select3DModeMenu.add(new AbstractAction(
							
							(MainGui.this.membraneViewListener.getStereoViewMode() == CM2_CONSTANTS.J3D.STEREO_VIEW_MODE__OVER_UNDER__TOP_RIGHT)?
									"[Over-Under (Top: Right)]":
									"Over-Under (Top: Right)") {

						private static final long serialVersionUID = -9009447816939075517L;

						public void actionPerformed(ActionEvent e) {
							MainGui.this.membraneViewListener.setStereoViewMode(CM2_CONSTANTS.J3D.STEREO_VIEW_MODE__OVER_UNDER__TOP_RIGHT);
							stereoEnable.setSelected(true);
							MainGui.this.membraneViewListener.validateStereoMode(MainGui.this.stereoEnable.isSelected());
						}
					});
					
					select3DModeMenu.add(new AbstractAction(
							
							(MainGui.this.membraneViewListener.getStereoViewMode() == CM2_CONSTANTS.J3D.STEREO_VIEW_MODE__SIDE_BY_SIDE__LEFT_LEFT)?
									"[Side-by-Side (Left: Left)]":
									"Side-by-Side (Left: Left)") {


						/**
										 * 
										 */
										private static final long serialVersionUID = 1765580847696298180L;

						public void actionPerformed(ActionEvent e) {
							MainGui.this.membraneViewListener.setStereoViewMode(CM2_CONSTANTS.J3D.STEREO_VIEW_MODE__SIDE_BY_SIDE__LEFT_LEFT);
							stereoEnable.setSelected(true);
							MainGui.this.membraneViewListener.validateStereoMode(MainGui.this.stereoEnable.isSelected());
						}
					});
					
					select3DModeMenu.add(new AbstractAction(
							
							(MainGui.this.membraneViewListener.getStereoViewMode() == CM2_CONSTANTS.J3D.STEREO_VIEW_MODE__SIDE_BY_SIDE__LEFT_RIGHT)?
									"[Side-by-Side (Left: Right)]":
									"Side-by-Side (Left: Right)") {

						private static final long serialVersionUID = -7180474433811527676L;

						public void actionPerformed(ActionEvent e) {
							MainGui.this.membraneViewListener.setStereoViewMode(CM2_CONSTANTS.J3D.STEREO_VIEW_MODE__SIDE_BY_SIDE__LEFT_RIGHT);
							stereoEnable.setSelected(true);
							MainGui.this.membraneViewListener.validateStereoMode(MainGui.this.stereoEnable.isSelected());
						}
					});
					
					menu.add(select3DModeMenu);

					// select the 3D scaling mode
					
					JMenu select3DScalingModeMenu = new JMenu("3D Stereo Scaling Modes");

					select3DScalingModeMenu.add(new AbstractAction(
							
							(MainGui.this.membraneViewListener.getStereoViewScalingMode() == CM2_CONSTANTS.J3D.STEREO_VIEW_MODE_SCALING__NORMAL)?
									"[Normal Scaling]":
									"Normal Scaling") {

						private static final long serialVersionUID = 7721755229291842324L;

						public void actionPerformed(ActionEvent e) {
							MainGui.this.membraneViewListener.setStereoViewScalingMode(CM2_CONSTANTS.J3D.STEREO_VIEW_MODE_SCALING__NORMAL);
							stereoEnable.setSelected(true);
							MainGui.this.membraneViewListener.validateStereoMode(MainGui.this.stereoEnable.isSelected());
						}
					});
					
					select3DScalingModeMenu.add(new AbstractAction(
							
							(MainGui.this.membraneViewListener.getStereoViewScalingMode() == CM2_CONSTANTS.J3D.STEREO_VIEW_MODE_SCALING__DOUBLE)?
									"[Double Scaling]":
									"Double Scaling") {

						private static final long serialVersionUID = 7966112838755270918L;

						public void actionPerformed(ActionEvent e) {
							MainGui.this.membraneViewListener.setStereoViewScalingMode(CM2_CONSTANTS.J3D.STEREO_VIEW_MODE_SCALING__DOUBLE);
							stereoEnable.setSelected(true);
							MainGui.this.membraneViewListener.validateStereoMode(MainGui.this.stereoEnable.isSelected());
						}
					});

					menu.add(select3DScalingModeMenu);
					
					// select the 3D auto focus mode
					
					JMenu select3DAutoFocusModeMenu = new JMenu("3D Stereo Auto Focus Modes");

					select3DAutoFocusModeMenu.add(new AbstractAction(
							
							(MainGui.this.membraneViewListener.getStereoViewAutoFocusMode() == CM2_CONSTANTS.J3D.STEREO_VIEW_MODE_AUTOFOCUS__FIX)?
									"[Fix Focus]":
									"Fix Focus") {


						private static final long serialVersionUID = 8125759614796087936L;

						public void actionPerformed(ActionEvent e) {
							MainGui.this.membraneViewListener.setStereoViewAutoFocusMode(CM2_CONSTANTS.J3D.STEREO_VIEW_MODE_AUTOFOCUS__FIX);
							stereoEnable.setSelected(true);
							// TODO reset value
//							ObjectPool.getMainWindowObject().getStereoPanel().getStereoPreferencesPanel().resetValue();
							MainGui.this.membraneViewListener.validateStereoMode(MainGui.this.stereoEnable.isSelected());
						}
					});
					
					select3DAutoFocusModeMenu.add(new AbstractAction(
							
							(MainGui.this.membraneViewListener.getStereoViewAutoFocusMode() == CM2_CONSTANTS.J3D.STEREO_VIEW_MODE_AUTOFOCUS__AUTO)?
									"[Auto Focus]":
									"Auto Focus") {

						private static final long serialVersionUID = 6436006372106316143L;

						public void actionPerformed(ActionEvent e) {
							MainGui.this.membraneViewListener.setStereoViewAutoFocusMode(CM2_CONSTANTS.J3D.STEREO_VIEW_MODE_AUTOFOCUS__AUTO);
							stereoEnable.setSelected(true);
							MainGui.this.membraneViewListener.validateStereoMode(MainGui.this.stereoEnable.isSelected());
						}
					});

					menu.add(select3DAutoFocusModeMenu);
					
					// show the menu
					
					menu.show(e.getComponent(), e.getX(), e.getY());
				}
			}

			public void mousePressed(MouseEvent e) {
			}

			public void mouseReleased(MouseEvent e) {
			}

			public void mouseEntered(MouseEvent e) {
			}

			public void mouseExited(MouseEvent e) {
			}
			
		});

		this.stereoSlider = new JSlider(CM2_CONSTANTS.J3D.STEREOVIEW_MIN,
				CM2_CONSTANTS.J3D.STEREOVIEW_MAX,
				CM2_CONSTANTS.J3D.STEREOVIEW_DEFAULT);
		this.stereoSlider.setPaintTicks(true);
		this.stereoSlider.setMajorTickSpacing(30);
		this.stereoSliderPosition = new JLabel();
		this.stereoSliderPosition.setText(String.format("%.3f",
				CM2_CONSTANTS.J3D.STEREOVIEW_DEFAULT
						/ CM2_CONSTANTS.J3D.STEREOVIEW_SCALE));
		this.stereoSlider.setVisible(false);
		this.stereoSliderPosition.setVisible(false);

		this.toolbar.addSeparator(new Dimension(5, 0));
		this.toolbar.add(this.newMembraneBtn);
		this.toolbar.addSeparator(new Dimension(5, 0));
		this.toolbar.add(this.loadMembraneBtn);
		this.toolbar.addSeparator(new Dimension(5, 0));
		this.toolbar.add(this.saveMembraneAsBtn);
		this.toolbar.addSeparator(new Dimension(5, 0));
		this.toolbar.add(this.quicksaveMembraneBtn);
		this.toolbar.addSeparator(new Dimension(5, 0));
		this.toolbar.add(this.saveImageAsBtn);
		this.toolbar.addSeparator(new Dimension(10, 0));
		this.toolbar.add(this.refreshBtn);
	
		this.toolbar.addSeparator(new Dimension(15, 0));
		this.toolbar.addSeparator();
		this.toolbar.addSeparator(new Dimension(15, 0));
		this.toolbar.add(this.newDBBtn);
		this.toolbar.addSeparator(new Dimension(5, 0));
		this.toolbar.add(this.loadDBBtn);
		this.toolbar.addSeparator(new Dimension(15, 0));
		this.toolbar.addSeparator();
		
		
		this.toolbar.addSeparator(new Dimension(15, 0));
		this.toolbar.add(this.editAttrsBtn);
		this.toolbar.addSeparator(new Dimension(5, 0));
		this.toolbar.add(this.changeSizeBtn);
		this.toolbar.addSeparator(new Dimension(5, 0));
		this.toolbar.add(this.newDomainBtn);
		this.toolbar.addSeparator(new Dimension(5, 0));
		this.toolbar.add(this.addMultilayerBtn);
		this.toolbar.addSeparator(new Dimension(15, 0));
		
		this.toolbar.addSeparator();

		this.toolbar.addSeparator(new Dimension(15, 0));
	
		
		this.toolbar.add(this.pdbSettingsBtn);
		this.toolbar.addSeparator(new Dimension(25, 0));
		this.toolbar.add(this.previewMembraneBtn);
		
		
		
		this.toolbar.addSeparator(new Dimension(5, 0));
		this.toolbar.add(this.previewMembraneBtn2);
		
		
		this.toolbar.addSeparator(new Dimension(8, 0));
		this.toolbar.add(analyzeMembraneBtn);

		this.toolbar.addSeparator(new Dimension(20, 0));
		this.toolbar.add(this.generateMembraneBtn);
		this.toolbar.addSeparator(new Dimension(15, 0));
		this.toolbar.addSeparator();

		// TODO As long as the Stereo View does not work on Mac OS X, the toolbar is deactivated
		if (!System.getProperty("os.name").toString().contains("Mac OS X")) {
			this.toolbar.add(this.stereoEnable);
			this.toolbar.add(this.stereoSlider);
			this.toolbar.add(this.stereoSliderPosition);
		}

		this.toolbar.add(new JPanel());
		this.toolbar.addSeparator();
		this.toolbar.add(this.helpBtn);

		this.newMembraneBtn.setIcon(Cm2_IconDepot
				.getSpecifiedIcon("Toolbar_NewMembrane"));
		this.loadMembraneBtn.setIcon(Cm2_IconDepot
				.getSpecifiedIcon("Toolbar_LoadMembrane"));
		this.quicksaveMembraneBtn.setIcon(Cm2_IconDepot
				.getSpecifiedIcon("Toolbar_QuickSaveMembrane"));
		this.saveMembraneAsBtn.setIcon(Cm2_IconDepot
				.getSpecifiedIcon("Toolbar_SaveMembraneAs"));
		this.saveImageAsBtn.setIcon(Cm2_IconDepot
				.getSpecifiedIcon("Toolbar_SaveImageAs"));
		this.generateMembraneBtn.setIcon(Cm2_IconDepot
				.getSpecifiedIcon("Toolbar_Generate"));
		this.newDBBtn.setIcon(Cm2_IconDepot.getSpecifiedIcon("Toolbar_NewDB"));
		this.loadDBBtn
				.setIcon(Cm2_IconDepot.getSpecifiedIcon("Toolbar_LoadDB"));
		this.previewMembraneBtn.setIcon(Cm2_IconDepot
				.getSpecifiedIcon("Toolbar_PreviewMembrane"));
		this.previewMembraneBtn2.setIcon(Cm2_IconDepot
				.getSpecifiedIcon("Toolbar_PreviewMembrane2"));
		analyzeMembraneBtn.setIcon(Cm2_IconDepot
				.getSpecifiedIcon("Toolbar_AnalyzeMembrane"));
		this.editAttrsBtn.setIcon(Cm2_IconDepot
				.getSpecifiedIcon("Toolbar_EditAttrs"));
		this.changeSizeBtn.setIcon(Cm2_IconDepot
				.getSpecifiedIcon("Toolbar_ChangeSize"));
		this.pdbSettingsBtn.setIcon(Cm2_IconDepot
				.getSpecifiedIcon("Toolbar_PDBSettings"));
		this.refreshBtn.setIcon(Cm2_IconDepot
				.getSpecifiedIcon("Frame_Refresh_Geometry"));
		this.newDomainBtn.setIcon(Cm2_IconDepot
				.getSpecifiedIcon("MV_NewDomain"));
		this.helpBtn.setIcon(Cm2_IconDepot.getSpecifiedIcon("Toolbar_Help"));

		this.addMultilayerBtn.setIcon(Cm2_IconDepot
				.getSpecifiedIcon("Toolbar_Multilayer"));

		this.newMembraneBtn.addActionListener(this);
		this.loadMembraneBtn.addActionListener(this);
		this.quicksaveMembraneBtn.addActionListener(this);
		this.saveMembraneAsBtn.addActionListener(this);
		this.saveImageAsBtn.addActionListener(this);
		this.generateMembraneBtn.addActionListener(this);
		this.editAttrsBtn.addActionListener(this);
		this.changeSizeBtn.addActionListener(this);
		this.pdbSettingsBtn.addActionListener(this);
		this.previewMembraneBtn.addActionListener(this);
		this.previewMembraneBtn2.addActionListener(this);
		analyzeMembraneBtn.addActionListener(this);
		this.newDBBtn.addActionListener(this);
		this.loadDBBtn.addActionListener(this);
		this.refreshBtn.addActionListener(this);
		this.helpBtn.addActionListener(this);
		this.newDomainBtn.addActionListener(this);
		this.stereoEnable.addActionListener(this);
		this.stereoSlider.addChangeListener(this);
		this.addMultilayerBtn.addActionListener(this);

		this.newMembraneBtn.setToolTipText(this.getTooltip(0));
		this.loadMembraneBtn.setToolTipText(this.getTooltip(1));
		this.quicksaveMembraneBtn.setToolTipText(this.getTooltip(17));
		this.saveMembraneAsBtn.setToolTipText(this.getTooltip(2));
		this.saveImageAsBtn.setToolTipText(this.getTooltip(30));
		this.generateMembraneBtn.setToolTipText(this.getTooltip(6));
		this.newDBBtn.setToolTipText(this.getTooltip(3));
		this.loadDBBtn.setToolTipText(this.getTooltip(4));
		this.helpBtn.setToolTipText(this.getTooltip(7));
		this.previewMembraneBtn.setToolTipText(this.getTooltip(18));
		this.previewMembraneBtn2.setToolTipText(this.getTooltip(28));
		analyzeMembraneBtn.setToolTipText(this.getTooltip(31));
		this.editAttrsBtn.setToolTipText(this.getTooltip(19));
		this.changeSizeBtn.setToolTipText(this.getTooltip(21));
		this.pdbSettingsBtn.setToolTipText(this.getTooltip(22));
		this.refreshBtn.setToolTipText(this.getTooltip(23));
		this.newDomainBtn.setToolTipText(this.getTooltip(24));
		this.stereoEnable.setToolTipText(this.getTooltip(25));
		this.stereoSlider.setToolTipText(this.getTooltip(27));
		this.stereoSliderPosition.setToolTipText(this.getTooltip(27));

	}

	/**
	 * builds up the menubar
	 * 
	 */
	private void createMenuBar() {

		this.menuBar = new JMenuBar();

		this.fileMenu = new JMenu("File");

		this.newMembrane = new JMenuItem("New Membrane Model");
		this.loadMembrane = new JMenuItem("Load Membrane Model");
		this.restoreFromPDBItem = new JMenuItem("Rebuild Membrane Model from PDB File");
		this.saveMembraneAs = new JMenuItem("Save Membrane Model As...");
		this.quicksaveMembrane = new JMenuItem("Save Membrane Model");
		this.saveImageAs = new JMenuItem("Save Membrane Image As...");
		this.previewMembrane = new JMenuItem("Preview Membrane in Jmol");
		this.previewMembrane2 = new JMenuItem("Preview Membrane as Pdb File");
		analyzeMembrane = new JMenuItem("Analyze Membrane with APL@Voro");
		this.editAttrs = new JMenuItem("Edit Attributes");
		this.changeSize = new JMenuItem("Change Size");
		this.pluginManager = new JMenuItem("Manage Plug-In Algorithms");
		this.pdbSettingsItem = new JMenuItem("Output Settings");
		this.refreshItem = new JMenuItem("Refresh Geometry");
		this.newDomainItem = new JMenuItem("New Microdomain");
		this.testItem = new JMenuItem("Run Test Class");
		this.addMultilayerItem = new JMenuItem("Add Secondary Bilayer");
		this.showAlgorithmProtocol = new JMenuItem("Show Protocol");

		this.newMembrane.setToolTipText(this.getTooltip(0));
		this.loadMembrane.setToolTipText(this.getTooltip(1));
		this.saveMembraneAs.setToolTipText(this.getTooltip(2));
		this.saveImageAs.setToolTipText(this.getTooltip(30));
		this.quicksaveMembrane.setToolTipText(this.getTooltip(17));

		this.previewMembrane.setToolTipText(this.getTooltip(18));
		this.previewMembrane2.setToolTipText(this.getTooltip(28));
		analyzeMembrane.setToolTipText(this.getTooltip(31));
		this.editAttrs.setToolTipText(this.getTooltip(19));
		this.changeSize.setToolTipText(this.getTooltip(21));
		this.refreshItem.setToolTipText(this.getTooltip(23));
		this.newDomainItem.setToolTipText(this.getTooltip(24));
		this.testItem
				.setToolTipText("<html>Executes the code in the test class. This button is<br>"
						+ "only available when the test class is active</html>");
		this.showAlgorithmProtocol.setToolTipText(this.getTooltip(29));

		this.addMultilayerItem
				.setToolTipText("Add a secondary bilayer to the model");
		this.addMultilayerBtn
				.setToolTipText("Add a secondary bilayer to the model");
		this.restoreFromPDBItem
				.setToolTipText("Restore a membrane model from an already generated pdb file");

		this.generateMembrane = new JMenuItem("Generate the PDB File As...");

		this.generateMembrane.setToolTipText(this.getTooltip(6));

		this.exit = new JMenuItem("Exit");

		this.exit.setToolTipText(this.getTooltip(9));

		this.fileMenu.add(this.newMembrane);
		this.fileMenu.add(this.loadMembrane);
		this.fileMenu.add(this.restoreFromPDBItem);
		this.fileMenu.add(this.quicksaveMembrane);
		this.fileMenu.add(this.saveMembraneAs);
		this.fileMenu.add(this.saveImageAs);
		this.fileMenu.addSeparator();
		this.fileMenu.add(this.previewMembrane);
		this.fileMenu.add(this.previewMembrane2);
		this.fileMenu.add(analyzeMembrane);
		this.fileMenu.add(this.generateMembrane);
		this.fileMenu.addSeparator();
		this.fileMenu.add(this.editAttrs);
		this.fileMenu.add(this.changeSize);
		this.fileMenu.add(this.newDomainItem);
		this.fileMenu.add(this.addMultilayerItem);
		this.fileMenu.addSeparator();
		this.fileMenu.add(this.refreshItem);
		this.fileMenu.addSeparator();

		this.fileMenu.addSeparator();
		this.fileMenu.add(this.exit);

		this.menuBar.add(this.fileMenu);

		this.databaseMenu = new JMenu("Database");

		this.newDB = new JMenuItem("New Protein Database");
		this.loadDB = new JMenuItem("Open Protein Database");
		this.openGlobal = new JMenuItem("Search and Download Pdb's");

		this.newDB.setToolTipText(this.getTooltip(3));
		this.loadDB.setToolTipText(this.getTooltip(4));

		this.databaseMenu.add(this.newDB);
		this.databaseMenu.add(this.loadDB);
		this.databaseMenu.addSeparator();
		this.databaseMenu.add(this.openGlobal);
		this.menuBar.add(this.databaseMenu);

		this.viewMenu = new JMenu("Gui");

		this.selectTheme = new JMenu("Select Theme");

		this.layoutMenu = new JMenu("Restore Layout");

		this.layoutMenu.add(new JMenuItem("Modern (default)"));
		this.layoutMenu.add(new JMenuItem("Widescreen"));
		this.layoutMenu.add(new JMenuItem("Cm2.1 Classic"));

		this.showToolbar = new JMenuItem("Show Toolbar");
		this.showToolbar.setToolTipText(this.getTooltip(10));
		
		this.showBordersAndTitles = new JMenuItem("Show Borders");
		this.showBordersAndTitles.setToolTipText(this.getTooltip(32));
		
		this.lockgui = new JMenuItem("L O C K");
		this.lockgui.setToolTipText(this.getTooltip(11));

		this.appearanceMenu = new JMenu("Editor Appearance");

		this.appearanceDarkItem = new JMenuItem("Dark");

		this.appearanceBrightItem = new JMenuItem("Bright");

		this.appearanceMenu.add(this.appearanceDarkItem);

		this.appearanceMenu.add(this.appearanceBrightItem);

		this.viewMenu.add(this.layoutMenu);
		this.viewMenu.add(this.selectTheme);
		this.viewMenu.add(this.appearanceMenu);

		this.viewMenu.addSeparator();
		this.viewMenu.add(this.showToolbar);
		this.viewMenu.add(this.showBordersAndTitles);
		this.viewMenu.add(this.lockgui);

		this.menuBar.add(this.viewMenu);

		this.settingsMenu = new JMenu("Settings");

		this.menuBar.add(this.settingsMenu);

		this.proxyItem = new JMenuItem("Connection");

		this.settingsMenu.add(this.proxyItem);

		this.extrasMenu = new JMenu("Extras");

		this.openExternal = new JMenuItem("Open External");

		this.j3dsettings = new JMenuItem("3D Settings");

		this.settingsMenu.add(this.j3dsettings);

		// this.extrasMenu.add(openExternal);
		// this.extrasMenu.add(this.restoreFromPDBItem);

		this.extrasMenu.add(this.showAlgorithmProtocol);
		
		if (TestArea.isActive && !Cm2_Toolkit.amIRunningOnWebstart())
			this.extrasMenu.add(this.testItem);

		this.menuBar.add(this.extrasMenu);

		this.openExternal.addActionListener(this);
		this.testItem.addActionListener(this);

		this.openExternal
				.setToolTipText("Open any Pdb file on your disc using the Molecule View");

		this.j3dsettings.addActionListener(this);

		this.proxyItem.setIcon(Cm2_IconDepot.getSpecifiedIcon("Frame_Proxy"));
		this.j3dsettings.setIcon(Cm2_IconDepot.getSpecifiedIcon("MV_Detail"));
		this.showAlgorithmProtocol.setIcon(Cm2_IconDepot.getSpecifiedIcon("Frame_Protocol"));
		
		this.pluginManager.setIcon(Cm2_IconDepot
				.getSpecifiedIcon("Frame_PlugInManager"));
		this.pdbSettingsItem.setIcon(Cm2_IconDepot
				.getSpecifiedIcon("Toolbar_PDBSettings"));

		this.pluginManager.setToolTipText(this.getTooltip(20));

		this.refreshItem.setIcon(Cm2_IconDepot
				.getSpecifiedIcon("Frame_Refresh_Geometry"));
		this.newDomainItem.setIcon(Cm2_IconDepot
				.getSpecifiedIcon("MV_NewDomain"));

		this.settingsMenu.add(this.pdbSettingsItem);
		if (CM2_CONSTANTS.GUI_SETTINGS_AND_CONTROLS.DEVELOPER_EDITION)
			this.settingsMenu.add(this.pluginManager);

		this.helpMenu = new JMenu("Help");

		this.contentsMenuItem = new JMenuItem("Contents");
		this.contentsMenuItem.setFont(this.contentsMenuItem.getFont()
				.deriveFont(Font.BOLD,
						this.contentsMenuItem.getFont().getSize()));
		this.aboutMenuItem = new JMenuItem("About Cm2");

		this.contentsMenuItem.setToolTipText(this.getTooltip(7));
		this.aboutMenuItem.setToolTipText(this.getTooltip(16));

		this.helpMenu.add(this.contentsMenuItem);
		this.helpMenu.add(this.aboutMenuItem);

		this.menuBar.add(this.helpMenu);

		// adding access Buttons for themes

		for (int i = 0; i < this.THEMES.length; i++) {
			final JMenuItem item = new JMenuItem(this.THEMES[i]);
			item.addActionListener(this);
			// name is set as an identifier for functionality
			item.setName("theme");
			item.setToolTipText(this.getTooltip(12));
			this.selectTheme.add(item);
		}

		this.pluginManager.addActionListener(this);
		this.pdbSettingsItem.addActionListener(this);
		this.appearanceDarkItem.addActionListener(this);
		this.appearanceBrightItem.addActionListener(this);
		this.newMembrane.addActionListener(this);
		this.loadMembrane.addActionListener(this);
		this.quicksaveMembrane.addActionListener(this);
		this.saveMembraneAs.addActionListener(this);
		this.saveImageAs.addActionListener(this);
		this.generateMembrane.addActionListener(this);
		this.previewMembrane.addActionListener(this);
		this.previewMembrane2.addActionListener(this);
		analyzeMembrane.addActionListener(this);
		this.editAttrs.addActionListener(this);
		this.changeSize.addActionListener(this);
		this.refreshItem.addActionListener(this);
		this.newDomainItem.addActionListener(this);
		this.exit.addActionListener(this);
		this.addMultilayerItem.addActionListener(this);
		this.restoreFromPDBItem.addActionListener(this);
		this.showAlgorithmProtocol.addActionListener(this);

		this.newDB.addActionListener(this);
		this.loadDB.addActionListener(this);
		this.openGlobal.addActionListener(this);

		this.contentsMenuItem.addActionListener(this);
		this.aboutMenuItem.addActionListener(this);

		this.showToolbar.addActionListener(this);

		this.showBordersAndTitles.addActionListener(this);
		
		this.lockgui.addActionListener(this);

		this.proxyItem.addActionListener(this);

		this.newMembrane.setIcon(Cm2_IconDepot
				.getSpecifiedIcon("Toolbar_NewMembrane"));
		this.loadMembrane.setIcon(Cm2_IconDepot
				.getSpecifiedIcon("Toolbar_LoadMembrane"));
		this.quicksaveMembrane.setIcon(Cm2_IconDepot
				.getSpecifiedIcon("Toolbar_QuickSaveMembrane"));
		this.saveMembraneAs.setIcon(Cm2_IconDepot
				.getSpecifiedIcon("Toolbar_SaveMembraneAs"));
		this.saveImageAs.setIcon(Cm2_IconDepot
				.getSpecifiedIcon("Toolbar_SaveImageAs"));
		this.generateMembrane.setIcon(Cm2_IconDepot
				.getSpecifiedIcon("Toolbar_Generate"));
		this.newDB.setIcon(Cm2_IconDepot.getSpecifiedIcon("Toolbar_NewDB"));
		this.loadDB.setIcon(Cm2_IconDepot.getSpecifiedIcon("Toolbar_LoadDB"));
		this.previewMembrane.setIcon(Cm2_IconDepot
				.getSpecifiedIcon("Toolbar_PreviewMembrane"));
		this.previewMembrane2.setIcon(Cm2_IconDepot
				.getSpecifiedIcon("Toolbar_PreviewMembrane2"));
		analyzeMembrane.setIcon(Cm2_IconDepot
				.getSpecifiedIcon("Toolbar_AnalyzeMembrane"));
		this.editAttrs.setIcon(Cm2_IconDepot
				.getSpecifiedIcon("Toolbar_EditAttrs"));
		this.changeSize.setIcon(Cm2_IconDepot
				.getSpecifiedIcon("Toolbar_ChangeSize"));
		this.openGlobal.setIcon(Cm2_IconDepot.getSpecifiedIcon("DB_Rcsb"));
		this.exit.setIcon(Cm2_IconDepot.getSpecifiedIcon("Exit"));
		this.addMultilayerItem.setIcon(Cm2_IconDepot
				.getSpecifiedIcon("Toolbar_Multilayer"));
		this.restoreFromPDBItem.setIcon(Cm2_IconDepot
				.getSpecifiedIcon("Toolbar_RestoreMembrane"));

		this.activateAccelerators(true);
		// setting some values for the layoutbuttons
		for (int i = 0; i < this.layoutMenu.getItemCount(); i++) {
			this.layoutMenu.getItem(i).setName("layout");
			this.layoutMenu.getItem(i).addActionListener(this);
			this.layoutMenu.getItem(i).setToolTipText(this.getTooltip(14));
		}

	}

	/**
	 * En-or disables the accelerator keys on the menuitems
	 * 
	 * @param b
	 */
	private void activateAccelerators(final boolean b) {
		if (b) {

			this.saveMembraneAs.setAccelerator(KeyStroke.getKeyStroke(
					KeyEvent.VK_A, Event.CTRL_MASK));
			this.saveImageAs.setAccelerator(KeyStroke.getKeyStroke(
					KeyEvent.VK_I, Event.CTRL_MASK));
			this.loadMembrane.setAccelerator(KeyStroke.getKeyStroke(
					KeyEvent.VK_L, Event.CTRL_MASK));
			this.restoreFromPDBItem.setAccelerator(KeyStroke.getKeyStroke(
					KeyEvent.VK_B, Event.CTRL_MASK));
			this.newMembrane.setAccelerator(KeyStroke.getKeyStroke(
					KeyEvent.VK_N, Event.CTRL_MASK));
			this.quicksaveMembrane.setAccelerator(KeyStroke.getKeyStroke(
					KeyEvent.VK_S, Event.CTRL_MASK));
			this.generateMembrane.setAccelerator(KeyStroke.getKeyStroke(
					KeyEvent.VK_G, Event.CTRL_MASK));
			analyzeMembrane.setAccelerator(KeyStroke.getKeyStroke(
					KeyEvent.VK_Y, Event.CTRL_MASK));

			this.exit.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_Q,
					Event.CTRL_MASK));

			this.editAttrs.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_E,
					Event.CTRL_MASK));

			this.refreshItem.setAccelerator(KeyStroke.getKeyStroke(
					KeyEvent.VK_R, Event.CTRL_MASK));

			this.previewMembrane.setAccelerator(KeyStroke.getKeyStroke(
					KeyEvent.VK_P, Event.CTRL_MASK));

			this.newDB.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_N,
					Event.SHIFT_MASK));
			this.loadDB.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_L,
					Event.SHIFT_MASK));

			this.changeSize.setAccelerator(KeyStroke.getKeyStroke(
					KeyEvent.VK_C, Event.CTRL_MASK));

			this.contentsMenuItem.setAccelerator(KeyStroke.getKeyStroke(
					KeyEvent.VK_F1, 0));

			this.openGlobal.setAccelerator(KeyStroke.getKeyStroke(
					KeyEvent.VK_F5, 0));
			this.j3dsettings.setAccelerator(KeyStroke.getKeyStroke(
					KeyEvent.VK_F6, 0));

			this.pdbSettingsItem.setAccelerator(KeyStroke.getKeyStroke(
					KeyEvent.VK_F7, 0));

			this.openExternal.setAccelerator(KeyStroke.getKeyStroke(
					KeyEvent.VK_F8, 0));

			this.newDomainItem.setAccelerator(KeyStroke.getKeyStroke(
					KeyEvent.VK_D, Event.CTRL_MASK));

			this.addMultilayerItem.setAccelerator(KeyStroke.getKeyStroke(
					KeyEvent.VK_L, Event.CTRL_MASK));

			this.layoutMenu.getItem(0).setAccelerator(
					KeyStroke.getKeyStroke(KeyEvent.VK_F11, 0));
			this.lockgui.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_F12,
					0));

		} else {

			this.saveMembraneAs.setAccelerator(null);
			// this.saveImageAs.setAccelerator(null); // should be always activated
			this.loadMembrane.setAccelerator(null);
			this.newMembrane.setAccelerator(null);
			this.quicksaveMembrane.setAccelerator(null);
			this.generateMembrane.setAccelerator(null);
			analyzeMembrane.setAccelerator(null);
			this.addMultilayerItem.setAccelerator(null);
			this.refreshItem.setAccelerator(null);
			this.layoutMenu.getItem(0).setAccelerator(null);
			this.j3dsettings.setAccelerator(null);
			this.newDomainItem.setAccelerator(null);

			this.exit.setAccelerator(null);
			this.openExternal.setAccelerator(null);
			this.pdbSettingsItem.setAccelerator(null);
			this.changeSize.setAccelerator(null);
			this.newDB.setAccelerator(null);
			this.loadDB.setAccelerator(null);
			this.openGlobal.setAccelerator(null);
			this.editAttrs.setAccelerator(null);
			this.previewMembrane.setAccelerator(null);
			this.previewMembrane2.setAccelerator(null);

			this.lockgui.setAccelerator(null);

		}
	}

	/**
	 * sets the gui's perspective and settings either as defined in the setting
	 * files or, if not available, to default values
	 */
	private void arrangeGuiSettings() {

		/*
		 * trying to restore the mainwindow bounds. If no proper values are
		 * available or if they wouldn't make sense, Exception will be thrown
		 */
		try {
			/*
			 * if the screen resolution was changed in the OS, window values
			 * received from the former screensize become likely invalid
			 */
			if (Integer.parseInt(ConfigParser.getValue("last_screenSize")[0]) != this.frame
					.getToolkit().getScreenSize().width
					|| Integer.parseInt(ConfigParser
							.getValue("last_screenSize")[1]) != this.frame
							.getToolkit().getScreenSize().height)
				throw new Exception();

			int x = Integer.parseInt(ConfigParser
					.getValue("last_windowLocation")[0]);
			int y = Integer.parseInt(ConfigParser
					.getValue("last_windowLocation")[1]);

			// ensure that frames upper-left corner is in accessible range on
			// screen
			if (x > this.frame.getToolkit().getScreenSize().width
					|| y > this.frame.getToolkit().getScreenSize().height)
				throw new Exception();

			this.frame.setLocation(x, y);

			x = Integer.parseInt(ConfigParser.getValue("last_windowSize")[0]);
			y = Integer.parseInt(ConfigParser.getValue("last_windowSize")[1]);

			// ensure that frames size has acceptable values
			if (x > this.frame.getToolkit().getScreenSize().width
					|| y > this.frame.getToolkit().getScreenSize().height
					|| x == 0 || y == 0)
				throw new Exception();

			this.rootWindow.setPreferredSize(new Dimension(x, y));

		}
		/*
		 * if something goes wrong, default bounds will be set regarding to the
		 * current screen resolution. This counts only for the windowsize and it
		 * doesnt effect the layout itself
		 */
		catch (final Exception e) {
			final Dimension screensize = this.frame.getToolkit()
					.getScreenSize();

			final Dimension prefSize = new Dimension(
					(int) (screensize.width * CM2_CONSTANTS.GUI_SETTINGS_AND_CONTROLS.MAIN_FRAME_SCALE),
					(int) (screensize.height * CM2_CONSTANTS.GUI_SETTINGS_AND_CONTROLS.MAIN_FRAME_SCALE));
			if (CM2_CONSTANTS.GUI_SETTINGS_AND_CONTROLS.MAIN_FRAME_SCALE == 1)
				prefSize.height -= screensize.height / 35;
			this.frame.setPreferredSize(prefSize);
			this.frame.setLocation((screensize.width - prefSize.width) / 2, 0);

		}

		// trying to get the currenttheme
		try {
			this.setTheme(ConfigParser.getValue("currenttheme")[0]);
		} catch (final Exception e) {
			this.setTheme("Shaped Gradient");
		}

		// trying to get the showtoolbar flag, else show toolbar by default
		try {
			this.showToolbar(Boolean.parseBoolean((ConfigParser
					.getValue("showToolbar")[0])));
		} catch (final Exception e) {
			this.showToolbar(true);
		}
		
		// trying to get the showBordersAndTitles flag, else show showBordersAndTitles by default
		try {
			this.showBordersAndTitles(Boolean.parseBoolean((ConfigParser
					.getValue("showBordersAndTitles")[0])));
		} catch (final Exception e) {
			this.showBordersAndTitles(true);
		}

		// trying to get the flag for locking the gui. If not possible, gui
		// remains unlocked
		try {
			this.lockGui(Boolean
					.parseBoolean((ConfigParser.getValue("lockGui")[0])));
		} catch (final Exception e) {
			this.lockGui(false);
		}

		// retrieving the last used projects and adding their links into the
		// file menu
		String[] lastones;
		try {
			lastones = ConfigParser.getValue("Last_Used_Projects");
		} catch (final Exception e) {
			lastones = new String[0];
		}
		for (int i = 0; i < lastones.length; i++) {
			if (new File(lastones[i]).exists()
					&& i < CM2_CONSTANTS.GUI_SETTINGS_AND_CONTROLS.MAX_QUICKLOAD_HISTORY_SIZE)
				this.addQuickloadLink(new File(lastones[i]));
		}

		this
				.setEditorAppearanceCheckHooks(DataPool.J3DSettings.bright_appearance);

		// these buttons and so their options should always be inactive at
		// first

		this.enableMembraneButtons(false);
		this.enableSaveButtons(false);

	}

	/**
	 * Sets only the checkboxIcons in the Editor Appearance Menu. If boolean is
	 * true, the bright mode is set, otherwise Dark mode.
	 * 
	 * @param bright_appearance
	 */
	private void setEditorAppearanceCheckHooks(final boolean bright_appearance) {
		if (bright_appearance) {
			this.appearanceDarkItem.setIcon(Cm2_IconDepot
					.getSpecifiedIcon("Check_Off"));
			this.appearanceBrightItem.setIcon(Cm2_IconDepot
					.getSpecifiedIcon("Check_On"));
		} else {
			this.appearanceDarkItem.setIcon(Cm2_IconDepot
					.getSpecifiedIcon("Check_On"));
			this.appearanceBrightItem.setIcon(Cm2_IconDepot
					.getSpecifiedIcon("Check_Off"));
		}

	}

	/**
	 * trying to retrieve the layout here from the layout file.
	 */
	public void restoreLayout() {

		try {
			this.doLayout(this.layoutFile);
		} catch (final Throwable e) {

			this.doLayout("", false);
			this.focusView(1);

		}

	}

	/**
	 * fetches the current settings and measurements of the gui and stores them
	 * to the ConfigParser. Also tries to save the layout
	 * 
	 */
	private void storeCurrentGuiSettings() {
		// in the next lines, the gui settings are stored to the Config class
		ConfigParser.deleteAllAt("showToolbar");
		ConfigParser.addValue("showToolbar", "" + this.toolbar.isVisible());

		if (this.toolbar.isVisible()) {

			ConfigParser.deleteAllAt("toolbar_Orientation");

			String tborient = "UNDEFINED";

			final Container cp = this.frame.getContentPane();

			if (cp.getComponentCount() > 1) {
				if (cp.getComponentAt(0, 0) == this.toolbar) {
					if (this.toolbar.getOrientation() == JToolBar.HORIZONTAL)
						tborient = "NORTH";
					else
						tborient = "WEST";
				} else {
					if (this.toolbar.getOrientation() == JToolBar.HORIZONTAL)
						tborient = "SOUTH";
					else
						tborient = "EAST";
				}

			} else
				tborient = "UNDEFINED";

			ConfigParser.addValue("toolbar_Orientation", tborient);

		}
		
		ConfigParser.deleteAllAt("showBordersAndTitles");
		ConfigParser.addValue("showBordersAndTitles", "" + this.showBordersAndTitlesIsSelected);

		ConfigParser.deleteAllAt("lockGui");
		ConfigParser.addValue("lockGui", "" + this.isGuilocked);

		ConfigParser.deleteAllAt("currenttheme");
		ConfigParser.addValue("currenttheme", this.currentTheme);

		ConfigParser.deleteAllAt("last_screenSize");
		ConfigParser.deleteAllAt("last_windowSize");
		ConfigParser.deleteAllAt("last_windowLocation");

		ConfigParser.addValue("last_screenSize", ""
				+ this.frame.getToolkit().getScreenSize().width);
		ConfigParser.addValue("last_screenSize", ""
				+ this.frame.getToolkit().getScreenSize().height);

		ConfigParser.addValue("last_windowSize", ""
				+ this.rootWindow.getWidth());
		ConfigParser.addValue("last_windowSize", ""
				+ this.rootWindow.getHeight());

		ConfigParser.addValue("last_windowLocation", ""
				+ this.frame.getLocation().x);
		ConfigParser.addValue("last_windowLocation", ""
				+ this.frame.getLocation().y);

		ConfigParser.deleteAllAt("Last_Used_Projects");

		final File[] lastones = this.getQuickLoadHistory();

		for (int i = 0; i < lastones.length; i++) {
			ConfigParser.addValue("Last_Used_Projects", lastones[i].toString());
		}

		/*
		 * trying to serialize the gui layout and write the data into the layout
		 * file
		 */
		try {
			// trying to create the path for the layout file if not existing yet
			if (!this.layoutFile.exists()) {
				final File tmp = new File(this.layoutFile.getParent());
				if (!tmp.isDirectory()) {
					if (!tmp.mkdirs()) {
						throw new IOException();
					}
				}
			}
			final ByteArrayOutputStream bos = new ByteArrayOutputStream();
			final ObjectOutputStream out = new ObjectOutputStream(bos);

			/*
			 * rootwindow writes in the Object output Stream. the OoS writes
			 * object data into Byta Array Output Stream
			 */
			this.rootWindow.write(out, true);
			out.close();

			// creates a Fileoutput Stream and writes the data bytes to the
			// layoutfile
			final FileOutputStream fos = new FileOutputStream(this.layoutFile);
			fos.write(bos.toByteArray());

			fos.close();

		} catch (final Exception e) {
			/*
			 * the layout wont be saved and the programm will have standard
			 * layout at next startup probably.
			 */
			System.err.println("Error while saving the Gui's Layout!");
		}

	}

	/**
	 * changes the gui's theme. if the given key(ie. the title of one of the
	 * themes menuItems) doesnt appoint to a theme in the Themes-hashtable,
	 * calling this method will have no effect
	 * 
	 * @param text
	 *            - the key for the particular theme
	 */
	@SuppressWarnings("deprecation")
	private void setTheme(final String text) {

		// replacing the current theme with requested one

		final DockingWindowsTheme dwt = this.createDWTheme(text);
		if (dwt != null) {
			this.rootwinprops.removeSuperObject();
			this.rootwinprops.addSuperObject(dwt.getRootWindowProperties());

			// setting the menuitems icon,according to the active theme
			for (int i = 0; i < this.THEMES.length; i++) {

				if (this.THEMES[i].equals(text))
					this.selectTheme.getItem(i).setIcon(
							Cm2_IconDepot.getSpecifiedIcon("Check_On"));
				else
					this.selectTheme.getItem(i).setIcon(
							Cm2_IconDepot.getSpecifiedIcon("Check_Off"));
				// updates the currenttheme field

			}
			this.currentTheme = text;
		}

	}

	/**
	 * creates a dockingWindowsTheme by the passed string
	 * 
	 * @param name
	 * @return
	 */
	private DockingWindowsTheme createDWTheme(final String name) {

		if (name.equals("Slim Flat")) {
			return new SlimFlatDockingTheme();
		} else if (name.equals("Gradient")) {
			return new GradientDockingTheme();
		} else if (name.equals("Shaped Gradient")) {
			return new ShapedGradientDockingTheme();
		} else if (name.equals("Ice")) {
			return new SoftBlueIceDockingTheme();
		} else if (name.equals("Classic")) {
			return new ClassicDockingTheme();
		} else

			return null;
	}

	/**
	 * freezes or unfreezes the gui layout
	 * 
	 * @param bool
	 *            det. wether or not the gui should freeze
	 */
	private void lockGui(final boolean bool) {

		this.isGuilocked = bool;
		// dis- or enable the features making the gui flexible
		this.rootwinprops.getDockingWindowProperties().setDragEnabled(!bool);
		this.rootwinprops.getDockingWindowProperties()
				.setMinimizeEnabled(!bool);
		this.rootwinprops.getDockingWindowProperties().setRestoreEnabled(!bool);
		this.rootwinprops.getDockingWindowProperties()
				.setMaximizeEnabled(!bool);
		this.rootwinprops.getDockingWindowProperties().setUndockEnabled(!bool);
		this.rootwinprops.getDockingWindowProperties().setDockEnabled(!bool);

		// updating the Lock-Buttons icon
		if (this.isGuilocked) {
			this.lockgui.setIcon(Cm2_IconDepot.getSpecifiedIcon("Check_On"));
		} else
			this.lockgui.setIcon(Cm2_IconDepot.getSpecifiedIcon("Check_Off"));

	}

	/**
	 * Switches visibility of the toolbar on or off
	 * 
	 * @param bool
	 *            - det wether or not the toolbar should be visible
	 */
	public void showToolbar(final boolean bool) {
		
		this.toolbar.setVisible(bool);

		// updating icon of the toolbar Button
		if (bool)
			this.showToolbar
					.setIcon(Cm2_IconDepot.getSpecifiedIcon("Check_On"));
		else
			this.showToolbar.setIcon(Cm2_IconDepot
					.getSpecifiedIcon("Check_Off"));

	}

	/**
	 * Switches visibility of borders and titles on or off
	 * 
	 * @param bool
	 *            - det wether or not the toolbar should be visible
	 */
	public void showBordersAndTitles(final boolean bool) {
		
		this.toggleAllTitlesAndBorders(bool);

		this.showBordersAndTitlesIsSelected = bool;
		
		// updating icon of the toolbar Button
		if (bool)
			this.showBordersAndTitles
					.setIcon(Cm2_IconDepot.getSpecifiedIcon("Check_On"));
		else
			this.showBordersAndTitles.setIcon(Cm2_IconDepot
					.getSpecifiedIcon("Check_Off"));

		membraneViewListener.makeToolbarsFloatable(!bool);
		
	}
	
	/**
	 * tries to accomplish layout by re-serializing from a file
	 * 
	 * @param layoutfile
	 *            - the file containing the requested layout
	 * @throws IOException
	 */
	private void doLayout(final File layoutfile) throws IOException {

		final FileInputStream fis = new FileInputStream(layoutfile);

		/*
		 * creating a new array of bytes with the length of the number of bytes
		 * in the file
		 */
		final byte[] bytes = new byte[(int) layoutfile.length()];

		// Read in the bytes
		int offset = 0;
		int numRead = 0;
		while (offset < bytes.length
				&& (numRead = fis.read(bytes, offset, bytes.length - offset)) >= 0) {
			offset += numRead;
		}
		fis.close();

		this.rootWindow.read(new ObjectInputStream(new ByteArrayInputStream(
				bytes)));

	}

	/**
	 * applies a predefined and hardcoded layout by name. Also provides the
	 * option to let the user confirm the layout change. Needs the exact name of
	 * the layout(is usually also the name on this layout's menuitem). If any
	 * other String is passed, the default layout will be set.
	 * 
	 * @param whichone
	 *            - the name of the layout
	 * 
	 * @param confirmLayout
	 *            - if true, user is forced to confirm
	 */
	private void doLayout(final String whichone, final boolean confirmLayout) {

		// do the cm2.1classic layout
		if (whichone.equals("Cm2.1 Classic")) {
			int val;
			// if confirming is set, a dialog opens here
			if (confirmLayout)
				val = JOptionPane.showConfirmDialog(this.frame,
						"Do you really want to discard "
								+ "your current view setting?",
						"Reset Layout?", JOptionPane.YES_NO_OPTION);
			else
				// hotwires the confirmoperation
				val = JOptionPane.OK_OPTION;

			if (val == JOptionPane.OK_OPTION) {

				this.rootWindow.setWindow(new SplitWindow(true, 0.2262324f,
						this.viewmap.getView(0), new TabWindow(
								new DockingWindow[] { this.viewmap.getView(1),
										this.viewmap.getView(2),
										this.viewmap.getView(3),
										this.viewmap.getView(4) })));

				this.viewmap.getView(1).restoreFocus();

			}
			return;
		}

		// do the second layout
		else if (whichone.equals("Widescreen")) {

			int val;
			// if confirming is set, a dialog opens here
			if (confirmLayout)
				val = JOptionPane.showConfirmDialog(this.frame,
						"Do you really want to discard "
								+ "your current view setting?",
						"Reset Layout?", JOptionPane.YES_NO_OPTION);
			else
				// hotwires the confirmoperation
				val = JOptionPane.OK_OPTION;

			if (val == JOptionPane.OK_OPTION) {

				this.rootWindow.setWindow(new SplitWindow(true, 0.667364f,
						new SplitWindow(true, 0.1595912f, this.viewmap
								.getView(0), this.viewmap.getView(2)),
						new SplitWindow(false, 0.44386694f, new SplitWindow(
								true, 0.49525318f, this.viewmap.getView(4),
								this.viewmap.getView(3)), this.viewmap
								.getView(1))));
				//
			}
			return;
		}

		// do the defaultLayout
		else {

			int val;
			if (confirmLayout)
				val = JOptionPane.showConfirmDialog(this.frame,
						"Do you really want to discard "
								+ "your current view setting?",
						"Reset Layout?", JOptionPane.YES_NO_OPTION);
			else
				val = JOptionPane.OK_OPTION;

			if (val == JOptionPane.OK_OPTION) {

				this.rootWindow
						.setWindow(new SplitWindow(
								false,
								0.78f,
								new SplitWindow(
										true,
										0.8f,
										new SplitWindow(
												true,
												0.2f,
												this.viewmap.getView(0),
												new TabWindow(
														new DockingWindow[] { this.viewmap
																.getView(2), })),
										new SplitWindow(false, 0.45f,

										this.viewmap.getView(3), this.viewmap
												.getView(4))), this.viewmap
										.getView(1)));

				this.viewmap.getView(2).restoreFocus();
				this.viewmap.getView(3).restoreFocus();
			}
			return;
		}

	}

	/**
	 * set the location of the layout file. if none exists, one will be created
	 * 
	 * @param file
	 */
	public void setLayoutFile(final File file) {
		this.layoutFile = file;
	}

	/**
	 * en- or disable the option to save the membrane
	 * 
	 * @param bool
	 */
	public void enableSaveButtons(final boolean bool) {
		this.saveMembraneAs.setEnabled(bool);
		if (bool) // should be always activated
			this.saveImageAs.setEnabled(bool);
		this.saveMembraneAsBtn.setEnabled(bool);
		if (bool) // should be always activated
			this.saveImageAsBtn.setEnabled(bool);
		this.quicksaveMembrane.setEnabled(bool);
		this.quicksaveMembraneBtn.setEnabled(bool);
	}

	/**
	 * en- or disable some mebrane options. Those are for example disabled when
	 * the membrane is empty
	 * 
	 * @param bool
	 */
	public void enableMembraneButtons(final boolean bool) {
		this.generateMembrane.setEnabled(bool);
		this.generateMembraneBtn.setEnabled(bool);
		this.editAttrsBtn.setEnabled(bool);
		this.changeSize.setEnabled(bool);
		this.loadDBBtn.setEnabled(bool);
		this.newDBBtn.setEnabled(bool);
		this.previewMembraneBtn.setEnabled(bool);
		this.previewMembraneBtn2.setEnabled(bool);
		if (APLVoroSettings.aplVoroExecutionLink != null) {
			if (!APLVoroSettings.aplVoroExecutionLink.isEmpty()) {
				analyzeMembrane.setEnabled(bool);
				analyzeMembraneBtn.setEnabled(bool);
			} else {
				analyzeMembrane.setEnabled(false);
				analyzeMembraneBtn.setEnabled(false);
				analyzeMembrane.setVisible(false);
				analyzeMembraneBtn.setVisible(false);
			}
		} else {
			analyzeMembrane.setEnabled(false);
			analyzeMembraneBtn.setEnabled(false);
			analyzeMembrane.setVisible(false);
			analyzeMembraneBtn.setVisible(false);
		}
		this.editAttrsBtn.setEnabled(bool);
		this.changeSizeBtn.setEnabled(bool);
		this.previewMembrane.setEnabled(bool);
		this.previewMembrane2.setEnabled(bool);
		this.pdbSettingsBtn.setEnabled(bool);
		this.refreshBtn.setEnabled(bool);
		this.refreshItem.setEnabled(bool);
		this.newDomainBtn.setEnabled(bool);
		this.newDomainItem.setEnabled(bool);
		this.addMultilayerItem.setEnabled(bool);
		this.addMultilayerBtn.setEnabled(bool);
	}

	/**
	 * Adds the GuiListener, that is needed to invoke Actions in the parent class
	 * of this gui
	 * 
	 * @param listener
	 */
	public void addGuiListener(final GuiListener listener) {
		this.guiListener = listener;
	}
	
	/**
	 * adds the MembraneViewListener, that is needed to invoke Actions in the parent class
	 * of this gui
	 * 
	 * @param listener
	 */
	public void addMembraneViewListener(final MembraneViewListener listener) {
		this.membraneViewListener = listener;
	}

	/**
	 * For a better overview, the Tooltips are defined in this method and they
	 * can be accessed by their index
	 * 
	 * @param which
	 *            - number of the requested tooltip
	 * @return the tooltip
	 */
	public String getTooltip(final int which) {

		final String[] tooltips = {
				/* 0 */"Create a new Membrane",
				"Open an existing Membrane...",
				"Save the current Membrane as...",
				"Create a new Database",
				"Open a Database...",
				/* 5 */"@deprecated: Save the current Database as...",
				"Generates a PDB-File of the current Membrane",
				"Open the Help Contents",
				"Toolbar",
				"Exit the Application",
				/* 10 */"Toggles the Toolbar On/Off",
				"Freezes the Window Layout, so that it cannot be adjusted accidently",
				"Select a Docking Window Theme",
				"@deprecated: Toggle a View On/Off",
				"Restore to a predefined Layout",
				/* 15 */"Show an Introduction",
				"Show the Credits and some Facts",
				"Quicksave the current Membrane",
				"Preview the current Membrane in Jmol", 
				"PDB Attributes",
				/* 20 */"Add or change a MembranePackingAlgorithm Plug-In",
				"Change the Size of the current Membrane",
				"Set up some parameters concerning the pdb Format",
				"Reload all 3D Objects in the Membrane",
				"Define a new Microdomain", 
				/* 25 */"Toggle the Stereo Mode On/Off",
				"Toggle the FullScreen Mode On/Off",
				"Change the Eye Distance for the Stereo Mode",
				"Preview the current Membrane's textual contents",
				"Show the Algorithm Protocol",
				/* 30 */"Make a Screenshot of the current screen and save is as a JPG file",
				"Analyze the current Membrane in APL@Voro",
				"Show Borders and Titles (deactivate this in StereoView mode)"};
		// makes sure the requested index exists, else return "Tooltip not
		// found"
		if (which < 0 || which >= tooltips.length)
			return "NO TOOLTIP FOUND!";
		else
			return tooltips[which];

	}

	/**
	 * the common implementation, where actionEvents are handled
	 */
	public void actionPerformed(final ActionEvent arg0) {

		// stores the gui settings on exit and tells the programm to initialize
		// exit
		if (arg0.getSource() == this.exit) {

			this.storeCurrentGuiSettings();
			if (this.guiListener != null)
				this.guiListener.exitApplication();

		}

		// invokes actions for the menuitems put in dynamically without
		// references
		if (arg0.getSource() instanceof MyMenuItem) {

			if (this.guiListener != null) {
				final File file = ((MyMenuItem) arg0.getSource()).getFile();

				if (this.isEnabled())
					this.guiListener.userChoseQuickLoadMembrane(file);
			}

		}

		if (arg0.getSource() instanceof JMenuItem) {
			final JMenuItem item = (JMenuItem) arg0.getSource();

			if (item.getName() != null) {
				if (item.getName().equals("theme"))
					this.setTheme(item.getText());
				else if (item.getName().equals("layout"))
					this.doLayout(item.getText(), true);
			}

		}

		// toggle toolbar and freeze

		if (arg0.getSource() == this.showToolbar) {
			this.showToolbar(!this.toolbar.isVisible());

		} else if (arg0.getSource() == this.showBordersAndTitles) {
			this.showBordersAndTitles(!showBordersAndTitlesIsSelected);

		} else if (arg0.getSource() == this.lockgui) {
			this.lockGui(!this.isGuilocked);

		}

		// summarizes buttons with same function to perform the same

		else if (arg0.getSource() == this.newMembrane
				|| arg0.getSource() == this.newMembraneBtn) {
			if (this.guiListener != null)
				this.guiListener.userChoseNewMembrane();
		} else if (arg0.getSource() == this.loadMembrane
				|| arg0.getSource() == this.loadMembraneBtn) {
			if (this.guiListener != null)
				this.guiListener.userChoseLoadMembrane();

		} else if (arg0.getSource() == this.quicksaveMembrane
				|| arg0.getSource() == this.quicksaveMembraneBtn) {
			if (this.guiListener != null)
				this.guiListener.userChoseQuickSaveMembrane();

		} else if (arg0.getSource() == this.saveMembraneAs
				|| arg0.getSource() == this.saveMembraneAsBtn) {
			if (this.guiListener != null)
				this.guiListener.userChoseSaveMembrane();

		} else if (arg0.getSource() == this.saveImageAs
				|| arg0.getSource() == this.saveImageAsBtn) {
			if (this.guiListener != null)
				this.guiListener.userChoseSaveImage();

		} else if (arg0.getSource() == this.newDB
				|| arg0.getSource() == this.newDBBtn) {
			if (this.guiListener != null)
				this.guiListener.userChoseNewDB();
		} else if (arg0.getSource() == this.loadDB
				|| arg0.getSource() == this.loadDBBtn) {
			if (this.guiListener != null)
				this.guiListener.userchoseLoadDB();

		} else if (arg0.getSource() == this.openGlobal) {
			if (this.guiListener != null)
				this.guiListener.userClickedOpenRCSB();

		} else if (arg0.getSource() == this.proxyItem) {
			new SetUpConnectionsDialog(this.guiListener).setVisible(true);

		} else if (arg0.getSource() == this.openExternal) {
			if (this.guiListener != null)
				this.guiListener.userClickedOpenExternal();

		} else if (arg0.getSource() == this.testItem) {
			try {
				TestArea.execute();
			} catch (final Exception e) {
				e.printStackTrace();
			}
		} else if (arg0.getSource() == this.showAlgorithmProtocol) {
			if (this.guiListener != null)
				this.guiListener.userChoseShowTimeProtocol();

		} else if (arg0.getSource() == this.appearanceDarkItem) {

			if (this.guiListener != null)
				this.guiListener.userClickedEditorMode(false);
			this.setEditorAppearanceCheckHooks(false);

		} else if (arg0.getSource() == this.appearanceBrightItem) {
			if (this.guiListener != null)
				this.guiListener.userClickedEditorMode(true);
			this.setEditorAppearanceCheckHooks(true);

		} else if (arg0.getSource() == this.j3dsettings) {
			if (this.guiListener != null)
				this.guiListener.userClicked3DSettings(null);

		} else if (arg0.getSource() == this.refreshBtn
				|| arg0.getSource() == this.refreshItem) {
			if (this.guiListener != null)
				this.guiListener.reloadGeometry();

		} else if (arg0.getSource() == this.newDomainBtn
				|| arg0.getSource() == this.newDomainItem) {
			if (this.guiListener != null)
				this.guiListener.startDomainMode(null);

		} else if (arg0.getSource() == this.addMultilayerBtn
				|| arg0.getSource() == this.addMultilayerItem) {
			if (this.guiListener != null)
				this.guiListener.addMultilayerPressed();

		} else if (arg0.getSource() == this.restoreFromPDBItem) {
			if (this.guiListener != null)
				this.guiListener.restoreMembraneFromPDBPressed();

		}

		else if (arg0.getSource() == this.previewMembrane
				|| arg0.getSource() == this.previewMembraneBtn) {
			if (this.guiListener != null)
				this.guiListener.userChosePreviewMembrane(0);

		}
		else if (arg0.getSource() == this.previewMembrane2
				|| arg0.getSource() == this.previewMembraneBtn2) {
			if (this.guiListener != null)
				this.guiListener.userChosePreviewMembrane(1);
		
		}
		else if (arg0.getSource() == analyzeMembrane 
				|| arg0.getSource() == analyzeMembraneBtn) {
				if (this.guiListener != null)
					this.guiListener.userChoseAnalyzeMembrane();
			
		}else if (arg0.getSource() == this.editAttrs
				|| arg0.getSource() == this.editAttrsBtn) {
			if (this.guiListener != null)
				this.guiListener.popUpAttrsDialog();

		} else if (arg0.getSource() == this.changeSize
				|| arg0.getSource() == this.changeSizeBtn) {
			Point location = null;
			if (arg0.getSource() == this.changeSizeBtn)
				location = this.changeSizeBtn.getLocationOnScreen();

			if (this.guiListener != null)
				this.guiListener.popUpChangeSize(location);

		}

		else if (arg0.getSource() == this.generateMembrane
				|| arg0.getSource() == this.generateMembraneBtn) {
			if (this.guiListener != null)
				this.guiListener.userChoseGeneratePDBFile(null);

		} else if (arg0.getSource() == this.contentsMenuItem
				|| arg0.getSource() == this.helpBtn) {

			new HelpDialog().popupHelp();

		} else if (arg0.getSource() == this.aboutMenuItem) {
			new AboutPane();

		} else if (arg0.getSource() == this.pluginManager) {
			new AlgorithmWizzard(this.guiListener.getPluginManager(), null);
		} else if (arg0.getSource() == this.pdbSettingsItem
				|| arg0.getSource() == this.pdbSettingsBtn) {
			this.guiListener.userClickedOpenRestrictionDialog();
//		} else if (arg0.getSource() == this.stereoEnable) {
//			this.listener.userChangedStereoEnable(this.stereoEnable
//					.isSelected());
		}
	}

	public void stateChanged(final ChangeEvent e) {
		if (e.getSource() == this.stereoSlider) {
			if (this.guiListener != null) {
				this.guiListener.userChangedStereoSlider();
			}
		}
	}

	/**
	 * Adds a quickloadlink to the filemenu. Kicks out the oldest one if the
	 * history gets larger than allowed
	 * 
	 * @param file
	 */
	public void addQuickloadLink(final File file) {

		int mymenuitemcount = 0;
		final int toinsert = MENU_POSITION_TO_ADD_USED_FILES;

		final MyMenuItem item = new MyMenuItem(file);
		item.addActionListener(this);

		for (int i = 0; i < this.fileMenu.getItemCount(); i++) {
			if (this.fileMenu.getItem(i) instanceof MyMenuItem) {
				final MyMenuItem myitem = (MyMenuItem) this.fileMenu.getItem(i);
				if (myitem.getFile().getPath().equals(file.getPath())) {
					this.fileMenu.remove(i);
					this.fileMenu.insert(item, toinsert);
					return;
				}
			}
		}

		this.fileMenu.insert(item, toinsert);

		for (int i = 0; i < this.fileMenu.getItemCount(); i++) {
			if (this.fileMenu.getItem(i) instanceof MyMenuItem) {
				mymenuitemcount++;
				if (mymenuitemcount > CM2_CONSTANTS.GUI_SETTINGS_AND_CONTROLS.MAX_QUICKLOAD_HISTORY_SIZE) {
					this.fileMenu.remove(i);
				}
			}
		}

	}
	
	/**
	 * (De)Activates all quickload links.
	 * 
	 * @param boolean activate
	 */
	public void activateQuickloadLinks(final boolean activate) {

		for (int i = MENU_POSITION_TO_ADD_USED_FILES; i < this.fileMenu.getItemCount(); i++) {
			if (this.fileMenu.getItem(i) instanceof MyMenuItem) {
				this.fileMenu.getItem(i).setEnabled(activate);
			}
		}
		
	}

	/**
	 * Returns all files that are in the quickloadhistory in the file menu
	 * 
	 * @return
	 */
	private File[] getQuickLoadHistory() {

		final Vector<File> v = new Vector<File>();
		for (int i = 0; i < this.fileMenu.getItemCount(); i++) {
			if (this.fileMenu.getItem(i) instanceof MyMenuItem) {
				v.add(((MyMenuItem) this.fileMenu.getItem(i)).getFile());
			}

		}
		return v.toArray(new File[0]);
		
	}

	/**
	 * Returns the Frame
	 * 
	 * @return
	 */
	public JFrame getFrame() {
		return this.frame;
	}

	/**
	 * sets the specified String as the Mainframe's Title. Sets the defaulttitle
	 * if the parameter is null
	 * 
	 * @param title
	 */
	public void setMainFrameTitle(final String title) {
		if (title != null)
			this.frame.setTitle(title);
		else
			this.frame.setTitle(this.defaultTitle);
	}

	/**
	 * sets the default Title for the MainFrame
	 * 
	 * @param defaultTitle
	 */
	public void setDefaultTitle(final String defaultTitle) {
		this.defaultTitle = defaultTitle;
	}

	/**
	 * A small extension of a JMenuItem that can store a File
	 * 
	 * @author Tim Dingersen dackelnose@gmx.net
	 * 
	 */
	@SuppressWarnings("serial")
	private class MyMenuItem extends JMenuItem {
		private final File file;

		public MyMenuItem(final File file) {
			this.setIcon(Cm2_IconDepot.getSpecifiedIcon("FC_Membrane"));
			this.file = file;
			this.setText(file.getName());
		}

		public File getFile() {
			return this.file;
		}
	}

	/**
	 * Tests whether the passed view is maximized or not
	 * 
	 * @param view
	 * @return
	 */
	public boolean isMaximized(final int view) {
		for (int i = 0; i < this.viewmap.getViewCount(); i++) {
			if (i != view && this.viewmap.getView(i).isShowing()) {
				return false;
			}
		}

		return true;
	}

	/**
	 * En or disables the generate options
	 * 
	 * @param b
	 */
	public void enableGenerateButton(final boolean b) {
		this.generateMembrane.setEnabled(b);
		this.generateMembraneBtn.setEnabled(b);
		this.previewMembrane.setEnabled(b);
		this.previewMembrane2.setEnabled(b);
		this.previewMembraneBtn.setEnabled(b);
		this.previewMembraneBtn2.setEnabled(b);
		if (APLVoroSettings.aplVoroExecutionLink != null) {
			if (!APLVoroSettings.aplVoroExecutionLink.isEmpty()) {
				analyzeMembrane.setEnabled(b);
				analyzeMembraneBtn.setEnabled(b);
			} else {
				analyzeMembrane.setEnabled(false);
				analyzeMembraneBtn.setEnabled(false);
				analyzeMembrane.setVisible(false);
				analyzeMembraneBtn.setVisible(false);
			}
		} else {
			analyzeMembrane.setEnabled(false);
			analyzeMembraneBtn.setEnabled(false);
			analyzeMembrane.setVisible(false);
			analyzeMembraneBtn.setVisible(false);
		}

		if (DataPool.getMembraneModel().isEmpty()) {
			this.refreshBtn.setEnabled(false);
			this.refreshItem.setEnabled(false);
		} else {
			this.refreshBtn.setEnabled(true);
			this.refreshItem.setEnabled(true);
		}
	}

	/**
	 * En-or disables all toolbar and menubar buttons
	 * 
	 * @param b
	 */
	public void setEnabled(final boolean b) {
		// for (int i = 0; i < this.toolbar.getComponentCount(); i++) {
		// this.toolbar.getComponent(i).setEnabled(b);
		// }
		//
		// for (int i = 0; i < this.menuBar.getComponentCount(); i++) {
		// this.menuBar.getComponent(i).setEnabled(b);
		// }

		this.enabled = b;
		this.activateAccelerators(b);

		this.newMembrane.setEnabled(b);
		this.saveMembraneAs.setEnabled(b);
		this.quicksaveMembrane.setEnabled(b);
		if (b) // should be always activated
			this.saveImageAs.setEnabled(b);
		this.loadMembrane.setEnabled(b);
		this.generateMembrane.setEnabled(b);
		this.previewMembrane.setEnabled(b);
		this.previewMembrane2.setEnabled(b);
		this.editAttrs.setEnabled(b);
		this.changeSize.setEnabled(b);
		// contentsMenuItem.setEnabled(b);
		this.refreshItem.setEnabled(b);
		this.j3dsettings.setEnabled(b);
		this.pluginManager.setEnabled(b);
		this.newDomainItem.setEnabled(b);
		this.addMultilayerItem.setEnabled(b);
		this.restoreFromPDBItem.setEnabled(b);

		this.newMembraneBtn.setEnabled(b);
		this.saveMembraneAsBtn.setEnabled(b);
		if (b) // should be always activated
			this.saveImageAsBtn.setEnabled(b);
		this.quicksaveMembraneBtn.setEnabled(b);
		this.loadMembraneBtn.setEnabled(b);
		this.generateMembraneBtn.setEnabled(b);
		this.previewMembraneBtn.setEnabled(b);
		this.previewMembraneBtn2.setEnabled(b);
		if (APLVoroSettings.aplVoroExecutionLink != null) {
			if (!APLVoroSettings.aplVoroExecutionLink.isEmpty()) {
				analyzeMembrane.setEnabled(b);
				analyzeMembraneBtn.setEnabled(b);
			} else {
				analyzeMembrane.setEnabled(false);
				analyzeMembraneBtn.setEnabled(false);
				analyzeMembrane.setVisible(false);
				analyzeMembraneBtn.setVisible(false);
			}
		} else {
			analyzeMembrane.setEnabled(false);
			analyzeMembraneBtn.setEnabled(false);
			analyzeMembrane.setVisible(false);
			analyzeMembraneBtn.setVisible(false);
		}
		
		this.changeSizeBtn.setEnabled(b);
		this.refreshBtn.setEnabled(b);
		this.newDomainBtn.setEnabled(b);
		this.addMultilayerBtn.setEnabled(b);

		this.loadDBBtn.setEnabled(b);
		this.newDBBtn.setEnabled(b);
		this.editAttrsBtn.setEnabled(b);
		this.pdbSettingsBtn.setEnabled(b);
		
		// this.fileMenu.setEnabled(b);  // should be always activated
		this.databaseMenu.setEnabled(b);
		this.viewMenu.setEnabled(b);
		this.settingsMenu.setEnabled(b);
		this.extrasMenu.setEnabled(b);
		this.helpMenu.setEnabled(b);
		
		this.activateQuickloadLinks(b);
		
		this.exit.setEnabled(b);
		
		if (b) {
			this.frame.removeWindowListener(this.windowCloserBusy);
			this.frame.addWindowListener(this.windowCloserStandard);
		} else {
			this.frame.removeWindowListener(this.windowCloserStandard);
			this.frame.addWindowListener(this.windowCloserBusy);
		}

	}

	/**
	 * maximizes the specified view
	 * 
	 * @param i
	 */
	public void maximizeView(final int i) {
		this.viewmap.getView(i).maximize();

	}

	/**
	 * Restores the specified view
	 * 
	 * @param i
	 */
	public void restoreView(final int i) {
		this.viewmap.getView(i).restore();

	}

	public static boolean isDark() {
		return UIManager.getLookAndFeel() != null
				&& UIManager.getLookAndFeel().getName().toLowerCase().contains(
						"ravengraphite");
	}

	public static void setDarkLookAndFeel() {
		if (DataPool.useSystemLookandFeel)
			return;
		try {
			UIManager
					.setLookAndFeel("org.jvnet.substance.skin.SubstanceRavenGraphiteLookAndFeel");
		} catch (final Exception e) {

			try {

				UIManager.setLookAndFeel(UIManager
						.getSystemLookAndFeelClassName());

			} catch (final Exception e1) {

			}
		}

	}

	public static void setBrightLookAndFeel() {
		if (DataPool.useSystemLookandFeel)
			return;

		try {
			UIManager
					.setLookAndFeel("org.jvnet.substance.skin.SubstanceNebulaLookAndFeel");
		} catch (final Exception e) {

			try {

				UIManager.setLookAndFeel(UIManager
						.getSystemLookAndFeelClassName());

			} catch (final Exception e1) {

			}
		}

	}
	
	public static void setupAnalyzeMembraneButton() {
		if (APLVoroSettings.aplVoroExecutionLink != null) {
			if (!APLVoroSettings.aplVoroExecutionLink.isEmpty()) {
				analyzeMembrane.setEnabled(true);
				analyzeMembraneBtn.setEnabled(true);
				analyzeMembrane.setVisible(true);
				analyzeMembraneBtn.setVisible(true);
			} else {
				analyzeMembrane.setEnabled(false);
				analyzeMembraneBtn.setEnabled(false);
				analyzeMembrane.setVisible(false);
				analyzeMembraneBtn.setVisible(false);
			}
		} else {
			analyzeMembrane.setEnabled(false);
			analyzeMembraneBtn.setEnabled(false);
			analyzeMembrane.setVisible(false);
			analyzeMembraneBtn.setVisible(false);
		}
	}
	
	/**
	 * Initialize the borderless Layout, required especially for the StereoView.
	 */
	public void initilizeBorderlessLayout() {

		// create the properties to hide the borders of all tab windows
		hideTabTitleStyleProperties = PropertiesUtil.createTitleBarStyleRootWindowProperties();
		
		hideTabTitleStyleProperties.getComponentProperties().setBorder(BorderFactory.createEmptyBorder());
		hideTabTitleStyleProperties.getComponentProperties().setInsets(new Insets(0,0,0,0));
		
		hideTabTitleStyleProperties.getTabWindowProperties().getTabbedPanelProperties().getContentPanelProperties().getComponentProperties().setBorder(BorderFactory.createEmptyBorder());
		hideTabTitleStyleProperties.getTabWindowProperties().getTabbedPanelProperties().getContentPanelProperties().getComponentProperties().setInsets(new Insets(0,0,0,0));
		hideTabTitleStyleProperties.getTabWindowProperties().getTabbedPanelProperties().setShadowBlendAreaSize(0);
		
		hideTabTitleStyleProperties.getTabWindowProperties().getTabProperties().getTitledTabProperties().getNormalProperties().setDirection(Direction.LEFT);
		hideTabTitleStyleProperties.getTabWindowProperties().getTabProperties().getTitledTabProperties().getNormalProperties().setTextVisible(false);
		hideTabTitleStyleProperties.getTabWindowProperties().getTabProperties().getTitledTabProperties().getNormalProperties().setTextTitleComponentGap(0);
		hideTabTitleStyleProperties.getTabWindowProperties().getTabProperties().getNormalButtonProperties().getCloseButtonProperties().setVisible(false);
		hideTabTitleStyleProperties.getTabWindowProperties().getTabProperties().getNormalButtonProperties().getDockButtonProperties().setVisible(false);
		hideTabTitleStyleProperties.getTabWindowProperties().getTabProperties().getNormalButtonProperties().getMinimizeButtonProperties().setVisible(false);
		hideTabTitleStyleProperties.getTabWindowProperties().getTabProperties().getNormalButtonProperties().getRestoreButtonProperties().setVisible(false);
		hideTabTitleStyleProperties.getTabWindowProperties().getTabProperties().getNormalButtonProperties().getUndockButtonProperties().setVisible(false);
		hideTabTitleStyleProperties.getTabWindowProperties().getTabProperties().getTitledTabProperties().setBorderSizePolicy(TitledTabBorderSizePolicy.INDIVIDUAL_SIZE);
		
		hideTabTitleStyleProperties.getViewProperties().getViewTitleBarProperties().setContentTitleBarGap(0);
		hideTabTitleStyleProperties.getViewProperties().getViewTitleBarProperties().getNormalProperties().setTitleVisible(false);
		hideTabTitleStyleProperties.getViewProperties().getViewTitleBarProperties().setVisible(false);
		
		hideTabTitleStyleProperties.getWindowAreaProperties().setBorder(BorderFactory.createEmptyBorder());
		hideTabTitleStyleProperties.getWindowAreaProperties().setInsets(new Insets(0,0,0,0));

	}
	
	/**
	 * Hide or show all titles and borders.
	 */
	public void toggleAllTitlesAndBorders(boolean showTitles) {
		if (!showTitles) {
			// Enable title bar style
			rootWindow.getRootWindowProperties().addSuperObject(hideTabTitleStyleProperties);
			this.viewmap.getView(1).getWindowParent().setBorder(new TabComponentNoBorder());
			this.viewmap.getView(1).getWindowParent().getInsets().set(0, 0, 0, 0);
		} else {
			// Disable title bar style
			rootWindow.getRootWindowProperties().removeSuperObject(hideTabTitleStyleProperties);
		}
	}
	
	/**
	 * This class prevents borders from being painted.
	 */
	public class TabComponentNoBorder implements ShapedBorder {

		public Insets getBorderInsets(Component c) {
			return new Insets(0, 0, 0, 0);
		}

		public boolean isBorderOpaque() {
			return true;
		}

		public void paintBorder(Component c, Graphics g, int x, int y, int width, int height) {
		      Graphics2D g2d = (Graphics2D) g;

		      // Get the shape we are going to paint
		      Shape shape = getShape(c, x, y, width, height);

		      g2d.setColor(Color.BLACK);
		      g2d.setStroke(new BasicStroke(0f));
		      g2d.draw(shape);
			
		}

		public Shape getShape(Component arg0, int x, int y, int width, int height) {
			RoundRectangle2D border = new RoundRectangle2D.Double(x, y,
					 0, 0, 0d, 0d);
			 return border;
		}
	}

	public boolean isEnabled() {
		return this.enabled;
	}
}
