/*******************************************************************************
 * CELLmicrocosmos 2.2 MembraneEditor (version 2.2.2_2)
 *
 * Copyright (c) 2005, 2017 The CELLmicrocosmos 2 Development Team
 * (Bjorn Sommer, Tim Dingersen et al.
 *  University of Konstanz, Monash University, Bielefeld University)
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *  
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *  
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *******************************************************************************/
package org.cellmicrocosmos.cm2.gui;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.Graphics;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.io.File;
import java.io.IOException;
import java.util.Vector;

import javax.swing.JFileChooser;
import javax.swing.JPanel;

import org.cellmicrocosmos.cm2.io.DatabaseIO;
import org.cellmicrocosmos.cm2.io.MembraneMetaInfo;
import org.cellmicrocosmos.cm2.io.MembraneXmlReader;
import org.cellmicrocosmos.cm2.io.PdbReader_3D;
import org.cellmicrocosmos.cm2.io.PdbReader_3D.PdbAttributes;
import org.cellmicrocosmos.cm2.misc.Cm2_Toolkit;
import org.jdom.JDOMException;


/**
 * A Component that gets added to the filechoosers. Displays various Properties
 * of the highlighted file during browsing
 * 
 * @author Tim Dingersen dackelnose@gmx.net
 * 
 */
@SuppressWarnings("serial")
public class MyFCAccessory extends JPanel implements PropertyChangeListener {

	private final int mode;

	private String[] str1;

	private String[] str2;

	private final String header;

	Color originalbackground;

	private String size;

	private final JFileChooser chooser;

	public MyFCAccessory(final JFileChooser chooser, final int mode,
			final String header) {

		this.mode = mode;
		this.header = header;
		this.chooser = chooser;
		chooser.addPropertyChangeListener(this);

		this.setPreferredSize(new Dimension(180, 350));
		this.originalbackground = this.getBackground();
	}

	public void propertyChange(final PropertyChangeEvent evt) {

		if (JFileChooser.SELECTED_FILES_CHANGED_PROPERTY.equals(evt
				.getPropertyName())
				&& this.chooser.getSelectedFiles().length > 1) {

			final File[] files = this.chooser.getSelectedFiles();
			this.str1 = new String[] { files.length + " files selected" };
			this.str2 = new String[] { "" };
			long l = 0;
			for (int i = 0; i < files.length; i++) {
				l += files[i].length();
			}
			this.size = Cm2_Toolkit.convertFileSizeToString(l);
			this.repaint();
			return;

		}

		if (JFileChooser.SELECTED_FILE_CHANGED_PROPERTY.equals(evt
				.getPropertyName())) {

			final File newFile = (File) evt.getNewValue();

			if (newFile == null) {
				this.str1 = null;
				this.str2 = null;
				this.size = null;
				this.repaint();
				return;
			}

			this.size = Cm2_Toolkit.convertFileSizeToString(newFile.length());

			final Vector<String> vstr1 = new Vector<String>();
			final Vector<String> vstr2 = new Vector<String>();

			if (this.mode == 2) {

				try {
					final PdbReader_3D reader = new PdbReader_3D();
					final PdbAttributes attrs = reader.readAttributes(newFile);

					try {
						if (attrs.getHeader() == null)
							throw new Exception();

						vstr2.add(attrs.getHeader());
						vstr1.add("Header");
					} catch (final Exception e) {

					}
					try {
						if (attrs.getCode() == null)
							throw new Exception();

						vstr2.add(attrs.getCode());
						vstr1.add("Code");
					} catch (final Exception e) {

					}

					try {
						if (attrs.getTitles() == null)
							throw new Exception();

						vstr2.add(attrs.getTitles());
						vstr1.add("Title");
					} catch (final Exception e) {

					}
					try {
						if (attrs.getSpecies() == null)
							throw new Exception();

						vstr2.add(attrs.getSpecies());
						vstr1.add("Species");
					} catch (final Exception e) {

					}

				} catch (final IOException e) {

				}

			}

			if (this.mode == 0) {

				try {
					final MembraneMetaInfo info = MembraneXmlReader
							.readPreview(newFile);

					try {
						if (info.header == null)
							throw new Exception();
						if (info.header.length() < 1)
							throw new Exception();
						vstr2.add(info.header);
						vstr1.add("Header");
					} catch (final Exception e) {

					}
					try {
						if (info.code == null)
							throw new Exception();
						if (info.code.length() < 1)
							throw new Exception();
						vstr2.add(info.code);
						vstr1.add("Code");
					} catch (final Exception e) {

					}

					try {
						vstr2.add(info.size_x + " X " + info.size_y);
						vstr1.add("Size");
					} catch (final Exception e) {

					}

					try {
						final int total = info.ProtCount + info.LipCountExtra
								+ info.LipCountIntra;
						vstr2.add("Total: " + total + "\n" + "Proteins: "
								+ info.ProtCount + "\n" + "Lipids Extra: "
								+ info.LipCountExtra + "\n" + "Lipids Intra: "
								+ info.LipCountIntra);
						vstr1.add("Molecule Counts");
					} catch (final Exception e) {

					}

					try {
						if (info.author == null)
							throw new Exception();
						if (info.author.length() < 1)
							throw new Exception();
						vstr2.add(info.author);
						vstr1.add("Author");
					} catch (final Exception e) {

					}
					try {
						if (info.date == null)
							throw new Exception();
						if (info.date.length() < 1)
							throw new Exception();
						vstr2.add(info.date);
						vstr1.add("Created");
					} catch (final Exception e) {

					}

				} catch (final IOException e) {

				} catch (final JDOMException e) {

				}
			}
			if (this.mode == 1) {

				try {
					final int[] ints = DatabaseIO.readPreview(newFile);

					try {

						vstr2.add("" + ints[0]);
						vstr1.add("Protein Entries");

						vstr2.add("" + ints[1]);
						vstr1.add("Lipid Entries");
					} catch (final Exception e) {

					}

				} catch (final Exception e) {

				}

			}

			this.str1 = vstr1.toArray(new String[0]);
			this.str2 = vstr2.toArray(new String[0]);

			this.repaint();

		}
	}

	@Override
	public void paintComponent(final Graphics g) {
		super.paintComponent(g);

		// FONTS
		final Font font1 = new Font("Arial", Font.BOLD, 11);
		final Font font2 = new Font("Arial", Font.PLAIN, 9);
		final Font fontheader = new Font("Arial", Font.PLAIN, 12);

		g.setColor(this.originalbackground.brighter());
		g.fillRect(0, 0, this.getWidth(), this.getHeight());

		g.setColor(this.originalbackground.darker());
		g.fillRoundRect(8, 3, this.getWidth() - 3, 20, 15, 15);

		g.setColor(this.originalbackground);
		g.fillRect(0, 0, 5, this.getHeight());

		g.setColor(this.originalbackground.darker().darker());
		g.drawLine(this.getWidth() - 1, 0, this.getWidth() - 1, this
				.getHeight());
		g.drawLine(5, this.getHeight() - 1, this.getWidth(),
				this.getHeight() - 1);

		g.setColor(this.originalbackground);
		g.fillRect(0, this.getHeight() - 20, this.getWidth(), 5);

		g.setColor(this.originalbackground.darker().darker());

		g.drawLine(5, this.getHeight() - 21, this.getWidth(),
				this.getHeight() - 21);

		g.setColor(this.originalbackground.brighter());
		g.setFont(fontheader);
		g.drawString(this.header, 15, 17);

		g.setColor(Color.black);

		g.setFont(font1);

		if (this.size == null)
			return;
		g.drawString("File Size:   " + this.size, 15, this.getHeight() - 5);

		if (this.str1 == null)
			return;

		int progress = 25;
		for (int i = 0; i < this.str1.length; i++) {

			final String s1 = this.str1[i];
			String s2 = this.str2[i];

			int counter = 0;
			for (int k = 0; k < s2.length(); k++) {
				if (s2.charAt(k) == '\n') {
					counter = 0;
				}
				if (counter > 20) {
					s2 = s2.substring(0, k + 1) + "\n" + s2.substring(k + 1);
					counter = 0;
				}
				counter++;
			}

			final String[] s2cut = s2.split("\n");

			progress += 20;

			if (progress > this.getHeight() - 20) {

				break;
			}

			g.setFont(font1);
			g.drawString(s1, 15, progress);
			progress += 15;

			g.setFont(font2);
			for (int j = 0; j < s2cut.length; j++) {

				if (s2cut[j].replaceAll(" ", "").length() < 1)
					continue;

				if (j > 0)
					progress += 12;

				if (j > 4) {
					g.drawString("...", 25, progress);

					break;
				}

				if (progress > this.getHeight() - 20) {

					break;
				}

				g.drawString(s2cut[j], 25, progress);

			}

		}

	}
}
