/*******************************************************************************
 * CELLmicrocosmos 2.2 MembraneEditor (version 2.2.2_2)
 *
 * Copyright (c) 2005, 2017 The CELLmicrocosmos 2 Development Team
 * (Bjorn Sommer, Tim Dingersen et al.
 *  University of Konstanz, Monash University, Bielefeld University)
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *  
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *  
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *******************************************************************************/
package org.cellmicrocosmos.cm2.gui;

import java.awt.Graphics;
import java.awt.Image;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.image.BufferedImage;

import javax.swing.GrayFilter;
import javax.swing.ImageIcon;
import javax.swing.JRadioButton;

import org.cellmicrocosmos.cm2.misc.Cm2_IconDepot;


/**
 * Class for the 3 buttons that control the camera mode. Because their look is
 * pretty much modified, they are defined in an own class
 * 
 * @author Dingi dackelnose@gmx.net
 * 
 */
@SuppressWarnings("serial")
public class MyJRadioButton extends JRadioButton implements MouseListener {

	private final int mode;

	private boolean isRollingOver = false;

	Image img;

	public MyJRadioButton(final int mode) {
		this.mode = mode;
		BufferedImage im;
		this.setRolloverEnabled(true);
		this.addMouseListener(this);
		if (mode == -1)
			im = new BufferedImage(35, 16, BufferedImage.TYPE_3BYTE_BGR);
		else if (mode == 3 || mode == 4) {

			im = new BufferedImage(35, 22, BufferedImage.TYPE_3BYTE_BGR);

		} else
			im = new BufferedImage(28, 6, BufferedImage.TYPE_3BYTE_BGR);

		this.setIcon(new ImageIcon(im));

		if (mode == -1) {
			this.setToolTipText("Go into Free Look Mode");
			this.img = Cm2_IconDepot.getSpecifiedIcon("MV_FREEBtn").getImage();
		}
		if (mode == 0) {
			this
					.setToolTipText("Change to a fixed Position viewing down the Y-Axis");
			this.img = Cm2_IconDepot.getSpecifiedIcon("MV_ECBtn").getImage();
		}
		if (mode == 1) {
			this
					.setToolTipText("Change to a fixed Position viewing up the Y-Axis");
			this.img = Cm2_IconDepot.getSpecifiedIcon("MV_ICBtn").getImage();
		}
		if (mode == 3) {
			this.setToolTipText("Change the camera");
			this.img = Cm2_IconDepot.getSpecifiedIcon("MV_FREEBtn").getImage();
		}
		if (mode == 4) {
			this.setToolTipText("Switch to Align Mode");
			this.img = Cm2_IconDepot.getSpecifiedIcon("MV_AlignSingle")
					.getImage();
		}
	}

	@Override
	public void paintComponent(final Graphics g) {

		final int W = this.getWidth();
		final int H = this.getHeight();
		g.setColor(this.getBackground());
		g.fillRect(0, 0, W, H);

		int a = 3;
		int b = 0;
		if (this.mode == -1)
			a = 7;
		if (this.mode == 3)
			a = 11;
		if (this.mode == 4)
			a = 15;

		if (this.mode == 4)
			b = 3;

		if (this.isEnabled())
			g.drawImage(this.img, a, b, null);
		else
			g.drawImage(GrayFilter.createDisabledImage(this.img), a, b, null);

		final int x = 1;
		int y = 1;
		if (this.mode == -1) {
			y = 8;

		}
		if (this.mode == 1)
			y = 3;

		if (this.mode == 4 || this.mode == 3)
			y = 8;

		if (this.isEnabled()) {
			if (this.isSelected()) {
				g.drawImage(Cm2_IconDepot.getSpecifiedIcon("MV_LampOn")
						.getImage(), x, y, null);
			} else {
				g.drawImage(Cm2_IconDepot.getSpecifiedIcon("MV_LampOff")
						.getImage(), x, y, null);

			}

			if (this.isRollingOver) {
				g.setColor(this.getBackground().darker());
				g.drawRoundRect(0, 0, W, H, 3, 3);
			}
		} else
			g.drawImage(GrayFilter.createDisabledImage(Cm2_IconDepot
					.getSpecifiedIcon("MV_LampOff").getImage()), x, y, null);

	}

	public void mouseClicked(final MouseEvent e) {

	}

	public void mouseEntered(final MouseEvent e) {
		this.isRollingOver = true;

	}

	public void mouseExited(final MouseEvent e) {
		this.isRollingOver = false;

	}

	public void mousePressed(final MouseEvent e) {

	}

	public void mouseReleased(final MouseEvent e) {

	}

}
