/*******************************************************************************
 * CELLmicrocosmos 2.2 MembraneEditor (version 2.2.2_2)
 *
 * Copyright (c) 2005, 2017 The CELLmicrocosmos 2 Development Team
 * (Bjorn Sommer, Tim Dingersen et al.
 *  University of Konstanz, Monash University, Bielefeld University)
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *  
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *  
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *******************************************************************************/
package org.cellmicrocosmos.cm2.gui;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.IOException;
import java.util.Vector;

import org.cellmicrocosmos.cm2.engine.DataPool;
import org.cellmicrocosmos.cm2.engine.Engine;
import org.cellmicrocosmos.cm2.interfaces.SearchListener;


/**
 * This class can search the entries.idx of the RCSB-Database for strings. If a
 * match is found, an Event will be fired where the rowdata for the table gets
 * delivered
 * 
 * @author Tim Dingersen dackelnose@gmx.net
 */
public class Searcher {

	private File file;

	private SearchListener listener;

	public int length = 0;

	public double step = 0;

	private BufferedReader br;

	/**
	 * Sets the searcher to the entries file
	 * 
	 * @param file
	 */
	public void setFile(final File file) {
		this.file = file;
	}

	public void close() {
		if (this.br != null)
			try {
				this.br.close();
			} catch (final IOException e) {

			}
	}

	/**
	 * Adds a searchListener to inform about events
	 * 
	 * @param listener
	 */
	public void addSearchListener(final SearchListener listener) {
		this.listener = listener;
	}

	/**
	 * Determines the length of the file in rows for the length field Also
	 * determines the number of rows that count as 1 percent of all rows
	 * 
	 * @throws IOException
	 */
	public void prepareLength() throws IOException {

		this.length = 0;

		this.br = new BufferedReader(new FileReader(this.file));
		while (this.br.readLine() != null) {
			this.length++;
		}
		this.br.close();

		this.step = 100 / new Integer(this.length).doubleValue();

	}

	/**
	 * Starts to search for the passed string(s)
	 * 
	 * @param searchstring
	 * @throws IOException
	 */
	public void startSearching(String searchstring,boolean or) throws IOException {

		searchstring = searchstring.toLowerCase().trim();

		// splitting the searchstring into terms
		final String[] searcharray = searchstring.split(" ");

		this.br = new BufferedReader(new FileReader(this.file));
		int counter = 0;

		// for each row...
		while (true) {
			counter++;

			final String wholeone = this.br.readLine();

			if (wholeone == null)
				break;

			// invalidates the first 2 rows in the file
			if (counter < 3)
				continue;

			
			
			final String[] firstfive = wholeone.split("\t", 6);

			CharSequence lastEC = "";
			
			// for each search term in the current row
			
			boolean b=true;
			
			
			if(!or) {
				
				for (int i = 0; i < searcharray.length; i++) {
					if(!wholeone.toLowerCase().contains(searcharray[i].trim())) {
						b=false;
						break;
					}
				}
				
			}
			
			if(b) {
			for (int i = 0; i < searcharray.length; i++) {

				
				
				boolean goon = false;

				final CharSequence singlesearchseq = searcharray[i].trim()
   						.subSequence(0, searcharray[i].length());
				
				// find out if the charsequence is an E.C. number (only needed for CmX)
				if (Engine.getMRNA()!=null && singlesearchseq.toString().matches("\\d+\\.\\d+\\.\\d+\\.\\d+")) {
					lastEC = singlesearchseq;
				}
				
				// for each word to in the row
				
				for (int j = 0; j < firstfive.length - 1; j++) {

					if (j == 2)
						continue;

					if (firstfive[j].toLowerCase().contains(singlesearchseq)) {
						goon = true;
						break;
					}

				}
				
					
				
				if (goon == false) {
					continue; // FORMER: break;
				}

				// FORMER: if (i == searcharray.length - 1) {

				final Vector<String> rowdata = new Vector<String>();

				for (int k = 0; k < firstfive.length - 1; k++) {
					if (k == 2)
						continue;

					rowdata.add(firstfive[k]);

				}
				
				rowdata.add(1, Boolean.toString(DataPool.getMifHandler().isElementOfOPM(firstfive[0])));
				rowdata.add(1, Boolean.toString(DataPool.getMifHandler().isElementOfPDBTM(firstfive[0])));

				final String[] datevalues = (firstfive[2].split("/"));

				final String datestring = datevalues[1] + "/"
						+ datevalues[0] + "/" + datevalues[2];

				rowdata.add(datestring);
				
				if (Engine.getMRNA()!=null)
					rowdata.add(lastEC.toString());

				this.listener.matchFound(rowdata);
				break;
				// FORMER: }

			}
			}
			this.listener.rowFinished(counter);

		}
		this.br.close();

	}

}
