/*******************************************************************************
 * CELLmicrocosmos 2.2 MembraneEditor (version 2.2.2_2)
 *
 * Copyright (c) 2005, 2017 The CELLmicrocosmos 2 Development Team
 * (Bjorn Sommer, Tim Dingersen et al.
 *  University of Konstanz, Monash University, Bielefeld University)
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *  
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *  
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *******************************************************************************/
package org.cellmicrocosmos.cm2.gui.dialogs;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;
import java.awt.event.WindowEvent;
import java.awt.event.WindowFocusListener;
import java.io.File;
import java.io.IOException;
import java.util.Vector;

import javax.swing.BorderFactory;
import javax.swing.BoxLayout;
import javax.swing.ButtonGroup;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;

import org.cellmicrocosmos.cm2.constants.CM2_CONSTANTS;
import org.cellmicrocosmos.cm2.interfaces.EditDBEntryDialogListener;
import org.cellmicrocosmos.cm2.io.PdbReader_3D;
import org.cellmicrocosmos.cm2.io.PdbReader_3D.PdbAttributes;
import org.cellmicrocosmos.cm2.misc.Cm2_IconDepot;


/**
 * A Dialog that lets the user Edit an Entry in the DB.
 * 
 * @author Tim Dingersen dackelnose@gmx.net
 * 
 */
@SuppressWarnings("serial")
public class EditDBEntryDialog extends JDialog implements ActionListener {

	// possible return values when showing up
	public final int APPROVE = 1;

	public final int CANCEL = 2;

	private Vector rowdata;

	private int out = this.CANCEL;

	private JScrollPane scroll;

	private JButton cancelBtn;

	private JButton okBtn;

	private JButton choose;

	private JPanel buttonbar;

	private JLabel tf1;

	private JLabel tf2;

	private JLabel tf3;

	private JLabel tf4;

	private JTextArea tf5;

	private JLabel tf6;

	private JLabel message;

	private JLabel message2;

	private JPanel ground1;

	private JPanel ground2;

	private JPanel ground3;

	private JPanel ground4;

	private JPanel ground5;

	private JPanel ground6;

	private JPanel ground7;

	private JPanel mground1;

	private JPanel mground2;

	private JPanel superground;

	private JPanel radioPanel;

	private final EditDBEntryDialogListener listener;

	private JRadioButton radio1, radio2;

	private String selectedcode;

	public EditDBEntryDialog(final EditDBEntryDialogListener listener) {
		this.listener = listener;
	}

	/**
	 * Pop up this Dialog and the user can edit the passed rowdata
	 * 
	 * @param rowdata
	 *            the rowdata to Edit and to get the tf values from
	 * @return
	 */
	@SuppressWarnings("unchecked")
	public int askParameters(final Vector row) {
		this.rowdata = row;

		this.tf1 = new JLabel();

		this.tf2 = new JLabel();

		this.tf3 = new JLabel();

		this.tf4 = new JLabel();

		this.tf5 = new JTextArea();

		this.tf6 = new JLabel();

		this.message = new JLabel();

		this.message2 = new JLabel();

		this.ground1 = new JPanel();

		this.ground2 = new JPanel();

		this.ground3 = new JPanel();

		this.ground4 = new JPanel();

		this.ground5 = new JPanel();

		this.ground6 = new JPanel();

		this.ground7 = new JPanel();

		this.mground1 = new JPanel();

		this.mground2 = new JPanel();

		this.buttonbar = new JPanel();

		this.choose = new JButton();

		this.okBtn = new JButton("Apply");

		this.cancelBtn = new JButton("Cancel");

		this.scroll = new JScrollPane(this.tf5);

		this.superground = new JPanel();

		this.radioPanel = new JPanel();

		this.radio1 = new JRadioButton("Protein");

		this.radio2 = new JRadioButton("Lipid");

		// setting the radiobuttons to the group the Entry belongs to
		if (this.listener.getTableOnFrontIndex() == 0)
			this.radio1.setSelected(true);
		else
			this.radio2.setSelected(true);

		// block the rest of the application until this window is closed
		this.setModal(true);

		this.setTitle("Edit Entry");
		this.superground.setLayout(new BoxLayout(this.superground,
				BoxLayout.Y_AXIS));
		this.ground7.setLayout(new BoxLayout(this.ground7, BoxLayout.Y_AXIS));

		this.getContentPane().add(this.superground);
		this.superground.add(this.radioPanel);
		this.superground.add(this.ground1);
		this.superground.add(this.ground2);
		this.superground.add(this.ground3);
		this.superground.add(this.ground4);
		this.superground.add(this.ground5);
		this.superground.add(this.ground6);
		this.superground.add(this.ground7);
		this.superground.setBorder(BorderFactory.createEtchedBorder());

		this.tf5.setLineWrap(false);

		this.choose.setIcon(Cm2_IconDepot.getSpecifiedIcon("Folder_Plain"));

		this.radioPanel.setBorder(BorderFactory.createTitledBorder("Category"));
		this.ground1.setBorder(BorderFactory.createTitledBorder("Pdb Code"));
		this.ground2.setBorder(BorderFactory.createTitledBorder("Header"));
		this.ground3.setBorder(BorderFactory.createTitledBorder("Discription"));
		this.ground4.setBorder(BorderFactory.createTitledBorder("Source"));
		this.ground5.setBorder(BorderFactory.createTitledBorder("Notes"));
		this.ground6.setBorder(BorderFactory.createTitledBorder("File"));

		this.tf5.setBorder(BorderFactory.createLoweredBevelBorder());

		this.message.setBorder(BorderFactory.createLineBorder(Color.orange));
		this.message2.setBorder(BorderFactory.createLineBorder(Color.red));

		this.choose.addActionListener(this);
		this.okBtn.addActionListener(this);
		this.cancelBtn.addActionListener(this);

		final ButtonGroup bg = new ButtonGroup();
		bg.add(this.radio1);
		bg.add(this.radio2);
		this.radioPanel.add(this.radio1);

		this.radioPanel.add(this.radio2);

		this.buttonbar.add(this.okBtn);

		this.buttonbar.add(this.cancelBtn);

		this.buttonbar.add(this.choose);

		// choose.setBackground(new Color(250, 250, 180));

		this.ground1.add(this.tf1);
		this.ground2.add(this.tf2);
		this.ground3.add(this.tf3);
		this.ground4.add(this.tf4);
		this.ground5.add(this.scroll);
		this.ground6.add(this.tf6);
		this.mground1.add(this.message);
		this.mground2.add(this.message2);

		this.ground7.add(this.mground1);
		this.ground7.add(this.mground2);

		this.superground.add(this.buttonbar);

		this.tf1.setFont(new Font("Arial", Font.PLAIN, 13));
		this.tf2.setFont(new Font("Arial", Font.PLAIN, 13));
		this.tf3.setFont(new Font("Arial", Font.PLAIN, 13));
		this.tf4.setFont(new Font("Arial", Font.PLAIN, 13));
		this.tf5.setFont(new Font("Arial", Font.PLAIN, 13));
		this.tf6.setFont(new Font("Arial", Font.PLAIN, 13));
		this.message.setFont(new Font("Arial", Font.PLAIN, 12));
		this.message2.setFont(new Font("Arial", Font.PLAIN, 12));

		// adding some keyboard functions
		final KeyAdapter kl = new KeyAdapter() {

			@Override
			public void keyPressed(final KeyEvent arg0) {
				if (arg0.getKeyCode() == KeyEvent.VK_ESCAPE) {
					EditDBEntryDialog.this.dispose();
				}
			}

		};

		this.tf5.addKeyListener(kl);

		// make sure this window is always on top(inside the application)
		this.addWindowFocusListener(new WindowFocusListener() {

			public void windowGainedFocus(final WindowEvent e) {

			}

			public void windowLostFocus(final WindowEvent e) {

				EditDBEntryDialog.this.requestFocus();

			}

		});

		this.selectedcode = (String) this.rowdata.get(6);

		this.refreshLabels();

		this.pack();

		final Dimension dim = this.getToolkit().getScreenSize();
		this.setLocation(dim.width / 2 - this.getWidth() / 2, dim.height / 2
				- this.getHeight() / 2);

		this.tf5.requestFocus();
		this.setVisible(true);
		// goes on here when the dialog gets closed
		return this.out;

	}

	/**
	 * Sets the values of the textfield for the code and the label with the
	 * filepath after a file has been chosen with the filechooser.
	 * 
	 */
	private void refreshLabels() {

		this.okBtn.setEnabled(true);
		this.message.setText("");
		this.message2.setText("");

		this.message.setVisible(false);
		this.message2.setVisible(false);

		if (this.rowdata.get(1) != null)
			this.tf1.setText((String) this.rowdata.get(1));
		else
			this.tf1.setText(null);
		if (this.rowdata.get(2) != null)
			this.tf2.setText((String) this.rowdata.get(2));
		else
			this.tf2.setText(null);
		if (this.rowdata.get(3) != null) {
			final String titles = (String) this.rowdata.get(3);
			this.tf3.setText("<html>" + titles.replaceAll("\n", "<br>")
					+ "</html>");
		} else
			this.tf3.setText(null);
		if (this.rowdata.get(4) != null)
			this.tf4.setText((String) this.rowdata.get(4));
		else
			this.tf4.setText(null);
		if (this.rowdata.get(5) != null)
			this.tf5.setText((String) this.rowdata.get(5));
		else
			this.tf5.setText(null);
		if (this.rowdata.get(6) != null)
			this.tf6.setText((String) this.rowdata.get(6));
		else
			this.tf6.setText(null);

		if ((new File(this.tf6.getText()).canRead() && new File(this.tf6
				.getText()).isFile()) == false) {

			this.message.setText("Please select a valid File for this Entry");
			this.message.setVisible(true);

		}

		else if (this.tf1.getText() == null) {

			this.message
					.setText("<html>A proper Pdb-Code has not been found in the File!<br>"
							+ "Using the name of the file instead");
			this.message.setVisible(true);
		}

		if (this.listener != null) {

			if (this.tf6.getText() == null)
				return;

			if (this.listener.dbContainsPdbFile(new File(this.tf6.getText()))
					&& !new File(this.selectedcode).getAbsoluteFile().equals(
							new File(this.tf6.getText()).getAbsoluteFile())
					&& !CM2_CONSTANTS.MISC.ALLOW_MULTIPLE_ENTRIES_IN_DB) {

				this.message2
						.setText("The Database already contains an Entry that has this ID!");
				this.message2.setVisible(true);
				this.okBtn.setEnabled(false);
			}

		}

	}

	/**
	 * Extends the dialogs pack method by setting the witdh regarding to the
	 * space needed for the complete Filepath
	 */
	@Override
	public void pack() {

		this.ground1.setVisible(true);
		this.ground2.setVisible(true);
		this.ground3.setVisible(true);
		this.ground4.setVisible(true);

		final JLabel[] labels = { this.tf1, this.tf2, this.tf3, this.tf4,
				this.tf6, this.message, this.message2 };

		int prefw = 300;

		for (int i = 0; i < labels.length; i++) {
			prefw = Math.max(prefw, labels[i].getPreferredSize().width);
		}

		this.radio1.setPreferredSize(new Dimension(prefw / 3, 20));

		if (this.tf1.getText() != null)
			this.tf1.setPreferredSize(new Dimension(prefw, 22));
		else
			this.ground1.setVisible(false);

		if (this.tf2.getText() != null)
			this.tf2.setPreferredSize(new Dimension(prefw, 22));
		else
			this.ground2.setVisible(false);

		if (this.tf3.getText() != null) {
			final String[] string = this.tf3.getText().split("<br>");
			this.tf3.setPreferredSize(new Dimension(prefw, string.length * 22));
		} else
			this.ground3.setVisible(false);

		if (this.tf4.getText() != null)
			this.tf4.setPreferredSize(new Dimension(prefw, 22));
		else
			this.ground4.setVisible(false);

		this.scroll.setPreferredSize(new Dimension(prefw, 75));

		this.tf6.setPreferredSize(new Dimension(prefw, 22));

		final String[] string2 = this.message.getText().split("<br>");
		this.message
				.setPreferredSize(new Dimension(prefw, string2.length * 20));
		this.message2.setPreferredSize(new Dimension(prefw, 20));

		super.pack();

	}

	/**
	 * Returns the rowdata that was passed to the dialog
	 * 
	 * @return
	 */
	public Vector getResultingRowdata() {
		return this.rowdata;
	}

	/**
	 * Returns which type of Molecule was selected
	 * 
	 * @return
	 */
	public int getSelectedType() {
		if (this.radio1.isSelected())
			return 0;
		else
			return 1;
	}

	/**
	 * Actions for the 3 Buttons in the Toolbar
	 */
	@SuppressWarnings("unchecked")
	public void actionPerformed(final ActionEvent arg0) {

		/*
		 * Lets the user edit the file associated with this Entry;
		 */
		if (arg0.getSource() == this.choose) {

			File init;
			if (this.tf6.getText() != null)
				init = new File(this.tf6.getText());
			else
				init = null;
			if (init.isFile() == false)
				init = null;

			File file;

			if (init != null)
				file = this.listener.askPDBFile(init.getAbsoluteFile());
			else
				file = this.listener.askPDBFile(null);

			if (file != null) {

				try {
					final PdbReader_3D reader = new PdbReader_3D();
					final PdbAttributes attrs = reader.readAttributes(file);

					this.rowdata.set(1, attrs.getCode());
					this.rowdata.set(2, attrs.getHeader());
					this.rowdata.set(3, attrs.getTitles());
					this.rowdata.set(4, attrs.getSpecies());

					this.rowdata.set(6, file.getPath());

				} catch (final IOException e) {
					this.rowdata.set(1, null);
					this.rowdata.set(2, null);
					this.rowdata.set(3, null);
					this.rowdata.set(4, null);

					this.rowdata.set(6, file.getPath());
				}

				this.refreshLabels();

				this.pack();
			}

		}

		/*
		 * Accepts the Changes
		 */
		if (arg0.getSource() == this.okBtn) {

			this.rowdata.set(1, this.tf1.getText());
			this.rowdata.set(2, this.tf2.getText());

			if (this.tf3.getText() != null)
				this.rowdata.set(3, this.tf3.getText().replaceAll("<br>", "\n")
						.replaceAll("<html>", "").replaceAll("</html>", ""));
			else
				this.rowdata.set(3, null);

			this.rowdata.set(4, this.tf4.getText());
			this.rowdata.set(5, this.tf5.getText());
			this.rowdata.set(6, this.tf6.getText());

//			this.rowdata
//					.set(
//							6,
//							Cm2_Toolkit
//									.getLocalPDBDirPathIfInsideProgramsFolder((String) this.rowdata
//											.get(6)));

			this.out = this.APPROVE;
			this.dispose();

		}
		// or cancels the changes
		if (arg0.getSource() == this.cancelBtn) {
			this.dispose();
		}
	}
}
