/*******************************************************************************
 * CELLmicrocosmos 2.2 MembraneEditor (version 2.2.2_2)
 *
 * Copyright (c) 2005, 2017 The CELLmicrocosmos 2 Development Team
 * (Bjorn Sommer, Tim Dingersen et al.
 *  University of Konstanz, Monash University, Bielefeld University)
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *  
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *  
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *******************************************************************************/
/**
 * 
 */
package org.cellmicrocosmos.cm2.gui.dialogs;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.TextField;
import java.awt.Toolkit;
import java.awt.datatransfer.Clipboard;
import java.awt.datatransfer.StringSelection;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextPane;
import javax.swing.ScrollPaneConstants;

import org.cellmicrocosmos.cm2.engine.DataPool;
import org.cellmicrocosmos.cm2.misc.Cm2_IconDepot;

/**
 * This class is used to show the actual protocols generated by the 
 * Membrane Packing Algorithms.
 * 
 * @author Bjoern Sommer bjoern@CELLmicrocosmos.org
 * 
 */
public class ProtocolDialog extends JDialog {
	
	private static final long serialVersionUID = 3863279064851261777L;

	public ProtocolDialog() {
		
		Dimension windowSizeBig = new Dimension(500, 500);
		Dimension buttonPanelSize = new Dimension(70, 500);
		
		this.setPreferredSize(windowSizeBig);
		this.setSize(windowSizeBig);
		this.setMinimumSize(windowSizeBig);
		this.setMaximumSize(windowSizeBig);
		this.setTitle("Algorithm Protocol");
		
		final JTextPane text = new JTextPane();
		text.setAlignmentX(TextField.LEFT_ALIGNMENT);
		text.setAlignmentY(TextField.TOP_ALIGNMENT);
		text.setContentType("text/html");
		text.setText(DataPool.getMembraneModel().getAlgorithmName()!=null?
						"<html><code>"+
						DataPool.getMembraneModel().getAlgorithmStringRepresentation().replace("\n", "<br>").replace(" ", "&nbsp;")+"</html>":
						"<html><code><b>No Algorithm Executed</b></code></html>");
		text.setBackground(Color.white);
		
		JPanel buttonPanel = new JPanel();
		buttonPanel.setLayout(new BoxLayout(buttonPanel, BoxLayout.Y_AXIS));
		
		buttonPanel.add(Box.createVerticalStrut(5));
		
		JButton copyButtonHTML = new JButton("Copy HTML");
		copyButtonHTML.addActionListener(new ActionListener() {
			
			public void actionPerformed(ActionEvent e) {
				StringSelection stringSelection = new StringSelection (text.getText().replaceAll("&#160;", " "));
				Clipboard clpbrd = Toolkit.getDefaultToolkit ().getSystemClipboard ();
				clpbrd.setContents (stringSelection, null);
			}
			
		});
		buttonPanel.add(copyButtonHTML);
		buttonPanel.add(Box.createVerticalStrut(5));
		
		JButton copyButtonText = new JButton("Copy Text");
		copyButtonText.addActionListener(new ActionListener() {
			
			public void actionPerformed(ActionEvent e) {
				StringSelection stringSelection = new StringSelection (text.getText().replaceAll("<br>", "\n").replaceAll("\\<.*?>","").replaceAll("&#160;", " "));
				Clipboard clpbrd = Toolkit.getDefaultToolkit ().getSystemClipboard ();
				clpbrd.setContents (stringSelection, null);
			}
			
		});
		buttonPanel.add(copyButtonText);
		buttonPanel.add(Box.createVerticalStrut(10));
		buttonPanel.setPreferredSize(buttonPanelSize);
		buttonPanel.setSize(buttonPanelSize);
		buttonPanel.setMinimumSize(buttonPanelSize);
		buttonPanel.setMaximumSize(buttonPanelSize);
		
		JButton closeButton = new JButton("Close");
		closeButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				dispose();
			}
		});
		buttonPanel.add(closeButton);
		
		JScrollPane scrollMain = new JScrollPane(text);
		scrollMain.setHorizontalScrollBarPolicy(ScrollPaneConstants.HORIZONTAL_SCROLLBAR_AS_NEEDED);
		scrollMain.setVerticalScrollBarPolicy(ScrollPaneConstants.VERTICAL_SCROLLBAR_AS_NEEDED);
		
		this.getContentPane().setLayout(new BoxLayout(this.getContentPane(), BoxLayout.X_AXIS));
		this.getContentPane().add(buttonPanel);
		this.getContentPane().add(scrollMain);
		this.setPreferredSize(windowSizeBig);
		this.setSize(windowSizeBig);
		this.setMinimumSize(windowSizeBig);
		this.setMaximumSize(windowSizeBig);
		this.pack();
		this.setIconImage(Cm2_IconDepot.getSpecifiedIcon("Frame_Protocol").getImage());
		this.setAlwaysOnTop(true);
		this.setModal(true);
		this.setVisible(true);
		
	}
	

	public void dispose() {
		this.getWindowListeners()[0].windowClosing(null);
		super.dispose();
		System.gc();
	}
	
}