/*******************************************************************************
 * CELLmicrocosmos 2.2 MembraneEditor (version 2.2.2_2)
 *
 * Copyright (c) 2005, 2017 The CELLmicrocosmos 2 Development Team
 * (Bjorn Sommer, Tim Dingersen et al.
 *  University of Konstanz, Monash University, Bielefeld University)
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *  
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *  
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *******************************************************************************/
package org.cellmicrocosmos.cm2.gui.elements;

import java.awt.Color;
import java.awt.Component;
import java.awt.Cursor;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.GradientPaint;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Point;
import java.awt.Rectangle;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.FocusEvent;
import java.awt.event.FocusListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;
import java.awt.geom.Rectangle2D;
import java.awt.image.BufferedImage;
import java.util.Random;
import java.util.Vector;

import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JColorChooser;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JPopupMenu;
import javax.swing.JScrollPane;
import javax.swing.JSpinner;
import javax.swing.JTabbedPane;
import javax.swing.JToolBar;
import javax.swing.SpinnerNumberModel;
import javax.swing.SwingConstants;
import javax.swing.SwingUtilities;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import org.cellmicrocosmos.cm2.constants.CM2_CONSTANTS;
import org.cellmicrocosmos.cm2.dataobjects.Lipid_3D;
import org.cellmicrocosmos.cm2.dataobjects.MembraneModel;
import org.cellmicrocosmos.cm2.dataobjects.MicroDomain_3D;
import org.cellmicrocosmos.cm2.dataobjects.Molecule_3D;
import org.cellmicrocosmos.cm2.dataobjects.Protein_3D;
import org.cellmicrocosmos.cm2.engine.DataPool;
import org.cellmicrocosmos.cm2.gui.HelpBtn;
import org.cellmicrocosmos.cm2.interfaces.MembraneViewListener;
import org.cellmicrocosmos.cm2.misc.Cm2_IconDepot;
import org.cellmicrocosmos.cm2.misc.Cm2_Toolkit;
import org.cellmicrocosmos.cm2.misc.ConfigParser;


@SuppressWarnings("serial")
/*
 * The view that displays and offers functions to change the contents in the
 * membrane. Is devided into 3 lists: one for proteins proteins and two for the
 * lipids on each side
 * 
 * @author Tim Dingersen, Sebastian Rubert, Bjoern Sommer bjoern@CELLmicrocosmos.org
 */
public class ComponentView extends JPanel implements ChangeListener {

	private MembraneViewListener listener;

	private final JCheckBox syncEcIc = new JCheckBox(
			"sync. EC/IC values         ");

	private final JCheckBox absolutevalues = new JCheckBox(
			"use absolute values       ");
	
	private final JCheckBox autosetpercent = new JCheckBox(
			"auto. adjust percentages");

	private final JCheckBox promptdelete = new JCheckBox(
			"prompt User first           ");

	private final SpinnerNumberModel spinnerModel = new SpinnerNumberModel(0,
			0, 3, 1);
	private final JSpinner decimalplaces = new JSpinner(this.spinnerModel);

	// private JButton confirmButton = new JButton("CONFIRM");

	private int oldDecimalPlaces;

	private final JLabel overallPercentage = new JLabel(
			"Overall LipidPercentage : ");

	DomainPanel domainPanel = new DomainPanel();

	// OptionsPanel optionsPanel = new OptionsPanel();

	JTabbedPane tab = new JTabbedPane();

	private final JToolBar Btnbar;

	private final JButton popUpBtn = new JButton();

	private MicroPanel selectedPanel = null;

	private int lastindex = 0;

	Vector<JPanel> lists = new Vector<JPanel>();

	private boolean updating = false;
	private boolean initing = false;

	public ComponentView() {

		this.setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));

		this.Btnbar = new JToolBar();
		this.Btnbar.setFloatable(false);

		this.syncEcIc.setSelected(true);
		this.absolutevalues.setSelected(false);
		this.autosetpercent.setSelected(true);
		this.promptdelete.setSelected(true);
		this.decimalplaces
				.setValue(CM2_CONSTANTS.MISC.LIPID_PERCENTAGE_DIVISOR);

		this.syncEcIc
				.setToolTipText("Synchronizes both EC/IC lipid types when editing");
		this.absolutevalues
				.setToolTipText("Uses absolute values for lipid types instead of percentages");
		this.autosetpercent
				.setToolTipText("Compounds the other percentages to an overall of 100% automatically");
		this.promptdelete
				.setToolTipText("Always prompt user when attempting to remove a type");
		this.decimalplaces
				.setToolTipText("Define the number of decimal places used for lipid percentage");
		this.decimalplaces.setFocusable(true);

		this.decimalplaces.setPreferredSize(new Dimension(50,
				this.decimalplaces.getPreferredSize().height));
		this.decimalplaces.setMaximumSize(new Dimension(50, this.decimalplaces
				.getPreferredSize().height));
		this.decimalplaces.setMinimumSize(new Dimension(50, this.decimalplaces
				.getPreferredSize().height));

		absolutevalues.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				ComponentView.this.autosetpercent.setEnabled(!ComponentView.this.absolutevalues.isSelected());
				ComponentView.this.decimalplaces.setEnabled(!ComponentView.this.absolutevalues.isSelected());
				ComponentView.this.decimalplaces.setValue(0);
				ComponentView.this.decimalplaces.revalidate();
				ComponentView.this.oldDecimalPlaces = 0;
				ComponentView.this.setComputeAbsoluteValues(ComponentView.this.absolutevalues.isSelected());
			}
		});

		this.popUpBtn.addActionListener(new ActionListener() {

			public void actionPerformed(final ActionEvent arg0) {

				// DataPool.setMicroPanels(lists);

				final JDialog pop = new JDialog();

				final JPanel popuppanel = new JPanel();

				final JPanel outerOne = new JPanel();

				pop.getContentPane().add(outerOne);

				outerOne.add(popuppanel);
				
				final JLabel l0 = new JLabel("Distribution Mode",
						SwingConstants.LEADING);
				final JLabel l1 = new JLabel("When Remove was Pressed",
						SwingConstants.LEADING);
				final JLabel l2 = new JLabel("When Lipids were Modified",
						SwingConstants.LEADING);
				final JLabel l3 = new JLabel("Change Decimal Places",
						SwingConstants.LEADING);
				popuppanel.add(l0);
				popuppanel.add(ComponentView.this.absolutevalues);
				popuppanel.add(l1);
				popuppanel.add(ComponentView.this.promptdelete);
				popuppanel.add(l2);
				popuppanel.add(ComponentView.this.autosetpercent);
				popuppanel.add(ComponentView.this.syncEcIc);
				popuppanel.add(l3);
				popuppanel.add(ComponentView.this.decimalplaces);
				// popuppanel.add(confirmButton);

				try {

					ComponentView.this.oldDecimalPlaces = Integer
							.parseInt(ComponentView.this.decimalplaces.getValue()
									.toString());
				} catch (final Exception e) {
					ComponentView.this.oldDecimalPlaces = 0;
				}

					
				try {

					outerOne.setBackground(new Color(outerOne.getBackground()
							.getRed() - 20,
							outerOne.getBackground().getGreen() - 20, outerOne
									.getBackground().getBlue() - 20));
					popuppanel.setBackground(outerOne.getBackground());
				} catch (final Exception e) {

				}

				final FocusListener fl = new FocusListener() {

					public void focusGained(final FocusEvent arg0) {

					}

					public void focusLost(final FocusEvent arg0) {
						
						if (arg0.getOppositeComponent() != ComponentView.this.promptdelete
								&& arg0.getOppositeComponent() != ComponentView.this.syncEcIc
								&& arg0.getOppositeComponent() != ComponentView.this.absolutevalues
								&& arg0.getOppositeComponent() != ComponentView.this.autosetpercent
								&& arg0.getOppositeComponent() != ComponentView.this.decimalplaces
										.getEditor().getComponent(0)) {

							pop.dispose();
							
						}

						// Get all Micropanels, set number of decimal places
						// and change lipid percentage
						final JPanel listpanel = ComponentView.this.lists
								.elementAt(ComponentView.this.tab
										.getSelectedIndex());

						for (int i = 0; i < listpanel.getComponentCount(); i++) {
							final Component c = listpanel.getComponent(i);

							if (c instanceof MicroPanel == false)
								continue;

							final MicroPanel mp = (MicroPanel) c;
							if (mp.mol instanceof Lipid_3D == false)
								continue;
							try {
								mp.setSpinnerDivisor((int) Math.pow(10, (Integer) ComponentView.this.decimalplaces.getValue()));
							} catch(Exception e) {
								mp.setSpinnerDivisor(1);
							}
						}

					}

				};

				ComponentView.this.promptdelete.addFocusListener(fl);
				ComponentView.this.absolutevalues.addFocusListener(fl);
				ComponentView.this.autosetpercent.addFocusListener(fl);
				ComponentView.this.syncEcIc.addFocusListener(fl);
				ComponentView.this.decimalplaces.getEditor().getComponent(0)
						.addFocusListener(fl);

				ComponentView.this.promptdelete.setBackground(popuppanel
						.getBackground());
				ComponentView.this.absolutevalues.setBackground(popuppanel
						.getBackground());
				ComponentView.this.autosetpercent.setBackground(popuppanel
						.getBackground());
				ComponentView.this.syncEcIc.setBackground(popuppanel
						.getBackground());
				ComponentView.this.decimalplaces.setBackground(popuppanel
						.getBackground());

				popuppanel.setBorder(BorderFactory.createTitledBorder(
						BorderFactory.createEtchedBorder(), "Settings"));

				outerOne.setBorder(BorderFactory.createEtchedBorder());

				popuppanel.setPreferredSize(new Dimension(160, 255));

				pop.setUndecorated(true);
				pop.setAlwaysOnTop(true);

				pop.pack();

				final Point btnp = ComponentView.this.popUpBtn
						.getLocationOnScreen();
				pop.setLocation(
						btnp.x + ComponentView.this.popUpBtn.getWidth(), btnp.y
								+ ComponentView.this.popUpBtn.getHeight()
								- pop.getHeight());
				pop.setVisible(true);

			}

		});

		this.popUpBtn.setToolTipText("Table Settings");
		this.popUpBtn.setIcon(Cm2_IconDepot.getSpecifiedIcon("CV_Settings"));

		this.Btnbar.add(this.popUpBtn);
		this.Btnbar.add(new JPanel());

		this.overallPercentage.setVisible(false);
		this.Btnbar.add(this.overallPercentage);

		this.Btnbar.addSeparator();
		this.Btnbar.add(new HelpBtn(this));

		this.syncEcIc.setFont(new Font(this.syncEcIc.getFont().getFontName(),
				Font.PLAIN, 10));
		this.absolutevalues.setFont(new Font(this.syncEcIc.getFont()
				.getFontName(), Font.PLAIN, 10));
		this.autosetpercent.setFont(new Font(this.syncEcIc.getFont()
				.getFontName(), Font.PLAIN, 10));
		this.promptdelete.setFont(new Font(
				this.syncEcIc.getFont().getFontName(), Font.PLAIN, 10));
		this.decimalplaces.setFont(new Font(this.syncEcIc.getFont()
				.getFontName(), Font.PLAIN, 10));

		try {
			this.syncEcIc.setSelected(Boolean.parseBoolean(ConfigParser
					.getValue("CV_syncEcIc")[0]));
		} catch (final Exception e) {
		}
		
		try {
			this.absolutevalues.setSelected(Boolean.parseBoolean(ConfigParser
					.getValue("CV_absolutevalues")[0]));
		} catch (final Exception e) {
		}

		try {
			this.autosetpercent.setSelected(Boolean.parseBoolean(ConfigParser
					.getValue("CV_autosetpercent")[0]));
			this.autosetpercent.setEnabled(!this.absolutevalues.isSelected());
			this.decimalplaces.setEnabled(!this.absolutevalues.isSelected());
		} catch (final Exception e) {
		}

		try {
			this.promptdelete.setSelected(Boolean.parseBoolean(ConfigParser
					.getValue("CV_promtdelete")[0]));
		} catch (final Exception e) {
		}

		try {
			this.decimalplaces.setValue(Integer.parseInt(ConfigParser
					.getValue("CV_decimalplaces")[0]));
			this.oldDecimalPlaces = Integer.parseInt(this.decimalplaces.getValue().toString());
		} catch (final Exception e) {
		}

		// ''''''''''''''

		// add(optionsPanel);
		final JPanel p = new JPanel();
		p.setLayout(new BoxLayout(p, BoxLayout.Y_AXIS));
		p.add(this.domainPanel);
		p.add(this.tab);
		p.setBorder(BorderFactory.createEtchedBorder());
		this.add(p);

		this.add(this.Btnbar);

		this.tab.setTabPlacement(SwingConstants.LEFT);
		this.tab.setTabLayoutPolicy(JTabbedPane.SCROLL_TAB_LAYOUT);
		// tab.setEnabled(false);
		this.setEnabled(false);

		// doLayout();
		// tab.validate();

	}

	@Override
	public void doLayout() {

		this.Btnbar.setPreferredSize(new Dimension(this.getWidth(), 25));

		this.tab.setPreferredSize(new Dimension(this.getWidth(), this
				.getHeight() - 25));

		super.doLayout();

	}

	public void setEnabled(final boolean b) {
		super.setEnabled(b);
		this.tab.setEnabled(b);
		this.popUpBtn.setEnabled(b);
		this.domainPanel.setEnabled(b);

		final JScrollPane scroll = (JScrollPane) this.tab
				.getSelectedComponent();

		try {
			final JPanel listpanel = (JPanel) (scroll.getViewport()
					.getComponent(0));
			for (int i = 0; i < listpanel.getComponentCount(); i++) {
				final Component c = listpanel.getComponent(i);
				if (c instanceof MicroPanel) {
					final MicroPanel mp = (MicroPanel) c;

					mp.setEnabled(b);

				}

			}

		} catch (final Exception e) {

		}

	}

	public void addMembraneViewListener(final MembraneViewListener listener) {
		this.listener = listener;

	}

	public void storeCheckBoxValues() {
		ConfigParser.deleteAllAt("CV_syncEcIc");
		ConfigParser.addValue("CV_syncEcIc", "" + this.syncEcIc.isSelected());
		ConfigParser.deleteAllAt("CV_absolutevalues");
		ConfigParser.addValue("CV_absolutevalues", ""
				+ this.absolutevalues.isSelected());
		ConfigParser.deleteAllAt("CV_autosetpercent");
		ConfigParser.addValue("CV_autosetpercent", ""
				+ this.autosetpercent.isSelected());
		ConfigParser.deleteAllAt("CV_promtdelete");
		ConfigParser.addValue("CV_promtdelete", ""
				+ this.promptdelete.isSelected());
		ConfigParser.deleteAllAt("CV_decimalplaces");
		ConfigParser.addValue("CV_decimalplaces", ""
				+ this.decimalplaces.getValue());

	}

	/**
	 * AutoSets other percentages for all lipis on one membrane side
	 * 
	 * @param mode
	 *            1=EC, 2=IC
	 * @param newval
	 *            the new percentage to accord to
	 * @param code
	 *            the code for the type of the regarded lipid
	 * @param domainID
	 *            the domainID of the value to change
	 */
	public void setPercentages(final int mode, final int newval,
			final String code, final int domainID) {

		final MembraneModel model = DataPool.getMembraneModel();

		final int layerID = DataPool.getCurrentLayerID();

		Lipid_3D lip;

		try {
			if (mode == 1) {
				lip = model.getAllExampleLipidsOfTypeExtra(code, domainID,
						layerID)[0];

			} else if (mode == 2) {
				lip = model.getAllExampleLipidsOfTypeIntra(code, domainID,
						layerID)[0];

			} else
				return;
		} catch (final Exception e) {
			return;
		}

		if (!this.autosetpercent.isSelected() || this.absolutevalues.isSelected()) {
			lip.setPercent(newval);
			DataPool.setMembraneHasBeenEditedAgain(true);

		} else {

			String[] types;

			if (mode == 1)
				types = model.getAllExampleLipidTypesExtraInDomain(domainID,
						layerID);
			else
				types = model.getAllExampleLipidTypesIntraInDomain(domainID,
						layerID);

			final int[] olds = new int[types.length];

			int reference = -1;
			for (int i = 0; i < types.length; i++) {

				Lipid_3D current = null;
				try {
					if (mode == 1) {
						current = model.getAllExampleLipidsOfTypeExtra(
								types[i], domainID, layerID)[0];

					} else if (mode == 2) {
						current = model.getAllExampleLipidsOfTypeIntra(
								types[i], domainID, layerID)[0];

					} else
						return;
				} catch (final Exception e) {
					return;
				}

				if (current.getPDBcode().equals(code) && !this.initing) {//TODO this must be changed
					reference = i;
					olds[i] = newval;
				} else
					olds[i] = (current.getPercent());

			}

			final int[] newPercents = Cm2_Toolkit.calcPercentages(reference,
					olds);

			for (int i = 0; i < types.length; i++) {

				Lipid_3D current = null;
				try {
					if (mode == 1) {
						current = model.getAllExampleLipidsOfTypeExtra(
								types[i], domainID, layerID)[0];

					} else if (mode == 2) {
						current = model.getAllExampleLipidsOfTypeIntra(
								types[i], domainID, layerID)[0];

					} else
						return;
				} catch (final Exception e) {
					return;
				}

				current.setPercent(newPercents[i]);
				DataPool.setMembraneHasBeenEditedAgain(true);

			}

		}

	}
	
	/**
	 * The ComponentView uses absolute values instead of percental values to compute
     * the lipid distribution.
	 * @param computeAbsoluteValues
	 */
	public void setComputeAbsoluteValues(boolean computeAbsoluteValues) {

		// activate the components associated with absolute values
		this.absolutevalues.removeChangeListener(this);
		this.absolutevalues.setSelected(computeAbsoluteValues);
		this.absolutevalues.addChangeListener(this);
		DataPool.getMembraneModel().setComputeAbsoluteValues(computeAbsoluteValues);
		
		// Get all Micropanels, set number of decimal places
		// and change lipid percentage
//		final JPanel listpanel = ComponentView.this.lists
//				.elementAt(ComponentView.this.tab
//						.getSelectedIndex());

		for (JPanel listpanel:ComponentView.this.lists) {
		
			for (int i = 0; i < listpanel.getComponentCount(); i++) {
				final Component c = listpanel.getComponent(i);
	
				if (c instanceof MicroPanel == false)
					continue;
	
				final MicroPanel mp = (MicroPanel) c;
				if (mp.mol instanceof Lipid_3D == false)
					continue;
				
				Lipid_3D[] molsOfTypeOneSide = (mp.type == MicroPanel.LIPID_EC)?
											DataPool.getMembraneModel()
												.getAllLipidsOfTypeExtra(mp.mol.getPDBcode(),
													mp.mol.getDomainID(), mp.mol.getLayerID()):
											DataPool.getMembraneModel()
												.getAllLipidsOfTypeIntra(mp.mol.getPDBcode(),
													mp.mol.getDomainID(), mp.mol.getLayerID());
												
				Lipid_3D[] allMolsOneSide = (mp.type == MicroPanel.LIPID_EC)?								
											DataPool.getMembraneModel().
												getAllLipidsExtraInDomain(mp.mol.getDomainID(), mp.mol.getLayerID()):
											DataPool.getMembraneModel().
												getAllLipidsIntraInDomain(mp.mol.getDomainID(), mp.mol.getLayerID());
				
				mp.setComputeAbsoluteValues( computeAbsoluteValues,computeAbsoluteValues?molsOfTypeOneSide.length:
												molsOfTypeOneSide.length*100/
													(allMolsOneSide.length<=0?1:allMolsOneSide.length) );
				mp.percentlabel.setVisible(!computeAbsoluteValues);
				mp.revalidate();
			}
			
		}
		
	}
	
	/**
	 * Check if the ComponentView uses absolute values instead of percental values 
	 * to compute the lipid distribution.
	 * @return
	 */
	public boolean getComputeAbsoluteValues() {
		return this.absolutevalues.isSelected();
	}

	/**
	 * 
	 */
	public void updateSpinner() {
		final int spinnerDivisor = (int) Math.pow(10,
				(Integer) this.decimalplaces.getValue());

//		System.out.println("updateSpinner() spinnerDivisor "+spinnerDivisor+" "+this.decimalplaces.getValue());
		
		// Get all Micropanels, set number of decimal places
		// and change lipid percentage
		final JPanel listpanel = this.lists.elementAt(this.tab
				.getSelectedIndex());

		for (int i = 0; i < listpanel.getComponentCount(); i++) {
			final Component c = listpanel.getComponent(i);

			if (c instanceof MicroPanel == false)
				continue;

			final MicroPanel mp = (MicroPanel) c;
			if (mp.mol instanceof Lipid_3D == false)
				continue;

			mp.setSpinnerDivisor(spinnerDivisor);

		}
	}

	private synchronized void updateIt() {
		
//		System.out.println("updateIt");

		this.updating = true;
		
		final MembraneModel model = DataPool.getMembraneModel();
		
		// change this values only if a membrane was loaded
		if (DataPool.getCurrentMembraneFile() != null) {
			
//			System.out.println("- loaded");
			
			this.initing = true;
			
			this.decimalplaces.removeChangeListener(this);
			if (model.getComputeAbsoluteValues()) {
				this.decimalplaces.setValue(0);
				this.oldDecimalPlaces = 0;
			} else {
				this.decimalplaces.setValue(model.getDecimalPlaces());
				this.oldDecimalPlaces = model.getDecimalPlaces();
			}
			this.decimalplaces.addChangeListener(this);
			this.decimalplaces.revalidate();
			
			this.absolutevalues.removeChangeListener(this);
			this.absolutevalues.setSelected(model.getComputeAbsoluteValues());
			this.autosetpercent.setEnabled(!this.absolutevalues.isSelected());
			this.decimalplaces.setEnabled(!this.absolutevalues.isSelected());
			this.setComputeAbsoluteValues(this.absolutevalues.isSelected());
			this.absolutevalues.addChangeListener(this);
			
			this.initing = false;

		} else {
			
//			System.out.println("- edited");
			
			model.setComputeAbsoluteValues(this.absolutevalues.isSelected());
			this.decimalplaces.addChangeListener(this);
		}

		this.lists.clear();

		this.lastindex = this.tab.getSelectedIndex();

		this.tab.removeChangeListener(this);
		this.tab.removeAll();

		// MicroPanel current = null;

		final int layerID = DataPool.getCurrentLayerID();

		final Vector<JScrollPane> scrollpanes = new Vector<JScrollPane>();
		final Vector<MicroDomain_3D> micros = new Vector<MicroDomain_3D>();
		for (int d = 0; d < model.countMicroDomains() + 1; d++) {

			if (d > 0 && model.getMicroDomain(d - 1).layerId != layerID)
				continue;

			MicroDomain_3D currentMicroDomain = null;
			if (d > 0) {
				currentMicroDomain = model.getMicroDomain(d - 1);
				micros.addElement(currentMicroDomain);
			}

			final JPanel listpanel = new JPanel();
			this.lists.add(listpanel);

			// JPanel mainpanel = new JPanel();

			final JScrollPane scrlist = new JScrollPane(listpanel);

			// mainpanel.add(scrlist, BorderLayout.SOUTH);

			scrollpanes.add(scrlist);
			listpanel.setLayout(new BoxLayout(listpanel, BoxLayout.Y_AXIS));
			this.tab.setFont(this.tab.getFont().deriveFont(Font.BOLD));

			final String[] Ptypes = model.getAllProteinTypesInDomain(d < 1 ? 0
					: currentMicroDomain.id, layerID);
			final String[] ECtypes = model
					.getAllExampleLipidTypesExtraInDomain(d < 1 ? 0
							: currentMicroDomain.id, layerID);
			final String[] ICtypes = model
					.getAllExampleLipidTypesIntraInDomain(d < 1 ? 0
							: currentMicroDomain.id, layerID);

			// generate the protein panels
			
			final JPanel p1 = new JPanel();

			p1.setLayout(new BoxLayout(p1, BoxLayout.X_AXIS));
			final JLabel l1 = new JLabel("Proteins");
			l1.setIcon(Cm2_IconDepot.getSpecifiedIcon("CV_Prot"));
			l1.setIconTextGap(5);
			p1.add(l1);
			p1.add(Box.createHorizontalGlue());
			// p1
			// .setBackground(CM2_CONSTANTS.MOLECULE_DEFAULTS.DEFAULTPROTEINCOLOR
			// );
			p1.setBorder(BorderFactory.createRaisedBevelBorder());
			listpanel.add(p1);

			for (int i = 0; i < Ptypes.length; i++) {
				final Protein_3D[] prots = model.getAllProteinsOfType(
						Ptypes[i], d < 1 ? 0 : currentMicroDomain.id, layerID);

				final MicroPanel mp = new MicroPanel(prots[0],
						MicroPanel.PROTEIN);
				mp.setNumber(prots.length);
				listpanel.add(mp);
				if (this.selectedPanel != null
						&& this.arrayContains(prots, this.listener
								.getCurrentSelection())) {
					mp.setSelected(true);
					// current = mp;
				}
			}

			listpanel.add(Box.createVerticalGlue());

			// generate the EC lipid panels
			
			final JPanel p2 = new JPanel();

			p2.setLayout(new BoxLayout(p2, BoxLayout.X_AXIS));
			final JLabel l2 = new JLabel("Sample Lipids EC");
			l2.setIcon(Cm2_IconDepot.getSpecifiedIcon("CV_LipEC"));
			l2.setIconTextGap(5);
			p2.add(l2);
			p2.add(Box.createHorizontalGlue());
			// p2.setBackground(CM2_CONSTANTS.MOLECULE_DEFAULTS.DEFAULTLIPIDCOLOR
			// );
			p2.setBorder(BorderFactory.createRaisedBevelBorder());
			listpanel.add(p2);

			for (int i = 0; i < ECtypes.length; i++) {
//				final Lipid_3D[] lips = model.getAllExampleLipidsOfTypeExtra(
//						ECtypes[i], d, layerID);
				final Lipid_3D[] lips = model.getAllExampleLipidsOfTypeExtra(
						ECtypes[i], d < 1 ? 0 : currentMicroDomain.id, layerID);

				final MicroPanel mp = new MicroPanel(lips[0],
						MicroPanel.LIPID_EC);
				mp.setNumber(lips[0].getPercent());
				listpanel.add(mp);
				if (this.selectedPanel != null
						&& this.selectedPanel.domainID == (d < 1 ? 0
								: currentMicroDomain.id)
						&& this.selectedPanel.mol.getPDBcode().equals(
								mp.mol.getPDBcode())
						&& this.selectedPanel.type == MicroPanel.LIPID_EC) {
					mp.setSelected(true);
					// current = mp;
				}
			}

			listpanel.add(Box.createVerticalGlue());

			// generate the IC lipid panels
			
			final JPanel p3 = new JPanel();

			p3.setLayout(new BoxLayout(p3, BoxLayout.X_AXIS));
			final JLabel l3 = new JLabel("Sample Lipids IC");
			l3.setIcon(Cm2_IconDepot.getSpecifiedIcon("CV_LipIC"));
			l3.setIconTextGap(5);
			p3.add(l3);
			p3.add(Box.createHorizontalGlue());
			// p3.setBackground(CM2_CONSTANTS.MOLECULE_DEFAULTS.DEFAULTLIPIDCOLOR
			// );
			p3.setBorder(BorderFactory.createRaisedBevelBorder());
			listpanel.add(p3);

			for (int i = 0; i < ICtypes.length; i++) {
				final Lipid_3D[] lips = model.getAllExampleLipidsOfTypeIntra(
						ICtypes[i], d < 1 ? 0 : currentMicroDomain.id, layerID);

				final MicroPanel mp = new MicroPanel(lips[0],
						MicroPanel.LIPID_IC);
				mp.setNumber(lips[0].getPercent());

				listpanel.add(mp);
				if (this.selectedPanel != null
						&& this.selectedPanel.domainID == (d < 1 ? 0
								: currentMicroDomain.id)
						&& this.selectedPanel.mol.getPDBcode().equals(
								mp.mol.getPDBcode())
						&& this.selectedPanel.type == MicroPanel.LIPID_IC) {
					mp.setSelected(true);
					// current = mp;
				}
			}
			listpanel.add(Box.createVerticalGlue());

			// scrlist.validate();
		}

		for (int i = 0; i < scrollpanes.size(); i++) {
			if (i == 0)
				this.tab.addTab("<html>D<br>E<br>F<br>A<br>U<br>L<br>T</html>",
						scrollpanes.get(i));
			else
				this.tab.addTab("<html><br>" + micros.get(i - 1).id
						+ "<br></html>", scrollpanes.get(i));
		}
		if (this.lastindex < this.tab.getTabCount())
			this.tab.setSelectedIndex(this.lastindex);
		else
			this.tab.setSelectedIndex(this.tab.getTabCount() - 1);

		this.tab.addChangeListener(this);

		// if (current != null) {
		// current.scrollRectToVisible(new Rectangle(0, 0, 0, 0));
		// }

		if (this.tab.getTabCount() <= 1)
			this.highlightMicroDomain(0);
		this.updating = false;
	}

	/**
	 * updates all tables
	 * 
	 */
	public synchronized void upDateAll() {

		final Thread t = new Thread() {
			public void run() {
				ComponentView.this.updateIt();
			}
		};
		t.setPriority(Thread.MIN_PRIORITY);
		SwingUtilities.invokeLater(t);

	}

	private boolean arrayContains(final Protein_3D[] prots,
			final Vector<Molecule_3D> vector) {
		if (vector == null)
			return false;
		for (int i = 0; i < prots.length; i++) {
			if (vector.contains(prots[i]))
				return true;
		}
		return false;
	}

	public synchronized void highlightMicroDomain(final int domainID) {

		final Thread t = new Thread() {
			public void run() {
				while (ComponentView.this.updating) {
					try {
						Thread.sleep(50);
					} catch (final InterruptedException e) {
						e.printStackTrace();
					}
				}
				// final int id = domainID == -1 ? 0 : domainID;
				//			

				// SwingUtilities.invokeLater(new Thread() {
				// public void run() {

				final ChangeListener cl = ComponentView.this.tab
						.getChangeListeners()[0];
				ComponentView.this.tab.removeChangeListener(cl);

				if (domainID < 1)
					ComponentView.this.tab.setSelectedIndex(0);
				else {
					for (int i = 0; i < ComponentView.this.tab.getTabCount(); i++) {

						if (ComponentView.this.tab.getTitleAt(i).replaceAll(
								"<html>", "").replaceAll("</html>", "")
								.replaceAll("<br>", "").equals("" + domainID))
							ComponentView.this.tab.setSelectedIndex(i);
					}
				}

				ComponentView.this.tab.addChangeListener(cl);
				ComponentView.this.domainPanel.setDomain(domainID);
				// }
				// });

			}
		};
		t.setPriority(Thread.MIN_PRIORITY);
		t.start();

	}

	public int getHighlightedDomainID() {
		return this.domainPanel.domainID;
	}

	public void select(final Molecule_3D mol, final int type) {

		if (this.selectedPanel != null) {
			if (this.selectedPanel.isSelected)
				this.selectedPanel.setSelected(false);
			this.selectedPanel = null;

		}

		if (mol == null) {

			return;
		}

		try {
			String s = this.tab.getTitleAt(this.tab.getSelectedIndex());
			s = s.replaceAll("<html>", "").replaceAll("</html>", "")
					.replaceAll("<br>", "").trim();
			if (Integer.parseInt(s) != mol.getDomainID())
				this.listener.highlightMicroDomain(mol.getDomainID());
		} catch (final Exception e) {
			this.listener.highlightMicroDomain(-1);
		}

		final JScrollPane scroll = (JScrollPane) this.tab
				.getSelectedComponent();

		final JPanel listpanel = (JPanel) (scroll.getViewport().getComponent(0));
		for (int i = 0; i < listpanel.getComponentCount(); i++) {
			final Component c = listpanel.getComponent(i);
			if (c instanceof MicroPanel) {
				final MicroPanel mp = (MicroPanel) c;

				if (mp.type == type
						&& mp.mol.getPDBcode().equalsIgnoreCase(
								mol.getPDBcode())) {

					mp.setSelected(true);

					this.selectedPanel = mp;
				} else {
					if (mp.isSelected)
						mp.setSelected(false);
				}
			}

		}
		if (this.selectedPanel != null)
			this.selectedPanel.scrollRectToVisible(new Rectangle(0, 0, 0, 0));

	}

	class MicroPanel extends JPanel implements MouseListener, ChangeListener,
			ActionListener, MouseMotionListener {

		/**
		 * 
		 */
		private static final long serialVersionUID = 1L;

		final public static int PROTEIN = 0;

		final public static int LIPID_EC = 1;

		final public static int LIPID_IC = 2;

		public int domainID;

		public int type;

		public Molecule_3D mol;

		private boolean isSelected = false;

		public double number = 0;

		JLabel textlabel = new JLabel("", JLabel.LEADING);

		JLabel numberlabel = new JLabel();

		JLabel percentlabel = new JLabel("%");

		private final JPanel north = new JPanel() {
			public void paintComponent(Graphics g) {

				BufferedImage bi = new BufferedImage(this.getWidth(), this
						.getHeight(), BufferedImage.TYPE_INT_BGR);

				final Graphics2D g2 = bi.createGraphics();

				final GradientPaint gp = new GradientPaint(0, 0, this
						.getBackground(), 0, this.getHeight()
						+ this.getHeight() / 2, new JPanel().getBackground());
				g2.setPaint(gp);

				final Rectangle2D rectangle = new Rectangle2D.Double(0, 0, this
						.getWidth(), this.getHeight());
				g2.fill(rectangle);

				g.drawImage(bi, 0, 0, null);

				// super.paintComponent(g);

			}
		};

		private final JToolBar south = new JToolBar();

		private final SpinnerNumberModel spinnerModel = new SpinnerNumberModel(
				0d, 0d, ComponentView.this.getComputeAbsoluteValues()?Integer.MAX_VALUE:100d, 1d);

		private final JSpinner spinner = new JSpinner(this.spinnerModel);

		private final JButton removeBtn = new JButton();

		private final JButton popupBtn = new JButton(">");

		private JMenuItem rotateBtn;

		private JMenuItem colorBtn;
		
		private JMenuItem atomColorBtn;

		private JMenuItem profileBtn;

		// private MyMouseAdapter mAdapter = new MyMouseAdapter();

		public MicroPanel(final Molecule_3D mol, final int type) {
			super();
			final MembraneModel model = DataPool.getMembraneModel();
			this.textlabel.setText(mol.getPDBcode().length() > 7 ? Cm2_Toolkit
					.extendRight(mol.getPDBcode(), 7)
					+ "..." : mol.getPDBcode());
			this.domainID = mol.getDomainID();
			this.type = type;
			this.mol = mol;

			this.textlabel.setIconTextGap(5);

			this.north.setBackground(mol.getColor());

			// south.setBackground(mol.getColor());

			this.setBorder(BorderFactory.createLineBorder(
					ComponentView.this.tab.getBackground(), 3));

			this.setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));

			this.textlabel.addMouseListener(this);
			this.north.addMouseListener(this);
			this.textlabel.addMouseMotionListener(this);
			this.north.addMouseMotionListener(this);

			this.add(this.north);
			this.add(this.south);

			this.north.setLayout(new BoxLayout(this.north, BoxLayout.X_AXIS));
			this.south.setLayout(new BoxLayout(this.south, BoxLayout.X_AXIS));

			this.north.add(this.textlabel);
			this.north.add(Box.createHorizontalGlue());
			this.north.add(this.numberlabel);

			this.south.setFloatable(false);

			this.south.add(this.spinner);
			this.south.add(this.percentlabel);

			this.south.add(this.removeBtn);
			this.south.add(this.popupBtn);
			this.south.add(Box.createHorizontalGlue());

			this.removeBtn.setIcon(Cm2_IconDepot.getSpecifiedIcon("CV_remove"));

			this.removeBtn.setToolTipText("Remove this type");

			this.popupBtn.setToolTipText("Options...");

			this.popupBtn.setPreferredSize(new Dimension(15, this.removeBtn
					.getPreferredSize().height));

			// this.popupBtn.setBorder(BorderFactory.createEtchedBorder());
			// this.removeBtn.setBorder(BorderFactory.createEtchedBorder());

			if (this.mol instanceof Lipid_3D)
				this.spinner.setToolTipText("Adjust percentages");
			else
				this.spinner.setToolTipText("Change count");

			this.setSpinnerDivisor((int) Math.pow(10, model.getDecimalPlaces()));

			this.spinner.setFont(this.spinner.getFont().deriveFont(
					this.spinner.getFont().getSize2D() + 3));
			this.spinner.setFont(this.spinner.getFont().deriveFont(Font.BOLD));

			this.numberlabel.setFont(this.numberlabel.getFont().deriveFont(
					this.numberlabel.getFont().getSize2D() + 2));

			this.textlabel.setFont(this.textlabel.getFont().deriveFont(
					this.textlabel.getFont().getSize2D() + 1));

			this.textlabel.setForeground(Color.black);
			this.numberlabel.setForeground(Color.black);
			this.numberlabel.setBorder(BorderFactory.createRaisedBevelBorder());
			this.spinner.setPreferredSize(new Dimension(60, this.spinner
					.getPreferredSize().height));
			this.spinner.setMaximumSize(new Dimension(70, this.spinner
					.getPreferredSize().height));
			this.spinner.setMinimumSize(new Dimension(70, this.spinner
					.getPreferredSize().height));

			this.spinner.addChangeListener(this);

			this.removeBtn.addActionListener(this);
			this.popupBtn.addActionListener(this);

			this.removeBtn.setOpaque(false);
			this.popupBtn.setOpaque(false);

			this.setSelected(false);

			// DataPool.addMicroPanel(this);

		}

		public void setSelected(final boolean b) {
			if (!this.isEnabled())
				return;
			this.isSelected = b;
			if (!b) {

				this.addMouseListener(this);
				this.textlabel.setIcon(new ImageIcon(this.mol.getImageSmall()));

				this.numberlabel.setVisible(true);
				this.south.setVisible(false);
				this.setBorder(BorderFactory.createLineBorder(
						ComponentView.this.tab.getBackground(), 3));

				if (this.type == MicroPanel.PROTEIN || ComponentView.this.getComputeAbsoluteValues()) {

					this.percentlabel.setVisible(false);
				} else {

					this.percentlabel.setVisible(true);
				}
				this.textlabel.setToolTipText(null);
				this.north.setToolTipText(null);
				return;
			}

			this.removeMouseListener(this);
			this.numberlabel.setVisible(false);

			this.south.setVisible(true);

			this.textlabel.setIcon(new ImageIcon(this.mol.getImageLarge()));

			if (this.mol instanceof Lipid_3D) {
				this.textlabel
						.setToolTipText("<html><font color=\"FFC800\">Hold down left mousebutton to drag an instance into the membrane!</font></html>");
				this.north
						.setToolTipText("<html><font color=\"FFC800\">Hold down left mousebutton to drag an instance into the membrane!</font></html>");
			}
			this.setBorder(BorderFactory.createLineBorder(
					CM2_CONSTANTS.MOLECULE_DEFAULTS.HIGLIGHTING_COLOR_DARK, 3));

			this.scrollRectToVisible(this.getBounds());

		}

		public void setEnabled(final boolean b) {

			this.spinner.setEnabled(b);

			this.removeBtn.setEnabled(b);

			// this.rotateBtn.setEnabled(b);

			// this.colorBtn.setEnabled(b);

			// this.profileBtn.setEnabled(b);

			this.popupBtn.setEnabled(b);

			if (b) {

				this.textlabel.addMouseListener(this);
				this.north.addMouseListener(this);
				this.textlabel.addMouseMotionListener(this);
				this.north.addMouseMotionListener(this);
				this.numberlabel.addMouseListener(this);
				this.addMouseListener(this);

			} else {
				this.textlabel.removeMouseListener(this);
				this.north.removeMouseListener(this);
				this.textlabel.removeMouseMotionListener(this);
				this.north.removeMouseMotionListener(this);
				this.numberlabel.removeMouseListener(this);
				this.removeMouseListener(this);
			}

		}

		private void showPopup(Component invoker, Point position) {
			JPopupMenu pop = new JPopupMenu();

			this.profileBtn = new JMenuItem("Select Profile");
			this.colorBtn = new JMenuItem("Change Color");
			this.atomColorBtn = new JMenuItem(mol.isUsingAtomColors()?"Use actual Color":"Use Atom Colors");
			this.rotateBtn = new JMenuItem("Change default Alignment");
			
			pop.add(this.profileBtn);
			pop.add(this.colorBtn);
			pop.add(this.atomColorBtn);
			if (this.mol instanceof Lipid_3D)
				pop.add(this.rotateBtn);

			pop.setBackground(this.mol.getColor());
			this.rotateBtn.setIcon(Cm2_IconDepot
					.getSpecifiedIcon("CV_alignLipid"));
			this.colorBtn.setIcon(Cm2_IconDepot
					.getSpecifiedIcon("CV_changeColor"));
			this.profileBtn.setIcon(Cm2_IconDepot
					.getSpecifiedIcon("CG_Manage_Profiles"));
			
			if (this.mol.isUsingAtomColors())
				this.atomColorBtn.setIcon(Cm2_IconDepot
						.getSpecifiedIcon("CV_useColor"));
			else
				this.atomColorBtn.setIcon(Cm2_IconDepot
					.getSpecifiedIcon("CV_useAtomColors"));
			
			this.rotateBtn
					.setToolTipText("Edit the default alignment if this type");
			this.colorBtn
					.setToolTipText("Change the color that represents this type");
			this.atomColorBtn
					.setToolTipText(mol.isUsingAtomColors()?
							"Use the actual color chosen for atomic view modes":
							"Use the original atom colouring for atomic view modes");
			this.profileBtn
					.setToolTipText("Set or Edit the Molecule Profile of this type");

			this.rotateBtn.addActionListener(this);
			this.colorBtn.addActionListener(this);
			this.atomColorBtn.addActionListener(this);
			this.profileBtn.addActionListener(this);

			// this.rotateBtn.setOpaque(false);
			// this.colorBtn.setOpaque(false);
			// this.profileBtn.setOpaque(false);

			pop.show(invoker, position.x, position.y);
		}

		public void actionPerformed(final ActionEvent arg0) {
			final int layerID = DataPool.getCurrentLayerID();

			// generate the PopUp
			if (arg0.getSource() == this.popupBtn) {
				this.showPopup(this.popupBtn, new Point(this.popupBtn
						.getWidth() / 2, this.popupBtn.getHeight() / 2));

			}
			
			// remove button was pressed
			if (arg0.getSource() == this.removeBtn) {

				if (ComponentView.this.promptdelete.isSelected()) {

					String category;
					if (this.type == MicroPanel.PROTEIN)
						category = "Protein";
					else
						category = "Lipid";
					if (JOptionPane.showConfirmDialog(this.removeBtn,
							"Are you sure?", "Removing " + category,
							JOptionPane.YES_NO_OPTION,
							JOptionPane.QUESTION_MESSAGE) == JOptionPane.NO_OPTION)
						return;

				}

				final String code = this.mol.getPDBcode();

				if (this.type == MicroPanel.PROTEIN) {
					final Protein_3D[] mols = DataPool.getMembraneModel()
							.getAllProteinsOfType(code, this.domainID, layerID);
					ComponentView.this.listener.removeProteinsFrom3D(mols);
					for (int i = 0; i < mols.length; i++) {

						DataPool.getMembraneModel().deleteProtein(mols[i]);
						ComponentView.this.listener
								.setShowAxis(ComponentView.this.listener
										.areAxisVisible());
					}
				} else if (this.type == MicroPanel.LIPID_EC) {

					final Lipid_3D[] mols = DataPool.getMembraneModel()
							.getAllExampleLipidsOfTypeExtra(code,
									this.domainID, layerID);
					for (int i = 0; i < mols.length; i++) {
						DataPool.getMembraneModel().deleteExampleLipidExtra(
								mols[i]);
					}

					if (ComponentView.this.syncEcIc.isSelected()) {

						final Lipid_3D[] mols2 = DataPool.getMembraneModel()
								.getAllExampleLipidsOfTypeIntra(code,
										this.domainID, layerID);
						for (int j = 0; j < mols2.length; j++) {
							DataPool.getMembraneModel()
									.deleteExampleLipidIntra(mols2[j]);
						}

					}

					String[] lips = DataPool.getMembraneModel()
							.getAllExampleLipidTypesExtraInDomain(
									this.domainID, layerID);

					if (lips.length > 0) {

						final String newcode = lips[0];
						final int newval = DataPool.getMembraneModel()
								.getAllExampleLipidsOfTypeExtra(newcode,
										this.domainID, layerID)[0].getPercent();

						if (ComponentView.this.autosetpercent.isSelected() && 
								!ComponentView.this.absolutevalues.isSelected()) {

							ComponentView.this.setPercentages(1, newval,
									newcode, this.domainID);

						}

					}

					if (ComponentView.this.syncEcIc.isSelected()) {

						lips = DataPool.getMembraneModel()
								.getAllExampleLipidTypesIntraInDomain(
										this.domainID, layerID);

						if (lips.length > 0) {

							final String newcode = lips[0];
							final int newval = DataPool.getMembraneModel()
									.getAllExampleLipidsOfTypeIntra(newcode,
											this.domainID, layerID)[0]
									.getPercent();

							if (ComponentView.this.autosetpercent.isSelected() && 
									!ComponentView.this.absolutevalues.isSelected()) {

								ComponentView.this.setPercentages(2, newval,
										newcode, this.domainID);

							}

						}

					}

				} else {

					final Lipid_3D[] mols = DataPool.getMembraneModel()
							.getAllExampleLipidsOfTypeIntra(code,
									this.domainID, layerID);
					for (int i = 0; i < mols.length; i++) {
						DataPool.getMembraneModel().deleteExampleLipidIntra(
								mols[i]);
					}

					if (ComponentView.this.syncEcIc.isSelected()) {

						final Lipid_3D[] mols2 = DataPool.getMembraneModel()
								.getAllExampleLipidsOfTypeExtra(code,
										this.domainID, layerID);
						for (int j = 0; j < mols2.length; j++) {
							DataPool.getMembraneModel()
									.deleteExampleLipidExtra(mols2[j]);
						}

					}

					String[] lips = DataPool.getMembraneModel()
							.getAllExampleLipidTypesIntraInDomain(
									this.domainID, layerID);

					if (lips.length > 0) {

						final String newcode = lips[0];

						final int newval = DataPool.getMembraneModel()
								.getAllExampleLipidsOfTypeIntra(newcode,
										this.domainID, layerID)[0].getPercent();

						if (ComponentView.this.autosetpercent.isSelected() && 
								!ComponentView.this.absolutevalues.isSelected()) {

							ComponentView.this.setPercentages(2, newval,
									newcode, this.domainID);

						}

					}

					if (ComponentView.this.syncEcIc.isSelected()) {

						lips = DataPool.getMembraneModel()
								.getAllExampleLipidTypesExtraInDomain(
										this.domainID, layerID);

						if (lips.length > 0) {

							final String newcode = lips[0];
							final int newval = DataPool.getMembraneModel()
									.getAllExampleLipidsOfTypeExtra(newcode,
											this.domainID, layerID)[0]
									.getPercent();

							if (ComponentView.this.autosetpercent.isSelected() && 
									!ComponentView.this.absolutevalues.isSelected()) {

								ComponentView.this.setPercentages(1, newval,
										newcode, this.domainID);

							}

						}

					}

				}
				DataPool.getMembraneModel().organizeDomains();
				ComponentView.this.listener.refreshViews(true, true, true,
						false, false, true, true);

			// color button was pressed
			} else if (arg0.getSource() == this.colorBtn) {
				
				final Point pointAsBoolean = new Point(0, 0);
				final JColorChooser colorChooser = new JColorChooser();
				final JDialog dialog = JColorChooser.createDialog(null,
						"Pick a Color", true, colorChooser,
						new ActionListener() {

							public void actionPerformed(final ActionEvent arg0) {

								pointAsBoolean.x = 1;

							}

						}, null);

				Color newColor;

				colorChooser.setColor(this.mol.getColor());
				dialog.setVisible(true);

				newColor = colorChooser.getColor();

				if (pointAsBoolean.x == 0
						|| newColor.equals(this.mol.getColor()))
					return;

				String code = null;

				boolean atomicRenderMode = (DataPool.J3DSettings.overallRenderMode==CM2_CONSTANTS.RENDERMODE.RENDER_MODE_POLYGON ||
											DataPool.J3DSettings.overallRenderMode==CM2_CONSTANTS.RENDERMODE.RENDER_MODE_COLORED_STICKS)?
											false:true;
				
				listener.dockScreen(false);
				
				code = this.mol.getPDBcode();
				Molecule_3D[] mols;
				if (this.type == MicroPanel.PROTEIN)
					mols = DataPool.getMembraneModel().getAllProteinsOfType(
							code, this.domainID, layerID);
				else if (this.type == MicroPanel.LIPID_EC)
					mols = DataPool.getMembraneModel().getAllLipidsOfTypeExtra(
							code, this.domainID, layerID);
				else
					mols = DataPool.getMembraneModel().getAllLipidsOfTypeIntra(
							code, this.domainID, layerID);

				for (int i = 0; i < mols.length; i++) {

					mols[i].setColor(newColor);
					if (atomicRenderMode) mols[i].updateAtomColors();

				}

				Molecule_3D[] mols2;
				if (this.type == MicroPanel.LIPID_EC
						&& ComponentView.this.syncEcIc.isSelected())
					mols2 = DataPool.getMembraneModel()
							.getAllLipidsOfTypeIntra(code, this.domainID,
									layerID);
				else if (this.type == MicroPanel.LIPID_IC
						&& ComponentView.this.syncEcIc.isSelected())
					mols2 = DataPool.getMembraneModel()
							.getAllLipidsOfTypeExtra(code, this.domainID,
									layerID);
				else
					mols2 = null;

				if (mols2 != null) {
					for (int i = 0; i < mols2.length; i++) {

						mols2[i].setColor(newColor);
						if (atomicRenderMode) mols2[i].updateAtomColors();

					}
				}

				Molecule_3D[] mols3;

				if (this.type == MicroPanel.LIPID_EC)
					mols3 = DataPool.getMembraneModel()
							.getAllExampleLipidsOfTypeExtra(code,
									this.domainID, layerID);
				else if (this.type == MicroPanel.LIPID_IC)
					mols3 = DataPool.getMembraneModel()
							.getAllExampleLipidsOfTypeIntra(code,
									this.domainID, layerID);
				else
					mols3 = null;

				if (mols3 != null) {

					for (int i = 0; i < mols3.length; i++) {

						mols3[i].setColor(newColor);
						if (atomicRenderMode) mols3[i].updateAtomColors();

					}
				}
				Molecule_3D[] mols4;
				if (this.type == MicroPanel.LIPID_EC
						&& ComponentView.this.syncEcIc.isSelected())
					mols4 = DataPool.getMembraneModel()
							.getAllExampleLipidsOfTypeIntra(code,
									this.domainID, layerID);
				else if (this.type == MicroPanel.LIPID_IC
						&& ComponentView.this.syncEcIc.isSelected())
					mols4 = DataPool.getMembraneModel()
							.getAllExampleLipidsOfTypeExtra(code,
									this.domainID, layerID);
				else
					mols4 = null;

				if (mols4 != null) {
					for (int i = 0; i < mols4.length; i++) {

						mols4[i].setColor(newColor);
						if (atomicRenderMode) mols4[i].updateAtomColors();

					}
				}
				
				listener.dockScreen(true);
				ComponentView.this.listener.refreshViews(true, false, false,
						true, false, false, true);
				DataPool.setMembraneHasBeenEditedAgain(true);
				
			// atom color button was pressed
			} else if (arg0.getSource() == this.atomColorBtn) {

				listener.dockScreen(false);
				
				String code = null;

				code = this.mol.getPDBcode();
				Molecule_3D[] mols;
				if (this.type == MicroPanel.PROTEIN)
					mols = DataPool.getMembraneModel().getAllProteinsOfType(
							code, this.domainID, layerID);
				else if (this.type == MicroPanel.LIPID_EC)
					mols = DataPool.getMembraneModel().getAllLipidsOfTypeExtra(
							code, this.domainID, layerID);
				else
					mols = DataPool.getMembraneModel().getAllLipidsOfTypeIntra(
							code, this.domainID, layerID);

				for (int i = 0; i < mols.length; i++) {
					
					mols[i].setUseAtomColors(!mols[i].isUsingAtomColors());
					mols[i].updateAtomColors();

				}

				Molecule_3D[] mols2;
				if (this.type == MicroPanel.LIPID_EC
						&& ComponentView.this.syncEcIc.isSelected())
					mols2 = DataPool.getMembraneModel()
							.getAllLipidsOfTypeIntra(code, this.domainID,
									layerID);
				else if (this.type == MicroPanel.LIPID_IC
						&& ComponentView.this.syncEcIc.isSelected())
					mols2 = DataPool.getMembraneModel()
							.getAllLipidsOfTypeExtra(code, this.domainID,
									layerID);
				else
					mols2 = null;

				if (mols2 != null) {
					for (int i = 0; i < mols2.length; i++) {

						mols2[i].setUseAtomColors(!mols2[i].isUsingAtomColors());
						mols2[i].updateAtomColors();

					}
				}

				Molecule_3D[] mols3;

				if (this.type == MicroPanel.LIPID_EC)
					mols3 = DataPool.getMembraneModel()
							.getAllExampleLipidsOfTypeExtra(code,
									this.domainID, layerID);
				else if (this.type == MicroPanel.LIPID_IC)
					mols3 = DataPool.getMembraneModel()
							.getAllExampleLipidsOfTypeIntra(code,
									this.domainID, layerID);
				else
					mols3 = null;

				if (mols3 != null) {

					for (int i = 0; i < mols3.length; i++) {

						mols3[i].setUseAtomColors(!mols3[i].isUsingAtomColors());
						mols3[i].updateAtomColors();

					}
				}
				Molecule_3D[] mols4;
				if (this.type == MicroPanel.LIPID_EC
						&& ComponentView.this.syncEcIc.isSelected())
					mols4 = DataPool.getMembraneModel()
							.getAllExampleLipidsOfTypeIntra(code,
									this.domainID, layerID);
				else if (this.type == MicroPanel.LIPID_IC
						&& ComponentView.this.syncEcIc.isSelected())
					mols4 = DataPool.getMembraneModel()
							.getAllExampleLipidsOfTypeExtra(code,
									this.domainID, layerID);
				else
					mols4 = null;

				if (mols4 != null) {
					for (int i = 0; i < mols4.length; i++) {

						mols4[i].setUseAtomColors(!mols4[i].isUsingAtomColors());
						mols4[i].updateAtomColors();

					}
				}
				
				listener.dockScreen(true);
				ComponentView.this.listener.refreshViews(true, false, false,
						true, false, false, true);
				DataPool.setMembraneHasBeenEditedAgain(true);
				
			// rotate button was pressed
			} else if (arg0.getSource() == this.rotateBtn) {

				String code = null;

				code = this.mol.getPDBcode();

				if (this.type == MicroPanel.LIPID_EC) {

					final Lipid_3D lip = DataPool.getMembraneModel()
							.getAllExampleLipidsOfTypeExtra(code,
									this.domainID, layerID)[0];

					ComponentView.this.listener.alignExampleLipid(lip, 0);
					return;
				}

				else {

					final Lipid_3D lip = DataPool.getMembraneModel()
							.getAllExampleLipidsOfTypeIntra(code,
									this.domainID, layerID)[0];

					ComponentView.this.listener.alignExampleLipid(lip, 1);
					return;
				}

			// profile button was pressed
			} else if (arg0.getSource() == this.profileBtn) {

				listener.openMoleculeEditorFromCV(this.mol);

			}

			// DataPool.setMicroPanels(lists);

		}

		/**
		 * Convert percentages is used to remove decimal places if the number
		 * of decimal places was changed.
		 * @param percent
		 * @return double percentages to convert
		 */
		private int convertPercentage(final double percent) {

			String oldval = String.valueOf(percent);
			String ganzeZahl;

			if (this.type != MicroPanel.PROTEIN && !ComponentView.this.getComputeAbsoluteValues()) {

				int k = 0;
				while ((oldval.charAt(k) != '.') && (k != oldval.length())) {
					k++;
				}

				if (k == 0) {
					return (int) percent;
				}

				// concat the part before the '.' and after it to one number
				ganzeZahl = oldval.substring(0, k);
				final int n = (Integer) ComponentView.this.decimalplaces
						.getValue();
				final int m = oldval.substring(k + 1).length();
				if (n > m) {
					for (int i = 0; i < n - m; i++) {
						oldval += '0';
					}
				}

				if (Integer.parseInt(ComponentView.this.decimalplaces
						.getValue().toString()) >= 1) {

					final String decimalVal = ComponentView.this.decimalplaces
							.getValue().toString();
					ganzeZahl += oldval.substring(k + 1, k + 1
							+ Integer.parseInt(decimalVal));
				}
				
			} else {

				ganzeZahl = oldval.substring(0, oldval.indexOf("."));

			}

			return Integer.parseInt(ganzeZahl);
			
		}

		/**
		 * Change the divisor/step size of the spinner.
		 * @param int divisor
		 */
		public void setSpinnerDivisor(final int divisor) {

			if (this.type != MicroPanel.PROTEIN && !ComponentView.this.getComputeAbsoluteValues()) {
				double stepSize = (1 / (double) divisor);

				((SpinnerNumberModel)this.spinner.getModel()).setStepSize(stepSize);
				
			}
			
		}

		/**
		 * Get the actual value of the spinner.
		 * @return double
		 */
		public double getSpinnerValue() {
			return Double.parseDouble(this.spinner.getValue().toString());
		}

		/**
		 * If the spinner was pressed, the protein or lipid values are changed 
		 * in this method.
		 */
		public void stateChanged(final ChangeEvent arg0) {
			
			// skip this method during initializing (if a membrane is loaded); otherwise there may
			// be problems with the former model, especially if it uses the same PDB codes
			if (initing)
				return;
			
			final int newval = this.convertPercentage(Double
					.parseDouble(this.spinner.getValue().toString()));

			int oldval;

			final MembraneModel model = DataPool.getMembraneModel();

			final int layerID = DataPool.getCurrentLayerID();

			if (this.type == MicroPanel.PROTEIN) {

				oldval = model.getAllProteinsOfType(this.mol.getPDBcode(),
						this.domainID, layerID).length;

			} else if (this.type == MicroPanel.LIPID_EC) {

				oldval = model.getAllExampleLipidsOfTypeExtra(this.mol
						.getPDBcode(), this.domainID, layerID)[0].getPercent();
			} else {

				oldval = model.getAllExampleLipidsOfTypeIntra(this.mol
						.getPDBcode(), this.domainID, layerID)[0].getPercent();
			}

			if (oldval == newval)
				return;

			int diff = newval - oldval;

			int addedOnes = -1;

			// in case of differences smaller than one complete molecule
			if (diff < 0) {
				// showLipidPercentage();
				diff *= -1;
				if (this.type == MicroPanel.PROTEIN) {

					final Vector<Protein_3D> prots2delete = new Vector<Protein_3D>();
					for (int i = 0; i < diff; i++) {
						final Protein_3D[] prots = model.getAllProteinsOfType(
								this.mol.getPDBcode(), this.domainID, layerID);
						final Protein_3D prot2delete = prots[(new Random()
								.nextInt(prots.length))];
						prots2delete.add(prot2delete);
						model.deleteProtein(prot2delete);
					}
					ComponentView.this.listener
							.removeProteinsFrom3D(prots2delete
									.toArray(new Protein_3D[0]));

					DataPool.getMembraneModel().organizeDomains();
					ComponentView.this.listener.refreshViews(true, true, false,
							true, false, true, false);
					ComponentView.this.listener
							.setShowAxis(ComponentView.this.listener
									.areAxisVisible());
					ComponentView.this.listener.selectProteinsByCode(this.mol
							.getPDBcode(), this.domainID);
				} else if (this.type > MicroPanel.PROTEIN) {

					ComponentView.this.setPercentages(this.type, newval,
							this.mol.getPDBcode(), this.domainID);

					// synchronize membrane sides
					if (this.type == MicroPanel.LIPID_EC
							&& ComponentView.this.syncEcIc.isSelected()) {
						ComponentView.this.setPercentages(MicroPanel.LIPID_IC, newval, this.mol
								.getPDBcode(), this.domainID);
					} else if (this.type == MicroPanel.LIPID_IC
							&& ComponentView.this.syncEcIc.isSelected()) {
						ComponentView.this.setPercentages(MicroPanel.LIPID_EC, newval, this.mol
								.getPDBcode(), this.domainID);
					}
					ComponentView.this.upDatePercentagesOnly();

				}
			// in case of differences of at least one complete molecule
			} else {
				// showLipidPercentage();
				if (this.type == MicroPanel.PROTEIN) {

					addedOnes = 0;

					final boolean axiswerevisible = ComponentView.this.listener
							.areAxisVisible();
					if (axiswerevisible)
						ComponentView.this.listener.setShowAxis(false);
					// System.out.println("Difference (new<->old) : " + diff);
					while (addedOnes < diff) {
						final Protein_3D tmp = (Protein_3D) model
								.getAllProteinsOfType(this.mol.getPDBcode(),
										this.domainID, layerID)[0];
						int modelnumber = tmp.getModelSet();
						try {
							modelnumber = DatabaseView.ModelByFile.get(
									tmp.getPDBFile().getAbsolutePath())
									.intValue();
						} catch (final Exception e) {

						}

						if (ComponentView.this.listener
								.addANewProteinAtRandomPosition(this.mol
										.getPDBcode(), tmp.getPDBFile()
										.getPath(), modelnumber, this.domainID) == false) {

							break;
						}

						addedOnes++;
					}
					DataPool.getMembraneModel().organizeDomains();

					if (axiswerevisible)
						ComponentView.this.listener.setShowAxis(true);

					ComponentView.this.listener.refreshViews(true, true, false,
							true, false, true, false);
					ComponentView.this.listener.selectProteinsByCode(this.mol
							.getPDBcode(), this.domainID);
				
				} else if (this.type > MicroPanel.PROTEIN) {

					ComponentView.this.setPercentages(this.type, newval,
							this.mol.getPDBcode(), this.domainID);

					// synchronize membrane sides
					if (this.type == MicroPanel.LIPID_EC
							&& ComponentView.this.syncEcIc.isSelected()) {
						ComponentView.this.setPercentages(MicroPanel.LIPID_IC, newval, this.mol
								.getPDBcode(), this.domainID);
					} else if (this.type == MicroPanel.LIPID_IC
							&& ComponentView.this.syncEcIc.isSelected()) {
						ComponentView.this.setPercentages(MicroPanel.LIPID_EC, newval, this.mol
								.getPDBcode(), this.domainID);
					}
					ComponentView.this.upDatePercentagesOnly();

				}
			}

			if (this.type == MicroPanel.PROTEIN && addedOnes < diff - 1 && addedOnes > -1)
				JOptionPane.showMessageDialog(null,
						"Unable to place the requested Number of Proteins\n\n"
								+ "Absolute Number of Proteins Added: "
								+ addedOnes, "Adding Interrupted",
						JOptionPane.WARNING_MESSAGE);
		}

		/**
		 * Set the value of the micropanel and convert according the decimal places
		 * in case of lipids
		 * @param double number to set
		 */
		public void setNumber(final double number) {
//System.out.println("setNumber "+number);
			this.number = new Double(number);

			this.spinner.removeChangeListener(this);
			if (this.type == MicroPanel.PROTEIN || ComponentView.this.getComputeAbsoluteValues()) {
				this.spinner.setValue(number);
				this.numberlabel.setText(this.convertPercentage(number)+" ");
			} else {
				if (Integer.parseInt(ComponentView.this.decimalplaces
						.getValue().toString()) == 0) {
					this.spinner.setValue(number);
					this.numberlabel.setText(this.convertPercentage(number) + " %");
				} else if (Integer.parseInt(ComponentView.this.decimalplaces
						.getValue().toString()) == 1) {
					this.spinner.setValue(number / 10d);
					this.numberlabel.setText("" + number / 10 + " %");
				} else if (Integer.parseInt(ComponentView.this.decimalplaces
						.getValue().toString()) == 2) {
					this.spinner.setValue(number / 100d);
					this.numberlabel.setText("" + number / 100 + " %");
				} else if (Integer.parseInt(ComponentView.this.decimalplaces
						.getValue().toString()) == 3) {
					this.spinner.setValue(number / 1000d);
					this.numberlabel.setText("" + number / 1000 + " %");
				}
			}
			this.spinner.addChangeListener(this);
		}
		
		/**
		 * Set the absolute or percental value of lipids in the spinner model.
		 * @param computeAbsoluteValues
		 * @param value
		 */
		public void setComputeAbsoluteValues(boolean computeAbsoluteValues, double value) {
//System.out.println("setComputeAbsoluteValues "+computeAbsoluteValues+" "+value);
			if (computeAbsoluteValues) {
				((SpinnerNumberModel)this.spinner.getModel()).setMaximum(Double.MAX_VALUE);
				this.setNumber(value);
				if (this.mol instanceof Lipid_3D == false)
					return;
				((Lipid_3D)this.mol).setPercent(new Integer(Double.toString(value).substring(0, Double.toString(value).indexOf("."))));
			} else {
//				((SpinnerNumberModel)this.spinner.getModel()).setMaximum(100d);
//				this.setNumber(value);
//				if (this.mol instanceof Lipid_3D == false)
//					return;
//				((Lipid_3D)this.mol).setPercent(new Integer(Double.toString(value).substring(0, Double.toString(value).indexOf("."))));
			}
		}

		public void mouseClicked(final MouseEvent arg0) {

		}

		public void mouseEntered(final MouseEvent arg0) {
			if (!this.isSelected)
				this.setBorder(BorderFactory.createLineBorder(this.mol
						.getColor(), 3));

		}

		public void mouseExited(final MouseEvent arg0) {
			if (!this.isSelected)
				this.setBorder(BorderFactory.createLineBorder(
						ComponentView.this.tab.getBackground(), 3));

		}

		public void mousePressed(final MouseEvent arg0) {
			if (!this.isSelected)
				ComponentView.this.listener.selectFromCV(this.mol, this.type);
			if (arg0.getButton() == MouseEvent.BUTTON3) {
				this.showPopup(arg0.getComponent(), arg0.getPoint());
			}
			if (this.mol instanceof Protein_3D)
				return;
			if ((arg0.getSource() == this.textlabel || arg0.getSource() == this.north) == false)
				return;

			if (arg0.getButton() == MouseEvent.BUTTON1
					&& !(arg0.getModifiers() == CM2_CONSTANTS.GUI_SETTINGS_AND_CONTROLS.RIGHT_BUTTON_MODIFIER)) {

				ComponentView.this.startDragTimer(arg0, this.mol);
			}

		}

		public void mouseReleased(final MouseEvent arg0) {
			if (this.mol instanceof Protein_3D)
				return;
			if ((arg0.getSource() == this.textlabel || arg0.getSource() == this.north) == false)
				return;
			if (arg0.getButton() == MouseEvent.BUTTON1
					&& !(arg0.getModifiers() == CM2_CONSTANTS.GUI_SETTINGS_AND_CONTROLS.RIGHT_BUTTON_MODIFIER)) {
				ComponentView.this.quitDnd(false);
			}

		}

		private Point lastPoint = new Point();

		public void mouseDragged(final MouseEvent arg0) {
			if (this.mol instanceof Protein_3D)
				return;
			if (DataPool.DNDBridge.getCurrentMolecule() == null)
				return;
			if (arg0.getPoint().equals(this.lastPoint))
				return;
			this.lastPoint = arg0.getPoint();

			final Point loc = arg0.getPoint();

			SwingUtilities.convertPointToScreen(loc, (Component) arg0
					.getSource());

			ComponentView.this.listener.DnDEventOccured(loc);

		}

		public void mouseMoved(final MouseEvent e) {

		}
	}

	// /DND Section starts here///////////////

	DNDThread dndThread;

	/**
	 * Starts -or restarts the Timer for Drag N Drop. Called each time the left
	 * Mouse Button is pressed down.
	 * 
	 * @param arg0
	 * @param molecule_3D
	 */
	@SuppressWarnings("deprecation")
	private void startDragTimer(final MouseEvent arg0, final Molecule_3D mol) {

		if (this.dndThread != null) {
			if (this.dndThread.isAlive())
				this.dndThread.stop();
		}
		this.dndThread = new DNDThread((Lipid_3D) mol);
		this.dndThread.start();
	}

	/**
	 * Ends the Dnd process.
	 * 
	 * @param cancelled
	 *            - if true, the Dragged Protein will be discarded
	 */
	@SuppressWarnings("deprecation")
	private void quitDnd(final boolean cancelled) {
		this.setCursor(Cursor.getDefaultCursor());

		if (this.dndThread == null)
			return;
		this.dndThread.stop();

		this.listener.DNDFinished(cancelled);

		DataPool.DNDBridge.deactivate();

	}

	/**
	 * The Thread that initializes Dnd Operation, unless it doesnt get stopped
	 * before countdown exceeds
	 * 
	 * @author Tim Dingersen dackelnose@gmx.net
	 * 
	 */
	private class DNDThread extends Thread {

		private final Lipid_3D currentLipid;

		public DNDThread(final Lipid_3D lip) {
			super();
			this.currentLipid = lip;

		}

		public void run() {
			try {

				Thread
						.sleep(CM2_CONSTANTS.GUI_SETTINGS_AND_CONTROLS.DND_TIMER_COUNTDOWN);

				ComponentView.this.setCursor(Cm2_IconDepot.DNDCursor);

				DataPool.DNDBridge.activate(this.currentLipid);

			} catch (final InterruptedException e) {

			}
		}

		// public Protein_3D getNewProtein() {
		// return newProt;
		// }

	}

	private class DomainPanel extends JToolBar {

		JLabel label = new JLabel("", SwingConstants.LEADING);

		JToolBar btnpanel = new JToolBar();

		JButton change = new JButton();

		JButton remove = new JButton();

		int domainID = 0;

		public DomainPanel() {
			this.setFloatable(false);
			this.setOrientation(JToolBar.VERTICAL);
			// this.setLayout(new BoxLayout(this,BoxLayout.X_AXIS));

			this.change
					.setIcon(Cm2_IconDepot.getSpecifiedIcon("CV_changeArea"));
			this.remove.setIcon(Cm2_IconDepot.getSpecifiedIcon("CV_remove"));

			this.btnpanel.setFloatable(false);

			this.change.addActionListener(new ActionListener() {

				public void actionPerformed(final ActionEvent arg0) {
					ComponentView.this.listener.startDomainChange(DataPool
							.getMembraneModel().getMicroDomain(
									DomainPanel.this.domainID - 1));

				}

			});
			this.remove.addActionListener(new ActionListener() {

				public void actionPerformed(final ActionEvent arg0) {

					ComponentView.this.listener.removeMicroDomain(DataPool
							.getMembraneModel().getMicroDomain(
									DomainPanel.this.domainID - 1));

				}

			});

			this.add(this.label);
			this.label.setFont(this.label.getFont().deriveFont(
					this.label.getFont().getSize2D() + 2));

			this.btnpanel.add(this.change);
			this.btnpanel.add(this.remove);
			this.btnpanel.setBorder(BorderFactory.createEtchedBorder());

			this
					.add(Box.createVerticalStrut(this.btnpanel
							.getPreferredSize().height));
			this.add(Box.createHorizontalGlue());

			this.add(this.btnpanel);

			this.btnpanel.setVisible(false);

		}

		public void setDomain(final int domainID) {
			this.domainID = domainID;
			if (domainID > 0) {
				this.label.setText("Micro Domain " + domainID);
				this.btnpanel.setVisible(true);
			} else {
				this.label.setText("Default Area");
				this.btnpanel.setVisible(false);
			}
		}

		public void setEnabled(final boolean b) {
			this.change.setEnabled(b);

			this.remove.setEnabled(b);
		}

	}

	public void stateChanged(final ChangeEvent arg0) {

		if (arg0.getSource() == this.tab) {
			// DataPool.setMicroPanels(lists);
			// SwingUtilities.invokeLater(new Thread() {
			// public void run() {

			if (this.tab.getSelectedIndex() < 0) {
				this.listener.highlightMicroDomain(0);
				return;
			}
			String s = this.tab.getTitleAt(this.tab.getSelectedIndex());
			try {
				s = s.replaceAll("<html>", "").replaceAll("</html>", "")
						.replaceAll("<br>", "").trim();

				this.listener.highlightMicroDomain(Integer.parseInt(s));
			} catch (final Exception e) {
				this.listener.highlightMicroDomain(0);
			}
			// }
			// });

		}
		
		if (arg0.getSource() == this.decimalplaces) {

			DataPool.getMembraneModel().setDecimalPlaces(
					Integer.parseInt(this.decimalplaces.getValue().toString()));

//			System.out.println("Action deciamlplaces "+this.decimalplaces.getValue().toString()+" "+this.oldDecimalPlaces);
			
			// handle here all lipids in the micropanels
			for (int a = 0; a < this.tab.getTabCount(); a++) {
				final JPanel listpanel = this.lists.elementAt(a);

				for (int i = 0; i < listpanel.getComponentCount(); i++) {
					final Component c = listpanel.getComponent(i);

					if (c instanceof MicroPanel == false)
						continue;

					final MicroPanel mp = (MicroPanel) c;
					if (mp.mol instanceof Lipid_3D == false)
						continue;

					if (((Lipid_3D) mp.mol).getPercent() != 0) {
						if (this.oldDecimalPlaces < Integer
								.parseInt(this.decimalplaces.getValue()
										.toString())) {
							((Lipid_3D) mp.mol).setPercent(((Lipid_3D) mp.mol)
									.getPercent() * 10);
						} else if (this.oldDecimalPlaces > Integer
								.parseInt(this.decimalplaces.getValue()
										.toString())) {
							((Lipid_3D) mp.mol).setPercent(((Lipid_3D) mp.mol)
									.getPercent() / 10);
						}
					}

					mp.setNumber(((Lipid_3D) mp.mol).getPercent());

				}
			}
			
			// handle here all lipids not active in the micropanels (in the other layers)
			
			final MembraneModel model = DataPool.getMembraneModel();

			final int layerID = DataPool.getCurrentLayerID();
			
			Vector<Lipid_3D> lipidsExtra = model.getAllExampleLipidsExtra();
			
			for (Lipid_3D lipid:lipidsExtra) {
				if (lipid.getLayerID() == layerID)
					continue;
				if (lipid.getPercent() != 0) {
					if (this.oldDecimalPlaces < Integer
							.parseInt(this.decimalplaces.getValue()
									.toString())) {
						lipid.setPercent(lipid.getPercent() * 10);
					} else if (this.oldDecimalPlaces > Integer
							.parseInt(this.decimalplaces.getValue()
									.toString())) {
						lipid.setPercent(lipid
								.getPercent() / 10);
					}
				}
			}
			
			Vector<Lipid_3D> lipidsIntra = model.getAllExampleLipidsExtra();
			
			for (Lipid_3D lipid:lipidsIntra) {
				if (lipid.getLayerID() == layerID)
					continue;
				if (lipid.getPercent() != 0) {
					if (this.oldDecimalPlaces < Integer
							.parseInt(this.decimalplaces.getValue()
									.toString())) {
						lipid.setPercent(lipid.getPercent() * 10);
					} else if (this.oldDecimalPlaces > Integer
							.parseInt(this.decimalplaces.getValue()
									.toString())) {
						lipid.setPercent(lipid
								.getPercent() / 10);
					}
				}
			}
			
			// store the actual decimal places
			this.oldDecimalPlaces = Integer.parseInt(this.decimalplaces
					.getValue().toString());

		}
	}

	private void upDatePercentagesOnly() {
		final JPanel listpanel = this.lists.elementAt(this.tab
				.getSelectedIndex());

		for (int i = 0; i < listpanel.getComponentCount(); i++) {

			final Component c = listpanel.getComponent(i);

			if (c instanceof MicroPanel == false)
				continue;

			final MicroPanel mp = (MicroPanel) c;
			if (mp.mol instanceof Lipid_3D == false)
				continue;

			mp.setNumber(((Lipid_3D) mp.mol).getPercent());

		}
		this.oldDecimalPlaces = Integer.parseInt(this.decimalplaces.getValue()
				.toString());
	}

	public void updateColors() {
		try {
			final JPanel listpanel = this.lists.elementAt(this.tab
					.getSelectedIndex());

			for (int i = 0; i < listpanel.getComponentCount(); i++) {
				final Component c = listpanel.getComponent(i);

				if (c instanceof MicroPanel == false)
					continue;

				final MicroPanel mp = (MicroPanel) c;

				mp.mouseEntered(null);

			}
		} catch (final Exception e) {

		}
	}

	// /**
	// * Shows overall Lipidpercentage helping to decide if
	// * Lipids are set to 100 %
	// * @param percentage
	// */
	// public void showLipidPercentage (){
	// this.overallPercentage.setVisible(true);
	// this.overallPercentage.setText("");
	//		
	// if (!syncEcIc.isEnabled()){
	// return;
	// }
	//		
	// if (autosetpercent.isEnabled()){
	// this.overallPercentage.setOpaque(true);
	// }
	//		
	// float count = 0;
	//		
	// JPanel listpanel = lists.elementAt(tab.getSelectedIndex());
	//
	// for (int i = 0; i < listpanel.getComponentCount()/2; i++) {
	// Component c = listpanel.getComponent(i);
	//
	// if (c instanceof MicroPanel == false)
	// continue;
	//
	// MicroPanel mp = (MicroPanel) c;
	// if (mp.mol instanceof Lipid_3D == false)
	// continue;
	//			
	// count += mp.getSpinnerValue();
	//			
	// System.out.println("count loop: " + count);
	// }
	//		
	//		
	// String percentage = String.valueOf(count);
	// if ((Integer)DataPool.getMembraneModel().getDecimalPlaces() != 0){
	//
	// if (percentage.length() >= percentage.indexOf(".") + 1 +
	// (Integer)DataPool.getMembraneModel().getDecimalPlaces()){
	// percentage = percentage.substring(0, percentage.indexOf("."))
	// +
	// percentage.substring(percentage.indexOf("."),percentage.indexOf(".")+1+(
	// Integer)DataPool.getMembraneModel().getDecimalPlaces());
	// }
	//			
	// }
	//				
	// count = Float.valueOf(percentage);
	//		
	// if ((Integer)DataPool.getMembraneModel().getDecimalPlaces() == 1){
	// if ((count <= 100.01)&&(count>=99.99)){
	// count = 100;
	// }
	// }else if((Integer)DataPool.getMembraneModel().getDecimalPlaces() == 2){
	// if ((count <= 100.001)&&(count>=99.999)){
	// count = 100;
	// }
	// }else if((Integer)DataPool.getMembraneModel().getDecimalPlaces() == 3){
	// if ((count <= 100.0001)&&(count>=99.9999)){
	// count = 100;
	// }
	// }
	//		
	// percentage = String.valueOf(count);
	//		
	// System.out.println("Percentage : "+ percentage);
	//		
	// this.overallPercentage.setText("Overall Lipid Percentage : " + percentage
	// + " %");
	// this.overallPercentage.setVisible(true);
	// }
	//	
	// /**
	// * Hides overall Lipidpercentage, invoke if autosetpercent is on
	// */
	// public void hideLipidPercentage (){
	// this.overallPercentage.setVisible(false);
	// this.overallPercentage.setText("");
	// }

}