/*******************************************************************************
 * CELLmicrocosmos 2.2 MembraneEditor (version 2.2.2_2)
 *
 * Copyright (c) 2005, 2017 The CELLmicrocosmos 2 Development Team
 * (Bjorn Sommer, Tim Dingersen et al.
 *  University of Konstanz, Monash University, Bielefeld University)
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *  
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *  
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *******************************************************************************/
package org.cellmicrocosmos.cm2.interfaces;

import java.awt.Point;
import java.io.File;
import java.util.Vector;

import javax.media.j3d.BranchGroup;
import javax.swing.JFrame;
import javax.swing.JPopupMenu;

import org.cellmicrocosmos.cm2.dataobjects.Lipid_3D;
import org.cellmicrocosmos.cm2.dataobjects.MicroDomain_3D;
import org.cellmicrocosmos.cm2.dataobjects.Molecule_3D;
import org.cellmicrocosmos.cm2.dataobjects.Protein_3D;
import org.cellmicrocosmos.cm2.membranepackingalgorithm.MembranePackingAlgorithm;
import org.cellmicrocosmos.cm2.pluginmanager.PluginManager;


/**
 * 
 * Listener used by the MembraneView
 * 
 * @author Tim Dingersen dackelnose@gmx.net
 * 
 */
public interface MembraneViewListener {

	PluginManager getPluginManager();

	void alignSingleProtein(Molecule_3D molecule_3D);
	
	void alignSingleProteinPlus(Molecule_3D molecule_3D);

	void pickMolecules(Molecule_3D[] molecule_3Ds);

	void removeProteinsFrom3D(Protein_3D[] prots2remove);

	void removeLipidsFrom3D(Lipid_3D[] toremove);

	void clearAllLipidsFrom3DandModel(int domainID, int layerid);

	void refreshViews(boolean compView, boolean memViewPanels,
			boolean database, boolean stats, boolean props,
			boolean intersectionPanel, boolean showDialog);

	void setCalculationEnabled();

	void displayedLayerChanged(int mode);

	void selectProteinsByCode(String code, int domainID);

	boolean addANewProteinAtRandomPosition( String code,
			String file, int modelnumber, int DomainID);

	
	void requestFocusOnView(int view);

	JPopupMenu createCanvasPopUpMenu();

	void popup3DSettingsDialog(Point location);

	boolean isProperties_Membrane_Atrribus_Showing();

	void updateProperties();

	void userChangedModelForFile(File file, int model);

	void setShowAxis(boolean b);

	boolean areAxisVisible();

	public void addLipidExtra(Lipid_3D lip2add);

	public void addLipidIntra(Lipid_3D lip2add);

	public void removeLipidExtra(Lipid_3D lip2add);

	public void removeLipidIntra(Lipid_3D lip2add);

	public void calculate(MembranePackingAlgorithm memb);

	void displayAlgoritmInfo(String info);

	void displayAlgorithmProgress(int progress);

	void alignExampleLipid(Lipid_3D lip, int side);

	void setAllEnabled(boolean b);

	void updateMicroDomainsIn3D();

	void highlightMicroDomain(int domainID);

	int getHighlightedMicroDomain();

	void startDomainChange(MicroDomain_3D domainToCHange);

	void removeMicroDomain(MicroDomain_3D domainToDelete);

	void selectFromCV(Molecule_3D mol, int i);

	public Vector<Molecule_3D> getCurrentSelection();

	public void showStereoView();

	void reloadGeometry();

	void startModelChange(Molecule_3D mol);

	void DnDEventOccured(Point loc);

	void DNDFinished(boolean cancelled);

	JFrame getMainFrame();

	void openMoleculeEditorFromCV(Molecule_3D mol);

	void requestRenderModeChange();

	void changeRenderModeFromAlgorithm();
	
	void dockScreen(boolean dockScreen);
	
//	void autoFocus(double ratio);
	
	void updateDistanceScale(double distanceFromCenter);
	
	BranchGroup getJ3DBranchGroup();
	
	int getStereoViewMode();
	
	void setStereoViewMode(int stereoViewMode);
	
	int getStereoViewScalingMode();
	
	void setStereoViewScalingMode(int stereoViewScalingMode);
	
	int getStereoViewAutoFocusMode();
	
	void setStereoViewAutoFocusMode(int stereoViewAutoFocusMode);
	
	void validateStereoMode(boolean isSelected);
	
	void makeToolbarsFloatable(boolean show);

}
