/*******************************************************************************
 * CELLmicrocosmos 2.2 MembraneEditor (version 2.2.2_2)
 *
 * Copyright (c) 2005, 2017 The CELLmicrocosmos 2 Development Team
 * (Bjorn Sommer, Tim Dingersen et al.
 *  University of Konstanz, Monash University, Bielefeld University)
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *  
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *  
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *******************************************************************************/
package org.cellmicrocosmos.cm2.membranepackingalgorithm;

import java.util.Random;


/**
 * 
 * Random Class generates new Unique Random numbers. This makes sure that
 * you always get the same random numbers if same starting value is used.
 * 
 * @author Maloche
 * @author Bjoern Sommer bjoern@CELLmicrocosmos.org
 *
 */
public class UniqueRandom extends SimpleRandom {

	private static final long serialVersionUID = -4659835891904415460L;

	long seed;
	Random rand;

	/**
	 * Creates a new Unique Random number by using the seed.
	 * 
	 * @param seed
	 */
	public UniqueRandom(long seed) {
		this.seed = seed;
		rand = new Random(seed);
	}

	/**
	 * Returns the next int value in the given range. The absval boolean
	 * indicates if the int is in the range from -scale to +scale or the
	 * absolut value.
	 * 
	 * 
	 * @param scale
	 * @param absval
	 * @return int
	 */
	public int nextInt(int scale, boolean absval) {
		rand.setSeed(seed);
		int i = rand.nextInt(scale);
		seed = rand.nextLong();
		if (absval == false) {

			if (seed % 2 == 0)
				i = -i;
		}

		return i;
	}
	
	/**
	 * Returns the next absolute int value in the given range.
	 * @param scale
	 * @return int
	 */
	public int nextInt(int scale) {
		return this.nextInt(scale, true);
	}

	/**
	 * Returns the next float The absval boolean indicates if the float is
	 * in the range from - to + or the absolut value.
	 * 
	 * 
	 * @return float
	 */
	public float nextFloat(boolean absval) {
		rand.setSeed(seed);
		float f = rand.nextFloat();

		seed = rand.nextLong();

		if (absval == false) {
			if (seed % 2 == 0)
				f = -f;
		}

		return f;
	}
	
	/**
	 * Returns the next absolute float value in the given range.
	 * @return float
	 */
	public float nextFloat() {
		return this.nextFloat(true);
	}

	/**
	 * Returns the next double The absval boolean indicates if the double is
	 * in the range from - to + or the absolut value.
	 * 
	 * @return double
	 */
	public double nextDouble(boolean absval) {
		rand.setSeed(seed);
		double d = rand.nextDouble();

		seed = rand.nextLong();

		if (absval == false) {
			if (seed % 2 == 0)
				d = -d;
		}

		return d;
	}
	
	/**
	 * Returns the next absolute double value in the given range.
	 * @return double
	 */
	public double nextDouble() {
		return this.nextDouble(true);
	}

	/**
	 * Get the actual used seed.
	 * @return seed
	 */
	public long getSeed() {
		return seed;
	}
	
	/**
	 * Returns the next random boolean value.
	 * @return boolean
	 */
	public boolean nextBoolean() {
		rand.setSeed(seed);
		return rand.nextBoolean();
	}

	/**
	 * Set the actual used seed.
	 */
	public void setSeed(long seed) {
		this.seed = seed;
	}

}
