/*******************************************************************************
 * CELLmicrocosmos 2.2 MembraneEditor (version 2.2.2_2)
 *
 * Copyright (c) 2005, 2017 The CELLmicrocosmos 2 Development Team
 * (Bjorn Sommer, Tim Dingersen et al.
 *  University of Konstanz, Monash University, Bielefeld University)
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *  
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *  
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *******************************************************************************/
package org.cellmicrocosmos.cm2.membranepackingalgorithms.atomlevelminimizer;

import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.lang.reflect.Method;
import java.net.URL;
import java.net.URLClassLoader;
import java.util.Hashtable;
import java.util.Vector;

import javax.swing.AbstractCellEditor;
import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.ButtonGroup;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComponent;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.JSlider;
import javax.swing.JTabbedPane;
import javax.swing.JTable;
import javax.swing.JTextField;
import javax.swing.JToolBar;
import javax.swing.SwingConstants;
import javax.swing.border.TitledBorder;
import javax.swing.table.TableCellEditor;
import javax.swing.table.TableCellRenderer;
import javax.swing.text.AttributeSet;
import javax.swing.text.BadLocationException;
import javax.swing.text.PlainDocument;
import javax.vecmath.Point2d;

import org.cellmicrocosmos.cm2.membranepackingalgorithm.ConfigIO;
import org.cellmicrocosmos.cm2.membranepackingalgorithm.EnergyFunction;
import org.cellmicrocosmos.cm2.membranepackingalgorithm.GraphPanel;
import org.cellmicrocosmos.cm2.membranepackingalgorithm.ParameterMatrixPanel;
import org.cellmicrocosmos.cm2.misc.Cm2_Toolkit;



/**
 * This class displays and handles all components of the graphical user interface of the algorithm
 * 
 * @author Tim Dingersen
 * dackelnose@gmx.net
 *
 */
@SuppressWarnings("serial")
public class Gui extends JToolBar implements ActionListener {

	JCheckBox controlsBtn = new JCheckBox(" Settings ");
	JCheckBox parametersBtn = new JCheckBox(" Energy Function ");
	JCheckBox graphBtn = new JCheckBox(" Statistics ");

	JButton startBtn = new JButton("Start");
	

	MyGraphPanel graphPanel = new MyGraphPanel();
	ControlPanel controlPanel;
	ParameterPanel paraPanel = new ParameterPanel();

	private Atom_Level_Minimizer algo;
	public int guiMode = -1;

	JFrame dialog = new JFrame();

	@SuppressWarnings("deprecation")
	public Gui(Atom_Level_Minimizer algo) {
		this.algo = algo;

		File[] files = algo.pluginsFile.listFiles();

		for (int i = 0; i < files.length; i++) {

			try {

				File jarfile;
				if (files[i].getName().toLowerCase().endsWith(".jar"))
					jarfile = files[i];
				else
					continue;

				final URL u = jarfile.toURL();

				final URLClassLoader loader = new URLClassLoader(
						new URL[] { u }, this.getClass().getClassLoader());

				final Method addURL = URLClassLoader.class.getDeclaredMethod(
						"addURL", new Class[] { URL.class });
				addURL.setAccessible(true);
				addURL.invoke(loader, new Object[] { u });

				final Class<?> c = Class.forName(Cm2_Toolkit
						.getSimpleName(jarfile), true, loader);

				final EnergyFunction enf = (EnergyFunction) c.newInstance();

				algo.energyFunctions.addElement(enf);
				enf.setMembranePackingAlgorithm(algo);

			} catch (Exception e) {

			}

		}

		controlPanel = new ControlPanel();
		this.setFloatable(false);

		// this.setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));

		// this.add(toolbar);

		addSeparator();
		addSeparator(new Dimension(5, 0));
		add(startBtn);
		addSeparator(new Dimension(5, 0));
		addSeparator();

		addSeparator(new Dimension(5, 0));
		add(controlsBtn);
		addSeparator(new Dimension(5, 0));
		addSeparator();
		addSeparator(new Dimension(5, 0));
		add(parametersBtn);
		addSeparator(new Dimension(5, 0));
		addSeparator();
		addSeparator(new Dimension(5, 0));
		add(graphBtn);

		add(new JPanel());

		controlsBtn.setIcon(new ImageIcon(this.algo
				.getAlgorithm_Directory_In_Workspace().getAbsolutePath()
				+ File.separator + "Icon1.GIF"));
		parametersBtn.setIcon(new ImageIcon(this.algo
				.getAlgorithm_Directory_In_Workspace().getAbsolutePath()
				+ File.separator + "Icon2.GIF"));
		graphBtn.setIcon(new ImageIcon(this.algo
				.getAlgorithm_Directory_In_Workspace().getAbsolutePath()
				+ File.separator + "Icon3.GIF"));
		startBtn.addActionListener(this);
		controlsBtn.addActionListener(this);
		parametersBtn.addActionListener(this);
		graphBtn.addActionListener(this);

		MouseAdapter ma = new MouseAdapter() {
			public void mouseEntered(MouseEvent arg0) {
				((JComponent) arg0.getSource()).setBackground(Gui.this
						.getBackground().darker());
			}

			public void mouseExited(MouseEvent arg0) {
				((JComponent) arg0.getSource()).setBackground(null);
			}
		};

		graphBtn.addMouseListener(ma);
		parametersBtn.addMouseListener(ma);
		controlsBtn.addMouseListener(ma);

		startBtn.setFont(startBtn.getFont().deriveFont(Font.BOLD, 16f));
		startBtn.setForeground(Color.green.darker());

	}

	public void actionPerformed(ActionEvent arg0) {

		if (arg0.getSource() == this.startBtn) {
			this.algo.pauseAlgorithm = !this.algo.pauseAlgorithm;
			startBtn.setForeground(null);
			this.startBtn.setText("Wait...");
			this.startBtn.setEnabled(false);

			if (this.controlsBtn.isSelected())
				this.controlsBtn.doClick();
			if (this.parametersBtn.isSelected())
				this.parametersBtn.doClick();

			if (this.guiMode < 2)
				dialog.dispose();
			
			if (!this.algo.pauseAlgorithm && this.algo.isStartInBackgroundSelected()) {
				this.algo.sendToSystemTray();
			}

		}

		if (arg0.getSource() == this.graphBtn) {

			this.switchToGuiMode(this.graphBtn.isSelected() ? 2 : -1);
		}
		if (arg0.getSource() == this.parametersBtn) {

			this.switchToGuiMode(this.parametersBtn.isSelected() ? 1 : -1);
		}
		if (arg0.getSource() == this.controlsBtn) {

			this.switchToGuiMode(this.controlsBtn.isSelected() ? 0 : -1);
		}

	}

	private void switchToGuiMode(final int mode) {
		this.guiMode = mode;
		this.controlsBtn.removeActionListener(this);
		this.graphBtn.removeActionListener(this);
		this.parametersBtn.removeActionListener(this);

		this.controlsBtn.setSelected(false);
		this.graphBtn.setSelected(false);
		this.parametersBtn.setSelected(false);

		if (mode == 0)
			controlsBtn.setSelected(true);
		else if (mode == 1)
			parametersBtn.setSelected(true);
		else if (mode == 2)
			graphBtn.setSelected(true);

		this.controlsBtn.addActionListener(this);
		this.graphBtn.addActionListener(this);
		this.parametersBtn.addActionListener(this);

		// if(this.getComponentCount()>1) {
		// this.remove(0);
		// }

		// if (dialog.getComponentCount() >= 0) {
		dialog.removeAll();
		//
		// }
		if (dialog.getWindowListeners().length > 0)
			dialog.removeWindowListener(dialog.getWindowListeners()[0]);

		dialog.dispose();

		if (mode == -1)
			return;

		dialog = new JFrame();

		dialog.addWindowListener(new WindowAdapter() {

			public void windowClosing(WindowEvent e) {

				// Gui.this.controlsBtn.removeActionListener(Gui.this);
				// Gui.this.graphBtn.removeActionListener(Gui.this);
				// Gui.this.parametersBtn.removeActionListener(Gui.this);

				Gui.this.switchToGuiMode(-1);

				// Gui.this.controlsBtn.setSelected(false);
				// Gui.this.graphBtn.setSelected(false);
				// Gui.this.parametersBtn.setSelected(false);

				// Gui.this.controlsBtn.addActionListener(Gui.this);
				// Gui.this.graphBtn.addActionListener(Gui.this);
				// Gui.this.parametersBtn.addActionListener(Gui.this);

			}
		});

		this.controlPanel.updateFields();
		
		if (mode == 2) {
			dialog.setIconImage(new ImageIcon(this.algo
					.getAlgorithm_Directory_In_Workspace().getAbsolutePath()
					+ File.separator + "Icon3.GIF").getImage());
			this.graphPanel.setPreferredSize(new Dimension((int) (this
					.getSize().width), 270));

			dialog.getContentPane().add(this.graphPanel);
			dialog.setTitle("Statistics");

		} else if (mode == 1) {

			dialog.setIconImage(new ImageIcon(this.algo
					.getAlgorithm_Directory_In_Workspace().getAbsolutePath()
					+ File.separator + "Icon2.GIF").getImage());

			paraPanel.setPreferredSize(new Dimension(paraPanel.tab
					.getPreferredSize().width + 5, paraPanel.tab
					.getPreferredSize().height + 5 + 30));

			if (this.controlPanel.rbLJ.isSelected()) {
				paraPanel.setPreferredSize(new Dimension(paraPanel.tab
						.getPreferredSize().width + 5, paraPanel.tab
						.getPreferredSize().height + 5 + 30));
				dialog.getContentPane().add(this.paraPanel);

			} else if (algo.energies.currentFunction != null
					&& algo.energies.currentFunction.getGui() != null) {
				dialog.getContentPane().add(
						algo.energies.currentFunction.getGui());

			} else {
				dialog.dispose();
			}

			dialog.setTitle("Energy Function");

		} else if (mode == 0) {

			dialog.setIconImage(new ImageIcon(this.algo
					.getAlgorithm_Directory_In_Workspace().getAbsolutePath()
					+ File.separator + "Icon1.GIF").getImage());

			//		
			dialog.getContentPane().add(this.controlPanel);
			dialog.setTitle("Options");

		}

		dialog.pack();

		// if (mode == 0) {
		// dialog.setSize((int)(dialog.getSize().width*1.4),(int)(dialog.getSize().height*1.4));
		// }
		dialog.setLocation(this.getLocationOnScreen().x, this
				.getLocationOnScreen().y
				- dialog.getSize().height);
		dialog.setVisible(true);

		if (mode == 2) {
			this.graphPanel.setValues(algo.myData.contents.getEnergyHistory(),
					algo.myData.contents.step);
		}

	}

	class MyGraphPanel extends GraphPanel {

		private static final long serialVersionUID = 1L;

		JLabel l1 = new JLabel();
		JLabel l2 = new JLabel();
		JLabel l3 = new JLabel();
		JLabel l4 = new JLabel();
		JLabel l5 = new JLabel();
		JLabel l6 = new JLabel();

		private Vector<Float> lastSteps = new Vector<Float>();

		public MyGraphPanel() {
			super("Statistics", "Steps", "Current State", new String[] {
					"Free Energy", "" }, new Color[] { Color.orange,
					Color.green });
			// "V(\u03b5)"

			// plot.getDomainAxis().setTickLabelsVisible(false);

			JPanel p1 = new JPanel();
			JPanel p2 = new JPanel();
			JPanel p3 = new JPanel();

			this.chartPanel.setDomainZoomable(false);

			this.add(p1);

			p1.setLayout(new GridLayout(0, 2));
			p2.setLayout(new BoxLayout(p2, BoxLayout.Y_AXIS));
			p3.setLayout(new BoxLayout(p3, BoxLayout.Y_AXIS));
			p1.add(p2);
			p1.add(p3);

			JLabel l11 = new JLabel("Energy Mode");
			JLabel l22 = new JLabel("Initial Value");
			JLabel l33 = new JLabel("Current Value");
			JLabel l44 = new JLabel("Time Elapsed");
			JLabel l55 = new JLabel("Last Cycle");
			JLabel l66 = new JLabel("Estimated Completion");

			JPanel p11 = new JPanel();
			JPanel p22 = new JPanel();
			JPanel p33 = new JPanel();
			JPanel p44 = new JPanel();
			JPanel p55 = new JPanel();
			JPanel p66 = new JPanel();

			p11.setLayout(new BoxLayout(p11, BoxLayout.X_AXIS));
			p22.setLayout(new BoxLayout(p22, BoxLayout.X_AXIS));
			p33.setLayout(new BoxLayout(p33, BoxLayout.X_AXIS));
			p44.setLayout(new BoxLayout(p44, BoxLayout.X_AXIS));
			p55.setLayout(new BoxLayout(p55, BoxLayout.X_AXIS));
			p66.setLayout(new BoxLayout(p66, BoxLayout.X_AXIS));

			p11.add(l11);
			p11.add(Box.createHorizontalGlue());
			p11.add(l1);

			p22.add(l22);
			p22.add(Box.createHorizontalGlue());
			p22.add(l2);

			p33.add(l33);
			p33.add(Box.createHorizontalGlue());
			p33.add(l3);

			p44.add(l44);
			p44.add(Box.createHorizontalGlue());
			p44.add(l4);

			p55.add(l55);
			p55.add(Box.createHorizontalGlue());
			p55.add(l5);

			p66.add(l66);
			p66.add(Box.createHorizontalGlue());
			p66.add(l6);

			p2.add(Box.createVerticalGlue());
			p2.add(p11);
			p2.add(Box.createVerticalGlue());
			p2.add(p22);
			p2.add(Box.createVerticalGlue());
			p2.add(p33);
			p2.add(Box.createVerticalGlue());
			p3.add(Box.createVerticalGlue());
			p3.add(p44);
			p3.add(Box.createVerticalGlue());
			p3.add(p55);
			p3.add(Box.createVerticalGlue());
			p3.add(p66);
			p3.add(Box.createVerticalGlue());

			l11.setFont(this.getFont().deriveFont(Font.BOLD));
			l22.setFont(this.getFont().deriveFont(Font.BOLD));
			l33.setFont(this.getFont().deriveFont(Font.BOLD));
			l44.setFont(this.getFont().deriveFont(Font.BOLD));
			l55.setFont(this.getFont().deriveFont(Font.BOLD));
			l66.setFont(this.getFont().deriveFont(Font.BOLD));

			p11.setBorder(BorderFactory.createLoweredBevelBorder());
			p22.setBorder(BorderFactory.createLoweredBevelBorder());
			p33.setBorder(BorderFactory.createLoweredBevelBorder());
			p44.setBorder(BorderFactory.createLoweredBevelBorder());
			p55.setBorder(BorderFactory.createLoweredBevelBorder());
			p66.setBorder(BorderFactory.createLoweredBevelBorder());

		}

		public void setValues(Double[] energyHistory, int step) {

			Point2d[] values = new Point2d[energyHistory.length + 1];

			for (int i = 0; i < energyHistory.length; i++) {

				if (energyHistory[i] != null) {
					values[i] = new Point2d(step < 75 ? i : (step - 75) + i,
							energyHistory[i].doubleValue());
				} else {
					values[i] = new Point2d(step < 75 ? i : (step - 75) + i,
							Double.NaN);
				}
			}
			values[energyHistory.length] = new Point2d(
					step < 75 ? energyHistory.length : (step - 75)
							+ energyHistory.length, Double.NaN);

			dialog.setTitle("Statistics  [Current Temp: "
					+ algo.myData.settings.currentTemparature + "]");
			this.setData(new Point2d[][] { values });

			if (algo.timeInit == 0)
				return;

			if (algo.energies.currentFunction == null) {
				this.l1.setText("Lennard Jones 12-6 Potential V(\u03b5)");

			}

			else {

				if (algo.energies.currentFunction != null
						&& algo.energies.currentFunction.getButtonName() != null)
					this.l1.setText(algo.energies.currentFunction
							.getButtonName());
				else
					this.l1.setText("Custom");
			}

			this.l2.setText(((long) algo.initialValue) + "");

			if (algo.myData.contents.currentFreeEnergy != null) {
				this.l3.setText((algo.myData.contents.currentFreeEnergy
						.longValue())
						+ "");
			} else {
				this.l3.setText("No Data");
			}

			long current = System.currentTimeMillis();
			long totaltime = current - algo.timeInit;
			long lastStep = current - algo.timeTaken;

			int seconds1 = (int) totaltime / 1000;

			if (seconds1 < 60) {
				this.l4.setText(seconds1 + " s");
			} else {

				int h = (int) ((float) seconds1 / 3600f);
				seconds1 -= h * 3600;

				int m = (int) ((float) seconds1 / 60f);
				seconds1 -= m * 60;

				if (h > 0)
					this.l4.setText(h + " h, " + m + " m, " + seconds1 + " s");

				else if (m > 0)
					this.l4.setText(m + " m, " + seconds1 + " s");
			}

			float seconds2 = (float) lastStep / 1000;

			this.lastSteps.addElement(seconds2);
			if (lastSteps.size() > 30) {
				lastSteps.removeElement(0);

			}

			float estimated = 0f;

			for (int i = 0; i < lastSteps.size(); i++) {
				estimated += lastSteps.get(i);
			}
			estimated /= (float) lastSteps.size();

			this.l5.setText(seconds2 + "  s");

			if (algo.myData.settings.currentTemparature > 0) {
				int steps2Do = (int) (algo.myData.settings.currentTemparature
						/ algo.myData.settings.coolingRate * algo.myData.settings.coolingIntervalLength);
				steps2Do -= algo.myData.contents.stepsSinceLastCooling;

				estimated *= steps2Do;

				int seconds3 = (int) estimated;

				if (seconds3 < 60) {
					this.l6.setText(seconds3 + " s");
				} else {

					int h = (int) ((float) seconds3 / 3600f);
					seconds3 -= h * 3600;

					int m = (int) ((float) seconds3 / 60f);
					seconds3 -= m * 60;

					if (h > 0)
						this.l6.setText(h + " h, " + m + " m, " + seconds3
								+ " s");

					else if (m > 0)
						this.l6.setText(m + " m, " + seconds3 + " s");
				}

			} else {
				this.l6.setText(" - ");
			}

		}

	}

	class ParameterPanel extends JPanel {

		MyParameterMatrixPanel panels[] = new MyParameterMatrixPanel[2];

		String[] allTypes;

		JTabbedPane tab = new JTabbedPane();

		public ParameterPanel() {
			super();

		}

		public void buildParameterMatrices(String[] types) {

			this.allTypes = types;

			for (int i = 0; i < panels.length; i++) {

				panels[i] = new MyParameterMatrixPanel(types,
						new MyTableCellRenderer(), new MyTableCellEditor());

			}

			panels[0].masterTF.defaultValue = new Double(
					MyDefaults.MASTER_EPSILON);
			panels[1].masterTF.defaultValue = new Double(
					MyDefaults.MASTER_SIGMA);
			panels[0].masterTF.setText("" + MyDefaults.MASTER_EPSILON);
			panels[1].masterTF.setText("" + MyDefaults.MASTER_SIGMA);

			this.setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));

			this.add(tab);

			tab
					.addTab(
							"<html><b><big><big>\u03b5</big></big></b><br>Pair Energies kJ/Mol</html>",
							panels[0]);
			// tab.addTab("<html><b><big><big>\u03c3</big></big></b><br> aka <i>r0</i>, distances of V(\u03b5)=0</html>",panels[1]);
			tab
					.addTab(
							"<html><b><big><big>\u03c3</big></big></b><br> aka <i>r0</i>, \u00c5 values</html>",
							panels[1]);

		}

		class MyParameterMatrixPanel extends ParameterMatrixPanel implements
				ActionListener {

			MyNumberTextField masterTF = new MyNumberTextField(0, 1000000,
					MyNumberTextField.DOUBLE, new Double(
							MyDefaults.MASTER_SIGMA), false);
			JButton defaultBtn = new JButton("Known Defaults");
			JButton testBtn = new JButton("Calculate Current Energy");

			public MyParameterMatrixPanel(String[] headers,
					TableCellRenderer cellRenderer, TableCellEditor cellEditor) {
				super(headers, cellRenderer, cellEditor);

				JPanel p1 = new JPanel();

				JLabel l1 = new JLabel("Master Value: ");

				p1.setLayout(new BoxLayout(p1, BoxLayout.X_AXIS));

				p1.add(Box.createHorizontalStrut(5));
				p1.add(defaultBtn);
				p1.add(Box.createHorizontalStrut(5));
				p1.add(testBtn);
				p1.add(Box.createHorizontalGlue());
				p1.add(Box.createVerticalStrut(30));
				p1.add(l1);
				p1.add(masterTF);
				p1.add(Box.createHorizontalStrut(5));

				masterTF.setMinimumSize(new Dimension(100, 20));
				masterTF.setMaximumSize(new Dimension(100, 20));
				masterTF.setPreferredSize(new Dimension(100, 20));

				masterTF
						.setToolTipText("<html>Substitutional Value for all blank Fields</html>");
				defaultBtn.addActionListener(this);
				testBtn.addActionListener(this);
				this.add(p1);

			}

			public void actionPerformed(ActionEvent arg0) {

				if (arg0.getSource() == defaultBtn) {
					ParameterPanel.this
							.fillWithDefaults(ParameterPanel.this.tab
									.getSelectedIndex());
				} else if (arg0.getSource() == testBtn) {

					algo.gui.controlPanel.updateFields();
					algo.gui.paraPanel.updateEnergies();

					algo.myData.contents.setRelevantMolecules(algo
							.getAllMoleculesInCurrentModel(), algo
							.getCurrentLayerID(), algo.area, algo);

					double value = 0;
					for (int i = 0; i < algo.myData.contents.atomsToTreat.length; i++) {

						double en = algo.energies
								.totalEnergy(algo.myData.contents.atomsToTreat[i]);

						en += algo.energies
								.totalInternalEnergy(algo.myData.contents.atomsToTreat[i]);

						value += en;
					}

					JOptionPane.showMessageDialog(testBtn,
							"The current Free Energy Value is:\n\n "
									+ (int) value + "\n", "Results",
							JOptionPane.INFORMATION_MESSAGE);

				}
			}

		}

		class MyTableCellRenderer extends JLabel implements TableCellRenderer {

			public Component getTableCellRendererComponent(JTable table,
					Object value, boolean isSelected, boolean hasFocus,
					int row, int column) {

				if (value != null)
					setText(value.toString());
				else
					setText("");

				setFont(table.getFont().deriveFont(Font.BOLD));

				// setBorder(UIManager.getBorder("TableHeader.cellBorder"));

				setHorizontalAlignment(SwingConstants.CENTER);

				this.setToolTipText("<html><b>"
						+ ParameterPanel.this.allTypes[column] + " - "
						+ ParameterPanel.this.allTypes[row] + "</b></html>");

				setOpaque(true);

				if ((row - 1) >= column) {
					this.setBackground(table.getBackground().darker());

				}

				else {

					this.setBackground(table.getBackground().brighter());
				}

				return this;
			}
		}

		class MyTableCellEditor extends AbstractCellEditor implements
				TableCellEditor {

			MyNumberTextField tf;

			public Component getTableCellEditorComponent(JTable arg0,
					Object arg1, boolean arg2, int row, int col) {

				tf = new MyNumberTextField(0d, 1000000d,
						MyNumberTextField.DOUBLE, null, true);

				if (arg1 != null)
					tf.setText(arg1.toString());
				else
					tf.setText("");

				setFont(arg0.getFont().deriveFont(Font.ITALIC,
						arg0.getFont().getSize() + 1));

				setBorder(BorderFactory.createLoweredBevelBorder());

				tf.setHorizontalAlignment(SwingConstants.CENTER);

				tf.setOpaque(true);

				tf.setBackground(arg0.getBackground().brighter());

				tf.setToolTipText("<html><b>"
						+ ParameterPanel.this.allTypes[col] + " - "
						+ ParameterPanel.this.allTypes[row] + "</b></html>");

				return tf;
			}

			public Object getCellEditorValue() {

				return tf.getText();

			}

		}

		// public void setValues(String[][]values,int table) {
		// for(int i=0;i<values.length;i++) {
		// for(int j=0;j<values[i].length;j++) {
		// tables[table].setValueAt(values[i][j],i,j);
		// }
		// }
		// }

		public void fillWithDefaults(int index) {

			Hashtable<String, Hashtable<String, Float>> hash;

			if (index == 0)
				hash = MyDefaults.createDefaultEpsilons();
			else
				hash = MyDefaults.createDefaultSigmas(Gui.this.algo);

			JTable table = this.panels[index].getTable();

			for (int i = 0; i < this.panels[index].headers.length; i++) {

				for (int j = 0; j < this.panels[index].headers.length; j++) {

					try {
						float f = hash.get(this.panels[index].headers[i]).get(
								this.panels[index].headers[j]);
						table.setValueAt(f, i, j);
					} catch (Exception e) {

					}

				}

			}

		}

		public void fillWithPreviousValues() {
			File base = Gui.this.algo.getAlgorithm_Directory_In_Workspace();

			base = new File(base.getAbsolutePath() + File.separator
					+ "LJ_Parameter");

			for (int t = 0; t < this.panels.length; t++) {

				for (int i = 0; i < this.panels[t].headers.length; i++) {

					File input = new File(base.getAbsolutePath()
							+ File.separator + (t == 0 ? "Epsilon_" : "Sigma_")
							+ this.panels[t].headers[i] + ".txt");

					if (input.canRead() == false)
						continue;

					BufferedReader br = null;
					try {
						br = new BufferedReader(new FileReader(input));

						Hashtable<String, Float> hash = new Hashtable<String, Float>();

						while (true) {
							String s = br.readLine();
							if (s == null)
								break;

							try {
								int split = s.lastIndexOf("=");
								String val = s.substring(split + 1).trim();

								String key = s.substring(0, split).trim();

								if (!val.equals("null"))
									hash.put(key, Float.parseFloat(val));
								else
									hash.put(key, new Float(-1));
							} catch (Exception e) {

							}

						}

						br.close();

						for (int j = 0; j < this.panels[t].headers.length; j++) {

							if (hash.get(this.panels[t].headers[j]) != null) {

								if (hash.get(this.panels[t].headers[j])
										.floatValue() == -1f)
									this.panels[t].table.setValueAt(null, i, j);
								else
									this.panels[t].table.setValueAt(hash
											.get(this.panels[t].headers[j]), i,
											j);
							}

						}

					} catch (Exception e) {
						if (br != null)
							try {
								br.close();
							} catch (IOException e1) {

							}
					}
				}

				BufferedReader br = null;
				try {
					br = new BufferedReader(new FileReader(new File(base
							.getAbsolutePath()
							+ File.separator
							+ "MASTER_"
							+ (t == 0 ? "Epsilon" : "Sigma") + ".txt")));
					String s = br.readLine();

					if (s != null)
						this.panels[t].masterTF.setText(s);
					br.close();
				} catch (IOException e) {
					if (br != null)
						try {
							br.close();
						} catch (IOException e1) {

						}
				}

			}

		}

		public void storeValues() {

			File base = Gui.this.algo.getAlgorithm_Directory_In_Workspace();

			base = new File(base.getAbsolutePath() + File.separator
					+ "LJ_Parameter");
			base.mkdirs();

			for (int t = 0; t < this.panels.length; t++) {

				JTable table = this.panels[t].getTable();

				for (int i = 0; i < this.panels[t].headers.length; i++) {

					File output = new File(base.getAbsolutePath()
							+ File.separator + (t == 0 ? "Epsilon_" : "Sigma_")
							+ this.panels[t].headers[i] + ".txt");
					BufferedWriter bw = null;
					try {
						bw = new BufferedWriter(new FileWriter(output));

						for (int j = 0; j < this.panels[t].headers.length; j++) {
							Object value = table.getValueAt(i, j);

							bw.write(this.panels[t].headers[j] + "=" + value);
							bw.newLine();

						}
						bw.close();
					} catch (Exception e) {
						if (bw != null)
							try {
								bw.close();
							} catch (IOException e1) {

							}
					}
				}

				BufferedWriter bw = null;
				try {
					bw = new BufferedWriter(new FileWriter(new File(base
							.getAbsolutePath()
							+ File.separator
							+ "MASTER_"
							+ (t == 0 ? "Epsilon" : "Sigma") + ".txt")));
					bw.write(this.panels[t].masterTF.getText());
					bw.close();
				} catch (IOException e) {
					if (bw != null)
						try {
							bw.close();
						} catch (IOException e1) {

						}
				}

			}

		}

		public void updateEnergies() {

			for (int t = 0; t < this.panels.length; t++) {

				JTable table = this.panels[t].getTable();

				for (int i = 0; i < this.panels[t].headers.length; i++) {

					Hashtable<String, Float> hash = new Hashtable<String, Float>();

					for (int j = 0; j < this.panels[t].headers.length; j++) {
						Object value = table.getValueAt(i, j);

						if (value != null)
							try {
								hash.put(this.panels[t].headers[j], Float
										.parseFloat(value.toString()));
							} catch (Exception e) {

							}

					}

					if (t == 0)
						Gui.this.algo.energies.hashEpsilon.put(
								this.panels[t].headers[i], hash);
					else
						Gui.this.algo.energies.hashSigma.put(
								this.panels[t].headers[i], hash);

				}

				try {
					if (t == 0)
						Gui.this.algo.energies.masterEpsilon = Float
								.parseFloat(this.panels[t].masterTF.getText()) / 1000f;
					else
						Gui.this.algo.energies.masterSigma = Float
								.parseFloat(this.panels[t].masterTF.getText());
				} catch (Exception e) {

				}

			}

		}

		public boolean anyChanges() {

			boolean change = false;

			for (int t = 0; t < this.panels.length; t++) {

				JTable table = this.panels[t].getTable();

				for (int i = 0; i < this.panels[t].headers.length; i++) {

					for (int j = 0; j < this.panels[t].headers.length; j++) {
						Object value = table.getValueAt(i, j);

						Float f = Gui.this.algo.energies.hashEpsilon.get(
								this.panels[t].headers[i]).get(
								this.panels[t].headers[j]);

						if ((f == null && value == null)) {

						} else if (f != null && value == null) {
							change = true;
						} else if (f == null && value != null) {
							change = true;
						} else if (!f.equals(value)) {
							change = true;
						}

					}

				}

				if (!this.panels[0].masterTF.getText().equals(
						"" + Gui.this.algo.energies.masterEpsilon))
					change = true;
				if (!this.panels[1].masterTF.getText().equals(
						"" + Gui.this.algo.energies.masterSigma))
					change = true;

			}

			return change;
		}

	}

	class ControlPanel extends JPanel {

		private final Vector<JRadioButton> EnergyBtns = new Vector<JRadioButton>();

		Atom_Level_Minimizer algo = Gui.this.algo;

		MyNumberTextField tfTemp = new MyNumberTextField(0.1d, 100d,
				MyNumberTextField.DOUBLE, new Double(25), false);

		MyNumberTextField tfInterval = new MyNumberTextField(1, 9999999,
				MyNumberTextField.INTEGER, new Double(5), false);

		MyNumberTextField tfCoolingRate = new MyNumberTextField(0.1d, 10d,
				MyNumberTextField.DOUBLE, new Double(0.20), false);

		public JTextField seedField = new JTextField();

		JCheckBox cbAllLayers = new JCheckBox("  Atoms of all Layers");
		JCheckBox cbProteins = new JCheckBox("  Atoms of Proteins");
		JCheckBox cbUnbonded = new JCheckBox("  Unbonded single Atoms");

		MyNumberTextField tfConstraints = new MyNumberTextField(0d, 1d,
				MyNumberTextField.DOUBLE, new Double(1), false);
		MyNumberTextField tfMovingRate = new MyNumberTextField(0.0001d, 1d,
				MyNumberTextField.DOUBLE, new Double(0.75d), false);

		JRadioButton rbMoveMolecules = new JRadioButton(
				"  Move only complete Molecules");
		JRadioButton rbMoveAtoms = new JRadioButton("  Move Atoms only");
		JRadioButton rbMoveBoth = new JRadioButton("  Move Molecules and Atoms");

		JSlider slAddNew = new JSlider();

		MyNumberTextField tfPressureY = new MyNumberTextField(-1, 1,
				MyNumberTextField.DOUBLE, new Double(0.1), false);
		MyNumberTextField tfPressureXZ = new MyNumberTextField(-1, 1,
				MyNumberTextField.DOUBLE, new Double(0), false);

		JCheckBox cbSolidBorders = new JCheckBox("  Enable Solid Borders");

		JRadioButton rbLJ = new JRadioButton(
				"  Lowest Free Energy by Lennard Jones Potential");

		MyNumberTextField tfCutoff = new MyNumberTextField(0.5, Math.min(algo
				.getMembraneSize().width, algo.getMembraneSize().height) / 2,
				MyNumberTextField.DOUBLE, new Double(8), false);
		JCheckBox cbAutoCutoff = new JCheckBox("  Auto Cutoff");
		JCheckBox cbContinuous = new JCheckBox(
				"  Periodic Borders (aka Simulation Box)");

		JButton defaultBtn = new JButton("Restore Defaults");

		// JRadioButton rbSchemeA=new JRadioButton("  Scheme A");
		// JRadioButton rbSchemeB=new JRadioButton("  Scheme B");

		// seedField
		//		
		// tfTemp
		//		
		// tfInterval
		//		
		// tfCoolingRate
		//		
		//
		// cbAllLayers
		// cbProteins
		// cbUnbonded
		//					
		// tfConstraints
		// tfMovingRate
		//					
		// tfPressureY
		// tfPressureXZ
		//					
		// cbSolidBorders
		//					
		// rbLJ
		// rbAtomDistances
		// tfCutoff
		// cbAutoCutoff
		// cbContinuous

		public ControlPanel() {

			seedField.setDocument(new PlainDocument() {
				public void insertString(final int offset, String str,
						final AttributeSet a) throws BadLocationException {

					if ((seedField.getText() + str).length() > 20)
						return;
					else
						super.insertString(offset, str, a);
				}
			});

			seedField.setFont(seedField.getFont().deriveFont(16f));

			seedField
					.setToolTipText("<html>Seed used by the Random of by this Algorithm. The characters you type in here<br>"
							+ "determine the exact sequence, leading to a same result everytime assuming that<br>"
							+ "all parameters, including the initial Membrane state, are the same.<br>"
							+ "A blank field will create a random seed.</html>");

			slAddNew.setValue(0);
			slAddNew.setMaximum(10);
			slAddNew.setMinimum(0);
			slAddNew.setSnapToTicks(true);
			slAddNew.setPaintTicks(true);
			slAddNew.setPaintLabels(true);
			slAddNew.setMajorTickSpacing(1);
			final Hashtable<Integer, JLabel> prec = new Hashtable<Integer, JLabel>();
			prec.put(new Integer(0), new JLabel("off"));
			prec.put(new Integer(1), new JLabel("1"));
			prec.put(new Integer(2), new JLabel("8"));
			prec.put(new Integer(5), new JLabel("125"));
			prec.put(new Integer(8), new JLabel("512"));
			prec.put(new Integer(10), new JLabel("1000"));
			slAddNew.setLabelTable(prec);

			MyPanel P1 = new MyPanel("Simulated Annealing", 0);
			P1.add(Box.createVerticalStrut(20));
			P1.addIt(new JLabel("  Current Temperature: "), tfTemp, new JLabel(
					"(0.1<=x<=100)"));
			P1.addIt(new JLabel("  Cooling Rate: "), tfCoolingRate, new JLabel(
					"(0.1<=x<=10)"));
			P1.addIt(new JLabel("  Interval Length: "), tfInterval, new JLabel(
					"(1<=x<=\u221e)"));
			P1.add(Box.createVerticalStrut(20));

			MyPanel P2 = new MyPanel("Random Atom Movement", 0);

			MyPanel p1 = new MyPanel("Internal Bonds", 1);
			p1.addIt(new JLabel("  Constraint Hardness: "), tfConstraints,
					new JLabel("(0.0<=x<=1.0)"));
			P2.add(Box.createVerticalStrut(20));
			P2.add(p1);
			P2.add(Box.createVerticalStrut(20));

			MyPanel p6 = new MyPanel("Random Placing of Atoms", 1);
			p6.addIt(new JLabel("  Max. Moving Factor: "), tfMovingRate,
					new JLabel("(0.001<=x<=1.0)"));
			P2.add(Box.createVerticalStrut(20));
			P2.add(p6);
			P2.add(Box.createVerticalStrut(20));

			p6.add(Box.createVerticalStrut(5));
			p6.addSingle(rbMoveBoth);
			p6.addSingle(rbMoveMolecules);
			p6.addSingle(rbMoveAtoms);

			MyPanel p2 = new MyPanel("Simulated Pressure", 1);
			p2.addIt(new JLabel("  From Sides: "), tfPressureXZ, new JLabel(
					"(-1.0<=x<=1.0)"));
			p2.addIt(new JLabel("  From Top/Bottom: "), tfPressureY,
					new JLabel("(-1.0<=x<=1.0)"));
			P2.add(p2);
			P2.add(Box.createVerticalStrut(20));

			MyPanel p3 = new MyPanel("Borders", 1);
			p3.addSingle(cbSolidBorders);
			P2.add(p3);
			P2.add(Box.createVerticalStrut(20));

			MyPanel P3 = new MyPanel("  Active Members", 0);
			MyPanel p4 = new MyPanel("  Involved Atoms", 1);
			p4.addSingle(cbAllLayers);
			p4.addSingle(cbProteins);
			p4.addSingle(cbUnbonded);
			P3.add(Box.createVerticalStrut(20));
			P3.add(p4);
			P3.add(Box.createVerticalStrut(20));

			ButtonGroup bg = new ButtonGroup();
			bg.add(rbLJ);

			MyPanel P4 = new MyPanel("  Energy Function", 0);
			MyPanel p5 = new MyPanel("  Value to Pursuit", 1);
			p5.addSingle(rbLJ);

			for (int i = 0; i < algo.energyFunctions.size(); i++) {
				EnergyFunction en = algo.energyFunctions.get(i);

				JRadioButton rb = new JRadioButton("" + en.getButtonName());
				rb.setToolTipText(en.getTooltip());
				bg.add(rb);
				this.EnergyBtns.addElement(rb);
				p5.addSingle(rb);
			}

			P4.add(Box.createVerticalStrut(20));
			P4.add(p5);
			P4.add(Box.createVerticalStrut(20));

			MyPanel p7 = new MyPanel("  Cutoff Settings", 1);
			p7.addIt(new JLabel("  Fixed Cutoff Distance: "), tfCutoff,
					new JLabel("(0.5<=x<="
							+ (Math.min(algo.getMembraneSize().width, algo
									.getMembraneSize().height) / 2) + ")"));
			p7.addSingle(cbAutoCutoff);
			p7.addSingle(cbContinuous);
			P4.add(Box.createVerticalStrut(20));
			P4.add(p7);
			P4.add(Box.createVerticalStrut(20));

			MyPanel P5 = new MyPanel("Random", 0);
			P5.addIt(new JLabel("  Random Seed: "), seedField, new JLabel(
					"(any String)"));

			MyPanel P6 = new MyPanel("Lipid Filling", 0);
			P6.addIt(new JLabel("  Number of Attempts: "), slAddNew,
					new JLabel());

			JPanel left = new JPanel();
			left.setLayout(new BoxLayout(left, BoxLayout.Y_AXIS));
			JPanel right = new JPanel();
			right.setLayout(new BoxLayout(right, BoxLayout.Y_AXIS));

			this.setLayout(new BoxLayout(this, BoxLayout.X_AXIS));

			left.add(P1);
			left.add(P2);
			right.add(P3);
			right.add(P4);
			right.add(P6);
			right.add(Box.createVerticalGlue());
			right.add(P5);
			JPanel p = new JPanel();
			p.setLayout(new BoxLayout(p, BoxLayout.X_AXIS));

			right.add(p);
			p.add(Box.createHorizontalGlue());
			p.add(Box.createVerticalStrut(35));
			p.add(defaultBtn);

			this.add(left);
			this.add(right);

			this.seedField.setMinimumSize(new Dimension(130, 25));
			this.seedField.setMaximumSize(new Dimension(130, 25));
			this.seedField.setPreferredSize(new Dimension(130, 25));

			ButtonGroup bg2 = new ButtonGroup();
			bg2.add(rbMoveBoth);
			bg2.add(rbMoveMolecules);
			bg2.add(rbMoveAtoms);

			tfTemp.setFixedSize(40, 25);

			tfInterval.setFixedSize(40, 25);

			tfCoolingRate.setFixedSize(40, 25);

			tfConstraints.setFixedSize(40, 25);
			tfMovingRate.setFixedSize(40, 25);

			tfPressureY.setFixedSize(40, 25);
			tfPressureXZ.setFixedSize(40, 25);

			tfCutoff.setFixedSize(40, 25);

			tfTemp
					.setToolTipText("<html>Current Temperature for the System. The Temperature effects the probability<br>"
							+ "of accepting states that are worse. The temperature decreases by the<br>"
							+ "cooling Rate during runtime</html>");

			tfInterval
					.setToolTipText("<html>The length of one interval. Defines the number of changes in the membrane state<br>"
							+ "before the temperature decreases</html>");

			tfCoolingRate
					.setToolTipText("<html>The lowering rate of the temperature at the end of one Interval</html>");

			cbAllLayers
					.setToolTipText("<html>If true, all Atoms in all Layers will be moved</html>");
			cbProteins
					.setToolTipText("<html>If true, Atoms of Proteins will also be moved</html>");
			cbUnbonded
					.setToolTipText("<html>If true, then all unbonded Atoms will be moved. Otherwise, they will<br>remain at their"
							+ "original Position.</html>");

			rbMoveBoth
					.setToolTipText("<html>Try to move the entire Molecules first, then move their Atoms</html>");
			rbMoveMolecules
					.setToolTipText("<html>Move entire Molecules only. Does not change their conformation</html>");
			cbProteins.setToolTipText("<html>Move Atoms only</html>");

			tfConstraints
					.setToolTipText("<html>Internal Covalent Bonds are simulated by constraints in order to keep a molecule<br>"
							+ "in one piece. The Value 1 will effect a total strength, while 0 will turn contraints off.</html>");
			tfMovingRate
					.setToolTipText("<html>The maximum distance an Atom can be randomly moved at one step (Angstrom)<br>"
							+ "at Temperature >10. Below this temperature, this value will be scaled down continuously.</html>");

			tfPressureY
					.setToolTipText("<html>Simulated Pressure on all Atoms to the Y-Plane</html>");
			tfPressureXZ
					.setToolTipText("<html>Simulated Pressure on all Atoms from all four sides</html>");

			cbSolidBorders
					.setToolTipText("<html>Prevents atoms to leave the specified Membrane Bounds</html>");

			rbLJ
					.setToolTipText("<html>Uses the Lennard Jones<sup>12-6</sup> Potential in order to evaluate the current state.<br>"
							+ "The proper Parameters must be set in the Parameter Table</html>");

			tfCutoff
					.setToolTipText("<html>A fixed cutoff radius for the Energy Function</html>");
			cbAutoCutoff
					.setToolTipText("<html>If true, a Cutoff specified for each differet pair of Elements will<br>"
							+ "be calculated and used</html>");
			cbContinuous
					.setToolTipText("<html>If true, atoms on the other side will also be takes into account by the Energy Function</html>");

			slAddNew
					.setToolTipText("<html>This Algorithm can try to add new Lipids at the beginnign of each step in order to close appearing gaps.<br>This slider modifies the number of attempts for this.</html>");

			cbAutoCutoff.addActionListener(new ActionListener() {

				public void actionPerformed(ActionEvent arg0) {
					if (cbAutoCutoff.isSelected()) {
						tfCutoff.setEnabled(false);
					} else {
						tfCutoff.setEnabled(true);
					}

				}

			});

			rbLJ.addActionListener(new ActionListener() {

				public void actionPerformed(ActionEvent arg0) {
					if (rbLJ.isSelected()) {

						cbAutoCutoff.setEnabled(true);
					} else {
						cbAutoCutoff.setEnabled(false);
						cbAutoCutoff.setSelected(false);
					}

				}

			});

			defaultBtn.addActionListener(new ActionListener() {

				public void actionPerformed(ActionEvent arg0) {
					fillWithDefaults();

				}

			});

			if (algo.getLayerCount() < 2) {
				cbAllLayers.setSelected(false);
				cbAllLayers.setEnabled(false);
			}
			if (algo.getAllProteins().size() < 1) {
				cbProteins.setSelected(false);
				cbProteins.setEnabled(false);
			}

		}

		public void storeSettings() {

			ConfigIO io = new ConfigIO();
			File base = Gui.this.algo.getAlgorithm_Directory_In_Workspace();

			base = new File(base.getAbsolutePath() + File.separator
					+ "config.txt");

			io.setConfigFile(base);

			io.addValue("seed", "" + "" + seedField.getText());

			io.addValue("temp", "" + Double.parseDouble(tfTemp.getText()));

			io.addValue("coolingInt", ""
					+ Integer.parseInt(tfInterval.getText()));

			io.addValue("coolingRate", ""
					+ Double.parseDouble(tfCoolingRate.getText()));

			io.addValue("movingRate", ""
					+ Double.parseDouble(tfMovingRate.getText()));

			io.addValue("cutoff", "" + Double.parseDouble(tfCutoff.getText()));

			io.addValue("constraints", ""
					+ Double.parseDouble(tfConstraints.getText()));

			io.addValue("pressureY", ""
					+ Double.parseDouble(tfPressureY.getText()));

			io.addValue("pressureXZ", ""
					+ Double.parseDouble(tfPressureXZ.getText()));

			io.addValue("allLayers", "" + cbAllLayers.isSelected());

			io.addValue("proteins", "" + cbProteins.isSelected());

			io.addValue("unbonded", "" + cbUnbonded.isSelected());

			io.addValue("solid", "" + cbSolidBorders.isSelected());

			io.addValue("autoCutoff", "" + cbAutoCutoff.isSelected());

			io.addValue("continuous", "" + cbContinuous.isSelected());

			io.addValue("addNew", "" + slAddNew.getValue());

			io.addValue("moveAtoms", "" + rbMoveAtoms.isSelected());

			io.addValue("moveMolecules", "" + rbMoveMolecules.isSelected());

			io.addValue("moveBoth", "" + rbMoveBoth.isSelected());

			try {
				io.writeDataToDisc();
			} catch (Exception e) {

			}

		}

		public void restoreSettings() {

			ConfigIO io = new ConfigIO();
			File base = Gui.this.algo.getAlgorithm_Directory_In_Workspace();

			base = new File(base.getAbsolutePath() + File.separator
					+ "config.txt");

			io.setConfigFile(base);

			try {
				io.readDataFromDisc();
			} catch (Exception e) {
				return;
			}

			if (io.isCorrectlyInitialized() == false)
				return;

			try {
				seedField.setText(io.getValue("seed")[0]);
			} catch (Exception e) {

			}

			// tfTemp.setT
			// io.addValue("temp",""+Float.parseFloat(tfTemp.getText()));

			try {
				tfInterval.setText(io.getValue("coolingInt")[0]);
			} catch (Exception e) {

			}

			try {
				tfCoolingRate.setText(io.getValue("coolingRate")[0]);
			} catch (Exception e8) {

			}

			try {
				tfMovingRate.setText(io.getValue("movingRate")[0]);
			} catch (Exception e7) {

			}

			try {
				tfCutoff.setText(io.getValue("cutoff")[0]);
			} catch (Exception e7) {

			}

			try {
				tfConstraints.setText(io.getValue("constraints")[0]);
			} catch (Exception e7) {

			}

			try {
				tfPressureY.setText(io.getValue("pressureY")[0]);
			} catch (Exception e7) {

			}

			try {
				tfPressureXZ.setText(io.getValue("pressureXZ")[0]);
			} catch (Exception e7) {

			}

			try {
				cbAllLayers.setSelected(Boolean.parseBoolean(io
						.getValue("allLayers")[0]));
			} catch (Exception e6) {

			}

			try {
				cbProteins.setSelected(Boolean.parseBoolean(io
						.getValue("proteins")[0]));
			} catch (Exception e5) {

			}

			try {
				cbUnbonded.setSelected(Boolean.parseBoolean(io
						.getValue("unbonded")[0]));
			} catch (Exception e4) {

			}

			try {
				cbSolidBorders.setSelected(Boolean.parseBoolean(io
						.getValue("solid")[0]));
			} catch (Exception e3) {

			}

			try {
				cbAutoCutoff.setSelected(Boolean.parseBoolean(io
						.getValue("autoCutoff")[0]));
			} catch (Exception e1) {

			}

			try {
				cbContinuous.setSelected(Boolean.parseBoolean(io
						.getValue("continuous")[0]));
			} catch (Exception e) {

			}
			try {
				slAddNew.setValue(Integer.parseInt(io.getValue("addNew")[0]));
			} catch (Exception e) {

			}

			try {
				rbMoveAtoms.setSelected(Boolean.parseBoolean(io
						.getValue("moveAtoms")[0]));
			} catch (Exception e) {

			}
			try {
				rbMoveBoth.setSelected(Boolean.parseBoolean(io
						.getValue("moveBoth")[0]));
			} catch (Exception e) {

			}
			try {
				rbMoveMolecules.setSelected(Boolean.parseBoolean(io
						.getValue("moveMolecules")[0]));
			} catch (Exception e) {

			}

		}

		public void updateFields() {

			algo.myData.settings.currentTemparature = Float.parseFloat(tfTemp
					.getText());
			algo.myData.settings.coolingIntervalLength = Integer
					.parseInt(tfInterval.getText());
			algo.myData.settings.coolingRate = Float.parseFloat(tfCoolingRate
					.getText());
			algo.myData.settings.currentMovingFactor = Float
					.parseFloat(tfMovingRate.getText());
			algo.myData.settings.cutoff = Float.parseFloat(tfCutoff.getText());

			algo.myData.settings.AllLayers = cbAllLayers.isSelected();
			algo.myData.settings.alsoProteins = cbProteins.isSelected();
			algo.myData.settings.alsoUnbonded = cbUnbonded.isSelected();
			algo.myData.settings.constraintFactor = Float
					.parseFloat(tfConstraints.getText());
			algo.myData.settings.pressureY = Float.parseFloat(tfPressureY
					.getText());
			algo.myData.settings.pressureXZ = Float.parseFloat(tfPressureXZ
					.getText());
			algo.myData.settings.solidBorders = cbSolidBorders.isSelected();

			algo.myData.settings.AutoCutoff = cbAutoCutoff.isSelected();
			algo.myData.settings.isBoxContinuous = cbContinuous.isSelected();
			algo.myData.settings.moleculeMovement = rbMoveMolecules
					.isSelected();
			algo.myData.settings.atomMovement = rbMoveAtoms.isSelected();
			algo.myData.settings.addNewLipidsFactor = slAddNew.getValue()
					* slAddNew.getValue() * slAddNew.getValue() * 100;

			if (rbMoveBoth.isSelected()) {
				algo.myData.settings.moleculeMovement = true;
				algo.myData.settings.atomMovement = true;
			}

			if (!rbLJ.isSelected()) {

				for (int i = 0; i < EnergyBtns.size(); i++) {

					if (EnergyBtns.get(i).isSelected()) {
						algo.energies.currentFunction = algo.energyFunctions
								.get(i);
					}
				}
			} else {
				algo.energies.currentFunction = null;
			}

			if (rbLJ.isSelected()
					|| (algo.energies.currentFunction != null && algo.energies.currentFunction
							.getGui() != null))
				Gui.this.parametersBtn.setEnabled(true);
			else
				Gui.this.parametersBtn.setEnabled(false);
		}

		public void fillWithDefaults() {

			tfTemp.setText("25.0");

			tfInterval.setText("5");

			tfCoolingRate.setText("0.20");

			cbAllLayers.setSelected(false);
			cbProteins.setSelected(false);
			cbUnbonded.setSelected(false);

			tfConstraints.setText("1");
			tfMovingRate.setText("0.75");

			tfPressureY.setText("0.1");
			tfPressureXZ.setText("0");

			cbSolidBorders.setSelected(false);

			rbLJ.setSelected(true);

			tfCutoff.setText("8");
			cbAutoCutoff.setSelected(true);
			cbContinuous.setSelected(true);

			rbMoveBoth.setSelected(true);
			slAddNew.setValue(0);

		}

		class MyPanel extends JPanel {

			public MyPanel(String text, int type) {
				super();
				this.setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));
				if (text != null && type == 0) {
					this.setBorder(BorderFactory.createTitledBorder(null, text,
							TitledBorder.DEFAULT_JUSTIFICATION,
							TitledBorder.DEFAULT_POSITION, this.getFont()
									.deriveFont(Font.BOLD, 14)));
				} else if (text != null && type == 1) {
					JPanel p = new JPanel();
					this.add(p);
					p.setLayout(new BoxLayout(p, BoxLayout.X_AXIS));
					JLabel l = new JLabel(" " + text);
					p.add(l);
					l.setFont(this.getFont().deriveFont(Font.PLAIN, 14));

					p.add(Box.createVerticalStrut(10));
				}

			}

			public void addIt(JComponent a, JComponent b, JComponent c) {
				JPanel p = new JPanel();
				this.add(p);
				p.setLayout(new BoxLayout(p, BoxLayout.X_AXIS));
				p.add(a);
				a.setPreferredSize(new Dimension(200, 30));
				p.add(Box.createHorizontalGlue());
				p.add(b);
				if (c != null) {
					p.add(Box.createHorizontalStrut(5));
					p.add(c);
					((JLabel) c)
							.setFont(this.getFont().deriveFont(Font.ITALIC));
					c.setPreferredSize(new Dimension(100, 30));
				} else {

				}

			}

			public void addSingle(JComponent c) {
				JPanel p = new JPanel();
				this.add(p);
				p.setLayout(new BoxLayout(p, BoxLayout.X_AXIS));
				p.add(c);
				c.setPreferredSize(new Dimension(280, 30));
				p.add(Box.createHorizontalGlue());
			}
		}

		public boolean anyRelevantChangesToFF() {

			return (algo.myData.settings.cutoff != Float.parseFloat(tfCutoff
					.getText()))
					|| (algo.myData.settings.constraintFactor != Float
							.parseFloat(tfConstraints.getText()))
					||

					(algo.myData.settings.AutoCutoff != cbAutoCutoff
							.isSelected())
					|| (algo.myData.settings.isBoxContinuous != cbContinuous
							.isSelected());

		}

	}

	public boolean anyChanges() {

		return this.controlPanel.anyRelevantChangesToFF()
				|| this.paraPanel.anyChanges();

	}
}
