/*******************************************************************************
 * CELLmicrocosmos 2.2 MembraneEditor (version 2.2.2_2)
 *
 * Copyright (c) 2005, 2017 The CELLmicrocosmos 2 Development Team
 * (Bjorn Sommer, Tim Dingersen et al.
 *  University of Konstanz, Monash University, Bielefeld University)
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *  
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *  
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *******************************************************************************/
package org.cellmicrocosmos.cm2.membranepackingalgorithms.atomlevelminimizer;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.event.FocusEvent;
import java.awt.event.FocusListener;

import javax.swing.JTextField;
import javax.swing.event.CaretEvent;
import javax.swing.event.CaretListener;

/**
 * This is an extended textfield, used as the number input fields in the options dialog
 * 
 * @author Tim Dingersen
 * dackelnose@gmx.net
 *
 */
public class MyNumberTextField extends JTextField implements CaretListener,
		FocusListener {

	public static final int INTEGER = 0;
	public static final int DOUBLE = 1;

	private static final long serialVersionUID = 1L;

	float lastGood = 0;

	private double min;

	private double max;
	private int mode;
	Double defaultValue;
	private boolean mayBeEmpty;
	public boolean isValid = true;

	public MyNumberTextField(final double min, final double max, int mode,
			Double defaultValue, boolean mayBeEmpty) {
		this.min = min;
		this.max = max;
		this.mode = mode;
		this.defaultValue = defaultValue;
		this.mayBeEmpty = mayBeEmpty;
		this.isValid = mayBeEmpty;
		this.addCaretListener(this);
		this.addFocusListener(this);
		this.getText();
	}

	public void caretUpdate(CaretEvent arg0) {

		try {
			String text = super.getText().trim();

			if (text.length() < 1) {
				if (this.mayBeEmpty) {
					this.isValid = true;
				} else
					this.isValid = false;
			} else {

				try {

					if (this.mode == INTEGER)
						new Integer(text);
					else
						new Double(text);

					double d = new Double(text).doubleValue();

					if (d < min || d > max) {
						this.isValid = false;
					} else
						this.isValid = true;
				} catch (Exception e) {
					this.isValid = false;
				}

			}

			if (this.isValid) {
				this.setForeground(null);
			} else
				this.setForeground(Color.red);
		} catch (Throwable e) {

		}

	}

	public String getText() {

		String text = super.getText().trim();

		if (text.length() < 1) {
			if (this.mayBeEmpty) {
				return "";
			}
			// else {
			// if(this.defaultValue!=null) {
			// this.setText(""+defaultValue.doubleValue())
			// }
			// }
		}

		if (this.isValid)
			return text;
		else {
			if (this.defaultValue != null) {

				if (this.mode == INTEGER) {
					this.setText("" + defaultValue.intValue());
					return "" + defaultValue.intValue();
				} else {
					this.setText("" + defaultValue.doubleValue());
					return "" + defaultValue.doubleValue();
				}
			} else {
				return "";
			}
		}

	}

	public void focusGained(FocusEvent arg0) {

	}

	public void focusLost(FocusEvent arg0) {
		this.getText();

	}

	

	public void setFixedSize(int x, int y) {

		this.setMaximumSize(new Dimension(x, y));
		this.setMinimumSize(new Dimension(x, y));
		this.setPreferredSize(new Dimension(x, y));
	}

}
