/*******************************************************************************
 * CELLmicrocosmos 2.2 MembraneEditor (version 2.2.2_2)
 *
 * Copyright (c) 2005, 2017 The CELLmicrocosmos 2 Development Team
 * (Bjorn Sommer, Tim Dingersen et al.
 *  University of Konstanz, Monash University, Bielefeld University)
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *  
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *  
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *******************************************************************************/
package org.cellmicrocosmos.cm2.pluginmanager;

import java.io.File;

/**
 * 
 * This class contains the MembranePackingAlgorithm compiled
 * from a .jar or .java file. In addition it stores the 
 * compatibility information of the algorithm to be shown
 * in the PluginManager.
 * 
 * @author Tim Dingersen dackelnose@gmx.net
 * 
 */
public class Plugin {

	public static final String JAVA_FILE = ".java";
	public static final String JAR_FILE = ".jar";
	public static final String INCLUDED = "Included";

	public static final String STATE_READY = "Ready";
	public static final String STATE_FILE_NOT_FOUND = "File not found";
	public static final String STATE_FILE_NOT_READABLE = "File not read";
	public static final String STATE_COMPILE_ERROR = "Compile Error";
	public static final String STATE_NO_ALGORITHMS = "Not found";

	private static final String FNF = "The file for this algorithm cannot be found\n"
			+ "at its former location!\n " + "It has to be restored manually.";
	private static final String CE = "This algorithm cannot be compiled!\n"
			+ "It propably contains errors.";

	private static final String NA = "This jar-file does not contain a Membrane\n"
			+ "Algorithm type!";
	private static final String FNRA = "The access to the algorithm file has been\n"
			+ "denied!";

	private String name;
	private String information;
	private File originalFile;
	public String type = null;
	public String state = null;
	private Boolean supportsMDs = null;
	private Boolean supportsAbsoluteValues = null;
	private Boolean overridesLipids = null;
	private boolean extendedPlugin = false;
	private long lastModified = 0;
	private String javapath = null;

	public Plugin() {

	}

	public String getName() {
		return this.name;
	}

	public void setName(final String name) {
		this.name = name;
	}

	public String getInformation() {

		if (this.state.equals(Plugin.STATE_FILE_NOT_FOUND))
			return Plugin.FNF;
		else if (this.state.equals(Plugin.STATE_FILE_NOT_READABLE))
			return Plugin.FNRA;
		else if (this.state.equals(Plugin.STATE_COMPILE_ERROR))
			return Plugin.CE;
		else if (this.state.equals(Plugin.STATE_NO_ALGORITHMS))
			return Plugin.NA;

		return this.information;

	}

	public void setInformation(final String information) {
		this.information = information;
	}

	public String getType() {
		return this.type;
	}

	public void setType(final String type) {
		this.type = type;
	}

	public String getState() {
		return this.state;
	}

	public void setState(final String state) {
		this.state = state;
	}

	public Boolean supportsMDs() {
		if (this.state.equals(Plugin.STATE_READY) == false)
			return null;
		return this.supportsMDs;
	}

	public void setSupportsMDs(final Boolean supportsMDs) {
		this.supportsMDs = supportsMDs;
	}
	
	public Boolean supportsAbsoluteValues() {
		if (this.state.equals(Plugin.STATE_READY) == false)
			return null;
		return this.supportsAbsoluteValues;
	}

	public void setSupportsAbsoluteValues(final Boolean supportsAbsoluteValues) {
		this.supportsAbsoluteValues = supportsAbsoluteValues;
	}

	public Boolean overridesLipids() {
		if (this.state.equals(Plugin.STATE_READY) == false)
			return null;
		return this.overridesLipids;
	}

	public void setOverridesLipids(final Boolean overridesLipids) {
		this.overridesLipids = overridesLipids;
	}

	public File getOriginalFile() {
		return this.originalFile;
	}

	public void setOriginalFile(final File originalFile) {
		this.originalFile = originalFile;
	}

	public long getLastModified() {
		return this.lastModified;
	}

	public void setLastModified(final long lastModified) {
		this.lastModified = lastModified;
	}

	public String getJavapath() {
		return this.javapath;
	}

	public void setJavapath(final String javapath) {
		this.javapath = javapath;
	}

	public boolean isExtendedPlugin() {
		return this.extendedPlugin;
	}

	public void setExtendedPlugin(boolean value) {
		this.extendedPlugin = value;
	}

}
